/* Generated by Cython 0.28.4 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_28_4"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__plyvel___plyvel
#define __PYX_HAVE_API__plyvel___plyvel
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#include <stdlib.h>
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <string>
#include "leveldb/db.h"
#include "leveldb/status.h"
#include "leveldb/options.h"
#include "leveldb/slice.h"
#include "leveldb/write_batch.h"
#include "leveldb/iterator.h"
#include "leveldb/comparator.h"
#include "leveldb/filter_policy.h"
#include "leveldb/cache.h"
#include "comparator.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "plyvel/_plyvel.pyx",
  "stringsource",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/*--- Type declarations ---*/
struct __pyx_obj_6plyvel_7_plyvel_DB;
struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB;
struct __pyx_obj_6plyvel_7_plyvel_WriteBatch;
struct __pyx_obj_6plyvel_7_plyvel_BaseIterator;
struct __pyx_obj_6plyvel_7_plyvel_Iterator;
struct __pyx_obj_6plyvel_7_plyvel_RawIterator;
struct __pyx_obj_6plyvel_7_plyvel_Snapshot;
struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__;

/* "plyvel/_plyvel.pyx":642
 * #
 * 
 * cdef enum IteratorState:             # <<<<<<<<<<<<<<
 *     BEFORE_START
 *     AFTER_STOP
 */
enum __pyx_t_6plyvel_7_plyvel_IteratorState {
  __pyx_e_6plyvel_7_plyvel_BEFORE_START,
  __pyx_e_6plyvel_7_plyvel_AFTER_STOP,
  __pyx_e_6plyvel_7_plyvel_IN_BETWEEN,
  __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED
};

/* "plyvel/_plyvel.pyx":649
 * 
 * 
 * cdef enum IteratorDirection:             # <<<<<<<<<<<<<<
 *     FORWARD
 *     REVERSE
 */
enum __pyx_t_6plyvel_7_plyvel_IteratorDirection {
  __pyx_e_6plyvel_7_plyvel_FORWARD,
  __pyx_e_6plyvel_7_plyvel_REVERSE
};

/* "plyvel/_plyvel.pyx":221
 * 
 * @cython.final
 * cdef class DB:             # <<<<<<<<<<<<<<
 *     cdef leveldb.DB* _db
 *     cdef Options options
 */
struct __pyx_obj_6plyvel_7_plyvel_DB {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_DB *__pyx_vtab;
  leveldb::DB *_db;
  leveldb::Options options;
  PyObject *name;
  PyObject *lock;
  PyObject *iterators;
};


/* "plyvel/_plyvel.pyx":453
 * 
 * 
 * cdef class PrefixedDB:             # <<<<<<<<<<<<<<
 *     cdef readonly DB db
 *     cdef readonly bytes prefix
 */
struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB {
  PyObject_HEAD
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
};


/* "plyvel/_plyvel.pyx":553
 * 
 * @cython.final
 * cdef class WriteBatch:             # <<<<<<<<<<<<<<
 *     cdef leveldb.WriteBatch* _write_batch
 *     cdef WriteOptions write_options
 */
struct __pyx_obj_6plyvel_7_plyvel_WriteBatch {
  PyObject_HEAD
  leveldb::WriteBatch *_write_batch;
  leveldb::WriteOptions write_options;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
  bool transaction;
};


/* "plyvel/_plyvel.pyx":654
 * 
 * 
 * cdef class BaseIterator:             # <<<<<<<<<<<<<<
 *     cdef DB db
 *     cdef leveldb.Iterator* _iter
 */
struct __pyx_obj_6plyvel_7_plyvel_BaseIterator {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *__pyx_vtab;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  leveldb::Iterator *_iter;
  PyObject *__weakref__;
};


/* "plyvel/_plyvel.pyx":703
 * 
 * @cython.final
 * cdef class Iterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef IteratorDirection direction
 *     cdef IteratorState state
 */
struct __pyx_obj_6plyvel_7_plyvel_Iterator {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator __pyx_base;
  enum __pyx_t_6plyvel_7_plyvel_IteratorDirection direction;
  enum __pyx_t_6plyvel_7_plyvel_IteratorState state;
  leveldb::Comparator *comparator;
  PyObject *start;
  PyObject *stop;
  leveldb::Slice start_slice;
  leveldb::Slice stop_slice;
  bool include_start;
  bool include_stop;
  bool include_key;
  bool include_value;
  PyObject *db_prefix;
  size_t db_prefix_len;
};


/* "plyvel/_plyvel.pyx":1010
 * 
 * @cython.final
 * cdef class RawIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     def valid(self):
 *         if self._iter is NULL:
 */
struct __pyx_obj_6plyvel_7_plyvel_RawIterator {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator __pyx_base;
};


/* "plyvel/_plyvel.pyx":1100
 * 
 * @cython.final
 * cdef class Snapshot:             # <<<<<<<<<<<<<<
 *     cdef leveldb.Snapshot* _snapshot
 *     cdef DB db
 */
struct __pyx_obj_6plyvel_7_plyvel_Snapshot {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot *__pyx_vtab;
  leveldb::Snapshot *_snapshot;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
};


/* "plyvel/_plyvel.pyx":662
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 */
struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ {
  PyObject_HEAD
  PyObject *__pyx_v_iterator_id;
  PyObject *__pyx_v_ref_dict;
};



/* "plyvel/_plyvel.pyx":221
 * 
 * @cython.final
 * cdef class DB:             # <<<<<<<<<<<<<<
 *     cdef leveldb.DB* _db
 *     cdef Options options
 */

struct __pyx_vtabstruct_6plyvel_7_plyvel_DB {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_DB *__pyx_vtabptr_6plyvel_7_plyvel_DB;
static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch);


/* "plyvel/_plyvel.pyx":654
 * 
 * 
 * cdef class BaseIterator:             # <<<<<<<<<<<<<<
 *     cdef DB db
 *     cdef leveldb.Iterator* _iter
 */

struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;


/* "plyvel/_plyvel.pyx":703
 * 
 * @cython.final
 * cdef class Iterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef IteratorDirection direction
 *     cdef IteratorState state
 */

struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator {
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_base;
  PyObject *(*current)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
  PyObject *(*real_next)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
  PyObject *(*real_prev)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator *__pyx_vtabptr_6plyvel_7_plyvel_Iterator;
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);


/* "plyvel/_plyvel.pyx":1010
 * 
 * @cython.final
 * cdef class RawIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     def valid(self):
 *         if self._iter is NULL:
 */

struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator {
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_base;
  PyObject *(*key)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
  PyObject *(*value)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator *__pyx_vtabptr_6plyvel_7_plyvel_RawIterator;
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);


/* "plyvel/_plyvel.pyx":1100
 * 
 * @cython.final
 * cdef class Snapshot:             # <<<<<<<<<<<<<<
 *     cdef leveldb.Snapshot* _snapshot
 *     cdef DB db
 */

struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot *__pyx_vtabptr_6plyvel_7_plyvel_Snapshot;
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch);

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetItemIntByteArray.proto */
#define __Pyx_GetItemInt_ByteArray(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_ByteArray_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "bytearray index out of range"), -1))
static CYTHON_INLINE int __Pyx_GetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i,
                                                         int wraparound, int boundscheck);

/* SetItemIntByteArray.proto */
#define __Pyx_SetItemInt_ByteArray(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_ByteArray_Fast(o, (Py_ssize_t)i, v, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "bytearray index out of range"), -1))
static CYTHON_INLINE int __Pyx_SetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i, unsigned char v,
                                                         int wraparound, int boundscheck);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   ((obj)->ob_type->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return __Pyx_PyObject_GetAttrStr(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#else
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* None.proto */
#include <new>

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_12BaseIterator_close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libcpp.string' */

/* Module declarations from 'libcpp' */

/* Module declarations from 'plyvel.leveldb' */

/* Module declarations from 'plyvel.comparator' */

/* Module declarations from 'plyvel._plyvel' */
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_DB = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_PrefixedDB = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_WriteBatch = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_BaseIterator = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_Iterator = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_RawIterator = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_Snapshot = 0;
static PyTypeObject *__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__ = 0;
static int __pyx_f_6plyvel_7_plyvel_raise_for_status(leveldb::Status); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_6plyvel_7_plyvel_db_get(struct __pyx_obj_6plyvel_7_plyvel_DB *, PyObject *, PyObject *, leveldb::ReadOptions); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_to_file_system_name(PyObject *); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_bytes_increment(PyObject *); /*proto*/
static int __pyx_f_6plyvel_7_plyvel_parse_options(leveldb::Options *, bool, bool, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyObject_string_to_py_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyUnicode_string_to_py_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyStr_string_to_py_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyBytes_string_to_py_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyByteArray_string_to_py_std__in_string(std::string const &); /*proto*/
static std::string __pyx_convert_string_from_py_std__in_string(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "plyvel._plyvel"
extern int __pyx_module_is_main_plyvel___plyvel;
int __pyx_module_is_main_plyvel___plyvel = 0;

/* Implementation of 'plyvel._plyvel' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_UnicodeEncodeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_hex;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_StopIteration;
static const char __pyx_k__8[] = "";
static const char __pyx_k_db[] = "db";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_st[] = "st";
static const char __pyx_k_d_d[] = "%d.%d";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_hex[] = "hex";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_put[] = "put";
static const char __pyx_k_ref[] = "ref";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_Lock[] = "Lock";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_sync[] = "sync";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Error[] = "Error";
static const char __pyx_k_UTF_8[] = "UTF-8";
static const char __pyx_k_ascii[] = "ascii";
static const char __pyx_k_clear[] = "clear";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_closed[] = "closed";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_delete[] = "delete";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_exc_tb[] = "exc_tb";
static const char __pyx_k_fsname[] = "fsname";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_prefix[] = "prefix";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_snappy[] = "snappy";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_default[] = "default";
static const char __pyx_k_exc_val[] = "exc_val";
static const char __pyx_k_options[] = "options";
static const char __pyx_k_popitem[] = "popitem";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_weakref[] = "weakref";
static const char __pyx_k_closed_2[] = " (closed)";
static const char __pyx_k_exc_type[] = "exc_type";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_iterator[] = "iterator";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_snapshot[] = "snapshot";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_db_prefix[] = "db_prefix";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_repair_db[] = "repair_db";
static const char __pyx_k_threading[] = "threading";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_block_size[] = "block_size";
static const char __pyx_k_comparator[] = "comparator";
static const char __pyx_k_destroy_db[] = "destroy_db";
static const char __pyx_k_fill_cache[] = "fill_cache";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_compression[] = "compression";
static const char __pyx_k_include_key[] = "include_key";
static const char __pyx_k_transaction[] = "transaction";
static const char __pyx_k_weakref_ref[] = "weakref_ref";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_include_stop[] = "include_stop";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_seek_to_stop[] = "seek_to_stop";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_StopIteration[] = "StopIteration";
static const char __pyx_k_include_start[] = "include_start";
static const char __pyx_k_include_value[] = "include_value";
static const char __pyx_k_max_file_size[] = "max_file_size";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_seek_to_start[] = "seek_to_start";
static const char __pyx_k_lru_cache_size[] = "lru_cache_size";
static const char __pyx_k_max_open_files[] = "max_open_files";
static const char __pyx_k_plyvel__plyvel[] = "plyvel._plyvel";
static const char __pyx_k_CorruptionError[] = "CorruptionError";
static const char __pyx_k_comparator_name[] = "comparator_name";
static const char __pyx_k_error_if_exists[] = "error_if_exists";
static const char __pyx_k_leveldb_version[] = "__leveldb_version__";
static const char __pyx_k_paranoid_checks[] = "paranoid_checks";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_verify_checksums[] = "verify_checksums";
static const char __pyx_k_approximate_sizes[] = "approximate_sizes";
static const char __pyx_k_bloom_filter_bits[] = "bloom_filter_bits";
static const char __pyx_k_create_if_missing[] = "create_if_missing";
static const char __pyx_k_write_buffer_size[] = "write_buffer_size";
static const char __pyx_k_Database_is_closed[] = "Database is closed";
static const char __pyx_k_UnicodeEncodeError[] = "UnicodeEncodeError";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_plyvel__plyvel_pyx[] = "plyvel/_plyvel.pyx";
static const char __pyx_k_IteratorInvalidError[] = "IteratorInvalidError";
static const char __pyx_k_init___locals_lambda[] = "__init__.<locals>.<lambda>";
static const char __pyx_k_getfilesystemencoding[] = "getfilesystemencoding";
static const char __pyx_k_block_restart_interval[] = "block_restart_interval";
static const char __pyx_k_pyx_unpickle_PrefixedDB[] = "__pyx_unpickle_PrefixedDB";
static const char __pyx_k_Database_or_iterator_is_closed[] = "Database or iterator is closed";
static const char __pyx_k_Database_or_snapshot_is_closed[] = "Database or snapshot is closed";
static const char __pyx_k_comparator_and_comparator_name[] = "'comparator' and 'comparator_name' must be specified together";
static const char __pyx_k_name_arg_must_be_a_byte_string[] = "'name' arg must be a byte string or a unicode string";
static const char __pyx_k_prefix_cannot_be_used_together[] = "'prefix' cannot be used together with 'start' or 'stop'";
static const char __pyx_k_Plyvel_a_Python_LevelDB_interfa[] = "\nPlyvel, a Python LevelDB interface.\n\nUse plyvel.DB() to create or open a database.\n";
static const char __pyx_k_Start_and_stop_of_range_must_be[] = "Start and stop of range must be byte strings";
static const char __pyx_k_compression_must_be_None_or_a_s[] = "'compression' must be None or a string";
static const char __pyx_k_compression_must_be_None_or_sna[] = "'compression' must be None or 'snappy'";
static const char __pyx_k_plyvel_DB_with_name_r_s_at_0x_s[] = "<plyvel.DB with name %r%s at 0x%s>";
static const char __pyx_k_plyvel_PrefixedDB_with_prefix_r[] = "<plyvel.PrefixedDB with prefix %r at 0x%s>";
static const char __pyx_k_self__db_self_options_cannot_be[] = "self._db,self.options cannot be converted to a Python object for pickling";
static const char __pyx_k_self__iter_self_comparator_self[] = "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling";
static const char __pyx_k_Cannot_convert_unicode_name_to_a[] = "Cannot convert unicode 'name' to a file system name: %s";
static const char __pyx_k_Cannot_operate_on_closed_LevelDB[] = "Cannot operate on closed LevelDB database";
static const char __pyx_k_Incompatible_checksums_s_vs_0x11[] = "Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))";
static const char __pyx_k_custom_comparator_object_must_be[] = "custom comparator object must be callable";
static const char __pyx_k_self__iter_cannot_be_converted_t[] = "self._iter cannot be converted to a Python object for pickling";
static const char __pyx_k_self__snapshot_cannot_be_convert[] = "self._snapshot cannot be converted to a Python object for pickling";
static const char __pyx_k_self__write_batch_self_write_opt[] = "self._write_batch,self.write_options cannot be converted to a Python object for pickling";
static PyObject *__pyx_kp_s_Cannot_convert_unicode_name_to_a;
static PyObject *__pyx_kp_s_Cannot_operate_on_closed_LevelDB;
static PyObject *__pyx_n_s_CorruptionError;
static PyObject *__pyx_kp_s_Database_is_closed;
static PyObject *__pyx_kp_s_Database_or_iterator_is_closed;
static PyObject *__pyx_kp_s_Database_or_snapshot_is_closed;
static PyObject *__pyx_n_s_Error;
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x11;
static PyObject *__pyx_n_s_IteratorInvalidError;
static PyObject *__pyx_n_s_Lock;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_kp_s_Start_and_stop_of_range_must_be;
static PyObject *__pyx_n_s_StopIteration;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_UTF_8;
static PyObject *__pyx_n_s_UnicodeEncodeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__8;
static PyObject *__pyx_n_s_approximate_sizes;
static PyObject *__pyx_n_s_ascii;
static PyObject *__pyx_n_s_block_restart_interval;
static PyObject *__pyx_n_s_block_size;
static PyObject *__pyx_n_s_bloom_filter_bits;
static PyObject *__pyx_n_s_clear;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_closed;
static PyObject *__pyx_kp_s_closed_2;
static PyObject *__pyx_n_s_comparator;
static PyObject *__pyx_kp_s_comparator_and_comparator_name;
static PyObject *__pyx_n_s_comparator_name;
static PyObject *__pyx_n_s_compression;
static PyObject *__pyx_kp_s_compression_must_be_None_or_a_s;
static PyObject *__pyx_kp_s_compression_must_be_None_or_sna;
static PyObject *__pyx_n_s_create_if_missing;
static PyObject *__pyx_kp_s_custom_comparator_object_must_be;
static PyObject *__pyx_kp_s_d_d;
static PyObject *__pyx_n_s_db;
static PyObject *__pyx_n_s_db_prefix;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_default;
static PyObject *__pyx_n_s_delete;
static PyObject *__pyx_n_s_destroy_db;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enter;
static PyObject *__pyx_n_s_error_if_exists;
static PyObject *__pyx_n_s_exc_tb;
static PyObject *__pyx_n_s_exc_type;
static PyObject *__pyx_n_s_exc_val;
static PyObject *__pyx_n_s_exit;
static PyObject *__pyx_n_s_fill_cache;
static PyObject *__pyx_n_s_fsname;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getfilesystemencoding;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_hex;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_include_key;
static PyObject *__pyx_n_s_include_start;
static PyObject *__pyx_n_s_include_stop;
static PyObject *__pyx_n_s_include_value;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_init___locals_lambda;
static PyObject *__pyx_n_s_iterator;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_leveldb_version;
static PyObject *__pyx_n_s_lru_cache_size;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_file_size;
static PyObject *__pyx_n_s_max_open_files;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_kp_s_name_arg_must_be_a_byte_string;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_options;
static PyObject *__pyx_n_s_paranoid_checks;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_kp_s_plyvel_DB_with_name_r_s_at_0x_s;
static PyObject *__pyx_kp_s_plyvel_PrefixedDB_with_prefix_r;
static PyObject *__pyx_n_s_plyvel__plyvel;
static PyObject *__pyx_kp_s_plyvel__plyvel_pyx;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_popitem;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_kp_s_prefix_cannot_be_used_together;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_put;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_PrefixedDB;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_ref;
static PyObject *__pyx_n_s_repair_db;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_n_s_seek_to_start;
static PyObject *__pyx_n_s_seek_to_stop;
static PyObject *__pyx_kp_s_self__db_self_options_cannot_be;
static PyObject *__pyx_kp_s_self__iter_cannot_be_converted_t;
static PyObject *__pyx_kp_s_self__iter_self_comparator_self;
static PyObject *__pyx_kp_s_self__snapshot_cannot_be_convert;
static PyObject *__pyx_kp_s_self__write_batch_self_write_opt;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_snappy;
static PyObject *__pyx_n_u_snappy;
static PyObject *__pyx_n_s_snapshot;
static PyObject *__pyx_n_s_st;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_sync;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_threading;
static PyObject *__pyx_n_s_transaction;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_verify_checksums;
static PyObject *__pyx_n_s_weakref;
static PyObject *__pyx_n_s_weakref_ref;
static PyObject *__pyx_n_s_write;
static PyObject *__pyx_n_s_write_buffer_size;
static PyObject *__pyx_n_s_xrange;
static int __pyx_pf_6plyvel_7_plyvel_2DB___init__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name, PyBoolObject *__pyx_v_create_if_missing, PyBoolObject *__pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_2close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_8get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_10put(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_12delete(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyBoolObject *__pyx_v_transaction, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_18iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_24get_property(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_ranges); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_34__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_36__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_transaction, PyBoolObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_repair_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2destroy_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix, PyBoolObject *__pyx_v_transaction, PyObject *__pyx_v_sync); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_12__enter__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_14__exit__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_wr); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_db_prefix, PyBoolObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyBoolObject *__pyx_v_include_start, PyBoolObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyBoolObject *__pyx_v_include_key, PyBoolObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_DB(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_PrefixedDB(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_WriteBatch(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_BaseIterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Iterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_RawIterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Snapshot(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static PyObject *__pyx_int_18786289;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_codeobj__57;
static PyObject *__pyx_codeobj__59;
static PyObject *__pyx_codeobj__61;
/* Late includes */

/* "plyvel/_plyvel.pyx":83
 * 
 * 
 * cdef int raise_for_status(Status st) except -1:             # <<<<<<<<<<<<<<
 *     if st.ok():
 *         return 0
 */

static int __pyx_f_6plyvel_7_plyvel_raise_for_status(leveldb::Status __pyx_v_st) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("raise_for_status", 0);

  /* "plyvel/_plyvel.pyx":84
 * 
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_1 = (__pyx_v_st.ok() != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":85
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     if st.IsIOError():
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":84
 * 
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":87
 *         return 0
 * 
 *     if st.IsIOError():             # <<<<<<<<<<<<<<
 *         raise IOError(st.ToString())
 * 
 */
  __pyx_t_1 = (__pyx_v_st.IsIOError() != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":88
 * 
 *     if st.IsIOError():
 *         raise IOError(st.ToString())             # <<<<<<<<<<<<<<
 * 
 *     if st.IsCorruption():
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IOError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_convert_PyBytes_string_to_py_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 88, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":87
 *         return 0
 * 
 *     if st.IsIOError():             # <<<<<<<<<<<<<<
 *         raise IOError(st.ToString())
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":90
 *         raise IOError(st.ToString())
 * 
 *     if st.IsCorruption():             # <<<<<<<<<<<<<<
 *         raise CorruptionError(st.ToString())
 * 
 */
  __pyx_t_1 = (__pyx_v_st.IsCorruption() != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":91
 * 
 *     if st.IsCorruption():
 *         raise CorruptionError(st.ToString())             # <<<<<<<<<<<<<<
 * 
 *     # Generic fallback
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_CorruptionError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __pyx_convert_PyBytes_string_to_py_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_6};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_6};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 91, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":90
 *         raise IOError(st.ToString())
 * 
 *     if st.IsCorruption():             # <<<<<<<<<<<<<<
 *         raise CorruptionError(st.ToString())
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":94
 * 
 *     # Generic fallback
 *     raise Error(st.ToString())             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_Error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __pyx_convert_PyBytes_string_to_py_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 94, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":83
 * 
 * 
 * cdef int raise_for_status(Status st) except -1:             # <<<<<<<<<<<<<<
 *     if st.ok():
 *         return 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.raise_for_status", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":101
 * #
 * 
 * cdef inline db_get(DB db, bytes key, object default, ReadOptions read_options):             # <<<<<<<<<<<<<<
 *     cdef string value
 *     cdef Status st
 */

static CYTHON_INLINE PyObject *__pyx_f_6plyvel_7_plyvel_db_get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_key, PyObject *__pyx_v_default, leveldb::ReadOptions __pyx_v_read_options) {
  std::string __pyx_v_value;
  leveldb::Status __pyx_v_st;
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  const char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("db_get", 0);

  /* "plyvel/_plyvel.pyx":104
 *     cdef string value
 *     cdef Status st
 *     cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 104, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 104, __pyx_L1_error)
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 104, __pyx_L1_error)
  }
  __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_1, __pyx_t_2);

  /* "plyvel/_plyvel.pyx":106
 *     cdef Slice key_slice = Slice(key, len(key))
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":107
 * 
 *     with nogil:
 *         st = db._db.Get(read_options, key_slice, &value)             # <<<<<<<<<<<<<<
 * 
 *     if st.IsNotFound():
 */
        __pyx_v_st = __pyx_v_db->_db->Get(__pyx_v_read_options, __pyx_v_key_slice, (&__pyx_v_value));
      }

      /* "plyvel/_plyvel.pyx":106
 *     cdef Slice key_slice = Slice(key, len(key))
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":109
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 *     if st.IsNotFound():             # <<<<<<<<<<<<<<
 *         return default
 *     raise_for_status(st)
 */
  __pyx_t_3 = (__pyx_v_st.IsNotFound() != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":110
 * 
 *     if st.IsNotFound():
 *         return default             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_default);
    __pyx_r = __pyx_v_default;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":109
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 *     if st.IsNotFound():             # <<<<<<<<<<<<<<
 *         return default
 *     raise_for_status(st)
 */
  }

  /* "plyvel/_plyvel.pyx":111
 *     if st.IsNotFound():
 *         return default
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     return value
 */
  __pyx_t_4 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 111, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":113
 *     raise_for_status(st)
 * 
 *     return value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_convert_PyBytes_string_to_py_std__in_string(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":101
 * #
 * 
 * cdef inline db_get(DB db, bytes key, object default, ReadOptions read_options):             # <<<<<<<<<<<<<<
 *     cdef string value
 *     cdef Status st
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.db_get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":116
 * 
 * 
 * cdef bytes to_file_system_name(name):             # <<<<<<<<<<<<<<
 *     if isinstance(name, bytes):
 *         return name
 */

static PyObject *__pyx_f_6plyvel_7_plyvel_to_file_system_name(PyObject *__pyx_v_name) {
  PyObject *__pyx_v_encoding = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("to_file_system_name", 0);

  /* "plyvel/_plyvel.pyx":117
 * 
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):             # <<<<<<<<<<<<<<
 *         return name
 * 
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_name); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":118
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):
 *         return name             # <<<<<<<<<<<<<<
 * 
 *     if not isinstance(name, unicode):
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyBytes_CheckExact(__pyx_v_name))||((__pyx_v_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_name)->tp_name), 0))) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_name);
    __pyx_r = ((PyObject*)__pyx_v_name);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":117
 * 
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):             # <<<<<<<<<<<<<<
 *         return name
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":120
 *         return name
 * 
 *     if not isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "'name' arg must be a byte string or a unicode string")
 */
  __pyx_t_2 = PyUnicode_Check(__pyx_v_name); 
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":121
 * 
 *     if not isinstance(name, unicode):
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "'name' arg must be a byte string or a unicode string")
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 121, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":120
 *         return name
 * 
 *     if not isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "'name' arg must be a byte string or a unicode string")
 */
  }

  /* "plyvel/_plyvel.pyx":124
 *             "'name' arg must be a byte string or a unicode string")
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'             # <<<<<<<<<<<<<<
 *     try:
 *         return name.encode(encoding)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getfilesystemencoding); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 124, __pyx_L1_error)
  if (!__pyx_t_1) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L5_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_n_s_ascii);
  __pyx_t_3 = __pyx_n_s_ascii;
  __pyx_L5_bool_binop_done:;
  __pyx_v_encoding = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    /*try:*/ {

      /* "plyvel/_plyvel.pyx":126
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:
 *         return name.encode(encoding)             # <<<<<<<<<<<<<<
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_encoding); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_encoding};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L7_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_encoding};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L7_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 126, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_INCREF(__pyx_v_encoding);
          __Pyx_GIVEREF(__pyx_v_encoding);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_encoding);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 126, __pyx_L7_error)
      __pyx_r = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;
      goto __pyx_L11_try_return;

      /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
 */
    }
    __pyx_L7_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "plyvel/_plyvel.pyx":127
 *     try:
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_UnicodeEncodeError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("plyvel._plyvel.to_file_system_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5) < 0) __PYX_ERR(0, 127, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_exc = __pyx_t_4;

      /* "plyvel/_plyvel.pyx":129
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_convert_unicode_name_to_a, __pyx_v_exc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 129, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_6);

      /* "plyvel/_plyvel.pyx":128
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)
 * 
 */
      __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 128, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 128, __pyx_L9_except_error)
    }
    goto __pyx_L9_except_error;
    __pyx_L9_except_error:;

    /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
 */
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L1_error;
    __pyx_L11_try_return:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":116
 * 
 * 
 * cdef bytes to_file_system_name(name):             # <<<<<<<<<<<<<<
 *     if isinstance(name, bytes):
 *         return name
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("plyvel._plyvel.to_file_system_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":132
 * 
 * 
 * cdef bytes bytes_increment(bytes s):             # <<<<<<<<<<<<<<
 *     # Increment the last byte that is not 0xff, and returned a new byte
 *     # string truncated after the position that was incremented. We use
 */

static PyObject *__pyx_f_6plyvel_7_plyvel_bytes_increment(PyObject *__pyx_v_s) {
  PyObject *__pyx_v_b = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("bytes_increment", 0);

  /* "plyvel/_plyvel.pyx":138
 *     # works the same in Python 2 and Python 3.
 * 
 *     b = bytearray(s)             # <<<<<<<<<<<<<<
 *     cdef int i = len(s) - 1
 *     while i >= 0:
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyByteArray_Type)), __pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":139
 * 
 *     b = bytearray(s)
 *     cdef int i = len(s) - 1             # <<<<<<<<<<<<<<
 *     while i >= 0:
 *         if b[i] == 0xff:
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 139, __pyx_L1_error)
  }
  __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_s); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_v_i = (__pyx_t_2 - 1);

  /* "plyvel/_plyvel.pyx":140
 *     b = bytearray(s)
 *     cdef int i = len(s) - 1
 *     while i >= 0:             # <<<<<<<<<<<<<<
 *         if b[i] == 0xff:
 *             i = i - 1
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_i >= 0) != 0);
    if (!__pyx_t_3) break;

    /* "plyvel/_plyvel.pyx":141
 *     cdef int i = len(s) - 1
 *     while i >= 0:
 *         if b[i] == 0xff:             # <<<<<<<<<<<<<<
 *             i = i - 1
 *             continue
 */
    __pyx_t_4 = __Pyx_GetItemInt_ByteArray(__pyx_v_b, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(__pyx_t_4 == -1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __pyx_t_3 = ((__pyx_t_4 == 0xff) != 0);
    if (__pyx_t_3) {

      /* "plyvel/_plyvel.pyx":142
 *     while i >= 0:
 *         if b[i] == 0xff:
 *             i = i - 1             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
      __pyx_v_i = (__pyx_v_i - 1);

      /* "plyvel/_plyvel.pyx":143
 *         if b[i] == 0xff:
 *             i = i - 1
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         # Found byte smaller than 0xff: increment and truncate
 */
      goto __pyx_L3_continue;

      /* "plyvel/_plyvel.pyx":141
 *     cdef int i = len(s) - 1
 *     while i >= 0:
 *         if b[i] == 0xff:             # <<<<<<<<<<<<<<
 *             i = i - 1
 *             continue
 */
    }

    /* "plyvel/_plyvel.pyx":146
 * 
 *         # Found byte smaller than 0xff: increment and truncate
 *         b[i] += 1             # <<<<<<<<<<<<<<
 *         return bytes(b[:i + 1])
 * 
 */
    __pyx_t_4 = __pyx_v_i;
    __pyx_t_5 = __Pyx_GetItemInt_ByteArray(__pyx_v_b, __pyx_t_4, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 146, __pyx_L1_error)
    if (unlikely((__pyx_t_5 + 1) < 0 || (__pyx_t_5 + 1) > 255)) {
      PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)"); __PYX_ERR(0, 146, __pyx_L1_error)
    }
    if (unlikely(__Pyx_SetItemInt_ByteArray(__pyx_v_b, __pyx_t_4, ((unsigned char)(__pyx_t_5 + 1)), int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 146, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":147
 *         # Found byte smaller than 0xff: increment and truncate
 *         b[i] += 1
 *         return bytes(b[:i + 1])             # <<<<<<<<<<<<<<
 * 
 *     # Input contained only 0xff bytes
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PySequence_GetSlice(__pyx_v_b, 0, (__pyx_v_i + 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;
    goto __pyx_L0;
    __pyx_L3_continue:;
  }

  /* "plyvel/_plyvel.pyx":150
 * 
 *     # Input contained only 0xff bytes
 *     return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":132
 * 
 * 
 * cdef bytes bytes_increment(bytes s):             # <<<<<<<<<<<<<<
 *     # Increment the last byte that is not 0xff, and returned a new byte
 *     # string truncated after the position that was incremented. We use
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.bytes_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":153
 * 
 * 
 * cdef int parse_options(Options *options, c_bool create_if_missing,             # <<<<<<<<<<<<<<
 *                        c_bool error_if_exists, object paranoid_checks,
 *                        object write_buffer_size, object max_open_files,
 */

static int __pyx_f_6plyvel_7_plyvel_parse_options(leveldb::Options *__pyx_v_options, bool __pyx_v_create_if_missing, bool __pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  size_t __pyx_v_c_lru_cache_size;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  bool __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  const char *__pyx_t_9;
  __Pyx_RefNannySetupContext("parse_options", 0);
  __Pyx_INCREF(__pyx_v_compression);

  /* "plyvel/_plyvel.pyx":162
 *     cdef size_t c_lru_cache_size
 * 
 *     options.create_if_missing = create_if_missing             # <<<<<<<<<<<<<<
 *     options.error_if_exists = error_if_exists
 * 
 */
  __pyx_v_options->create_if_missing = __pyx_v_create_if_missing;

  /* "plyvel/_plyvel.pyx":163
 * 
 *     options.create_if_missing = create_if_missing
 *     options.error_if_exists = error_if_exists             # <<<<<<<<<<<<<<
 * 
 *     if paranoid_checks is not None:
 */
  __pyx_v_options->error_if_exists = __pyx_v_error_if_exists;

  /* "plyvel/_plyvel.pyx":165
 *     options.error_if_exists = error_if_exists
 * 
 *     if paranoid_checks is not None:             # <<<<<<<<<<<<<<
 *         options.paranoid_checks = paranoid_checks
 * 
 */
  __pyx_t_1 = (__pyx_v_paranoid_checks != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":166
 * 
 *     if paranoid_checks is not None:
 *         options.paranoid_checks = paranoid_checks             # <<<<<<<<<<<<<<
 * 
 *     if write_buffer_size is not None:
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_paranoid_checks); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 166, __pyx_L1_error)
    __pyx_v_options->paranoid_checks = __pyx_t_3;

    /* "plyvel/_plyvel.pyx":165
 *     options.error_if_exists = error_if_exists
 * 
 *     if paranoid_checks is not None:             # <<<<<<<<<<<<<<
 *         options.paranoid_checks = paranoid_checks
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":168
 *         options.paranoid_checks = paranoid_checks
 * 
 *     if write_buffer_size is not None:             # <<<<<<<<<<<<<<
 *         options.write_buffer_size = write_buffer_size
 * 
 */
  __pyx_t_2 = (__pyx_v_write_buffer_size != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":169
 * 
 *     if write_buffer_size is not None:
 *         options.write_buffer_size = write_buffer_size             # <<<<<<<<<<<<<<
 * 
 *     if max_open_files is not None:
 */
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_write_buffer_size); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 169, __pyx_L1_error)
    __pyx_v_options->write_buffer_size = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":168
 *         options.paranoid_checks = paranoid_checks
 * 
 *     if write_buffer_size is not None:             # <<<<<<<<<<<<<<
 *         options.write_buffer_size = write_buffer_size
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":171
 *         options.write_buffer_size = write_buffer_size
 * 
 *     if max_open_files is not None:             # <<<<<<<<<<<<<<
 *         options.max_open_files = max_open_files
 * 
 */
  __pyx_t_1 = (__pyx_v_max_open_files != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":172
 * 
 *     if max_open_files is not None:
 *         options.max_open_files = max_open_files             # <<<<<<<<<<<<<<
 * 
 *     if lru_cache_size is not None:
 */
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_max_open_files); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 172, __pyx_L1_error)
    __pyx_v_options->max_open_files = __pyx_t_5;

    /* "plyvel/_plyvel.pyx":171
 *         options.write_buffer_size = write_buffer_size
 * 
 *     if max_open_files is not None:             # <<<<<<<<<<<<<<
 *         options.max_open_files = max_open_files
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":174
 *         options.max_open_files = max_open_files
 * 
 *     if lru_cache_size is not None:             # <<<<<<<<<<<<<<
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
 */
  __pyx_t_2 = (__pyx_v_lru_cache_size != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":175
 * 
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 */
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_lru_cache_size); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 175, __pyx_L1_error)
    __pyx_v_c_lru_cache_size = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":176
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":177
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
 *             options.block_cache = NewLRUCache(c_lru_cache_size)             # <<<<<<<<<<<<<<
 * 
 *     if block_size is not None:
 */
          __pyx_v_options->block_cache = leveldb::NewLRUCache(__pyx_v_c_lru_cache_size);
        }

        /* "plyvel/_plyvel.pyx":176
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L9;
          }
          __pyx_L9:;
        }
    }

    /* "plyvel/_plyvel.pyx":174
 *         options.max_open_files = max_open_files
 * 
 *     if lru_cache_size is not None:             # <<<<<<<<<<<<<<
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
 */
  }

  /* "plyvel/_plyvel.pyx":179
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 *     if block_size is not None:             # <<<<<<<<<<<<<<
 *         options.block_size = block_size
 * 
 */
  __pyx_t_1 = (__pyx_v_block_size != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":180
 * 
 *     if block_size is not None:
 *         options.block_size = block_size             # <<<<<<<<<<<<<<
 * 
 *     if block_restart_interval is not None:
 */
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_block_size); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 180, __pyx_L1_error)
    __pyx_v_options->block_size = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":179
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 *     if block_size is not None:             # <<<<<<<<<<<<<<
 *         options.block_size = block_size
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":182
 *         options.block_size = block_size
 * 
 *     if block_restart_interval is not None:             # <<<<<<<<<<<<<<
 *         options.block_restart_interval = block_restart_interval
 * 
 */
  __pyx_t_2 = (__pyx_v_block_restart_interval != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":183
 * 
 *     if block_restart_interval is not None:
 *         options.block_restart_interval = block_restart_interval             # <<<<<<<<<<<<<<
 * 
 *     if max_file_size is not None:
 */
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_block_restart_interval); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L1_error)
    __pyx_v_options->block_restart_interval = __pyx_t_5;

    /* "plyvel/_plyvel.pyx":182
 *         options.block_size = block_size
 * 
 *     if block_restart_interval is not None:             # <<<<<<<<<<<<<<
 *         options.block_restart_interval = block_restart_interval
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":185
 *         options.block_restart_interval = block_restart_interval
 * 
 *     if max_file_size is not None:             # <<<<<<<<<<<<<<
 *         options.max_file_size = max_file_size
 * 
 */
  __pyx_t_1 = (__pyx_v_max_file_size != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":186
 * 
 *     if max_file_size is not None:
 *         options.max_file_size = max_file_size             # <<<<<<<<<<<<<<
 * 
 *     if compression is None:
 */
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_max_file_size); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L1_error)
    __pyx_v_options->max_file_size = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":185
 *         options.block_restart_interval = block_restart_interval
 * 
 *     if max_file_size is not None:             # <<<<<<<<<<<<<<
 *         options.max_file_size = max_file_size
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":188
 *         options.max_file_size = max_file_size
 * 
 *     if compression is None:             # <<<<<<<<<<<<<<
 *         options.compression = leveldb.kNoCompression
 *     else:
 */
  __pyx_t_2 = (__pyx_v_compression == Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":189
 * 
 *     if compression is None:
 *         options.compression = leveldb.kNoCompression             # <<<<<<<<<<<<<<
 *     else:
 *         if isinstance(compression, bytes):
 */
    __pyx_v_options->compression = leveldb::kNoCompression;

    /* "plyvel/_plyvel.pyx":188
 *         options.max_file_size = max_file_size
 * 
 *     if compression is None:             # <<<<<<<<<<<<<<
 *         options.compression = leveldb.kNoCompression
 *     else:
 */
    goto __pyx_L13;
  }

  /* "plyvel/_plyvel.pyx":191
 *         options.compression = leveldb.kNoCompression
 *     else:
 *         if isinstance(compression, bytes):             # <<<<<<<<<<<<<<
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
 */
  /*else*/ {
    __pyx_t_1 = PyBytes_Check(__pyx_v_compression); 
    __pyx_t_2 = (__pyx_t_1 != 0);
    if (__pyx_t_2) {

      /* "plyvel/_plyvel.pyx":192
 *     else:
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')             # <<<<<<<<<<<<<<
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_compression, __pyx_n_s_decode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 192, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 192, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_compression, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "plyvel/_plyvel.pyx":191
 *         options.compression = leveldb.kNoCompression
 *     else:
 *         if isinstance(compression, bytes):             # <<<<<<<<<<<<<<
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
 */
    }

    /* "plyvel/_plyvel.pyx":193
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):             # <<<<<<<<<<<<<<
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
 */
    __pyx_t_2 = PyUnicode_Check(__pyx_v_compression); 
    __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":194
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")             # <<<<<<<<<<<<<<
 *         if compression == u'snappy':
 *             options.compression = leveldb.kSnappyCompression
 */
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 194, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 194, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":193
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):             # <<<<<<<<<<<<<<
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
 */
    }

    /* "plyvel/_plyvel.pyx":195
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':             # <<<<<<<<<<<<<<
 *             options.compression = leveldb.kSnappyCompression
 *         else:
 */
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_compression, __pyx_n_u_snappy, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 195, __pyx_L1_error)
    if (likely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":196
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
 *             options.compression = leveldb.kSnappyCompression             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("'compression' must be None or 'snappy'")
 */
      __pyx_v_options->compression = leveldb::kSnappyCompression;

      /* "plyvel/_plyvel.pyx":195
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':             # <<<<<<<<<<<<<<
 *             options.compression = leveldb.kSnappyCompression
 *         else:
 */
      goto __pyx_L16;
    }

    /* "plyvel/_plyvel.pyx":198
 *             options.compression = leveldb.kSnappyCompression
 *         else:
 *             raise ValueError("'compression' must be None or 'snappy'")             # <<<<<<<<<<<<<<
 * 
 *     if bloom_filter_bits > 0:
 */
    /*else*/ {
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 198, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 198, __pyx_L1_error)
    }
    __pyx_L16:;
  }
  __pyx_L13:;

  /* "plyvel/_plyvel.pyx":200
 *             raise ValueError("'compression' must be None or 'snappy'")
 * 
 *     if bloom_filter_bits > 0:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 */
  __pyx_t_1 = ((__pyx_v_bloom_filter_bits > 0) != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":201
 * 
 *     if bloom_filter_bits > 0:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":202
 *     if bloom_filter_bits > 0:
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)             # <<<<<<<<<<<<<<
 * 
 *     if (comparator is None) != (comparator_name is None):
 */
          __pyx_v_options->filter_policy = leveldb::NewBloomFilterPolicy(__pyx_v_bloom_filter_bits);
        }

        /* "plyvel/_plyvel.pyx":201
 * 
 *     if bloom_filter_bits > 0:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L20;
          }
          __pyx_L20:;
        }
    }

    /* "plyvel/_plyvel.pyx":200
 *             raise ValueError("'compression' must be None or 'snappy'")
 * 
 *     if bloom_filter_bits > 0:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 */
  }

  /* "plyvel/_plyvel.pyx":204
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 *     if (comparator is None) != (comparator_name is None):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "'comparator' and 'comparator_name' must be specified together")
 */
  __pyx_t_1 = (__pyx_v_comparator == Py_None);
  __pyx_t_2 = (__pyx_v_comparator_name == ((PyObject*)Py_None));
  __pyx_t_8 = ((__pyx_t_1 != __pyx_t_2) != 0);
  if (unlikely(__pyx_t_8)) {

    /* "plyvel/_plyvel.pyx":205
 * 
 *     if (comparator is None) != (comparator_name is None):
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 205, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":204
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 *     if (comparator is None) != (comparator_name is None):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "'comparator' and 'comparator_name' must be specified together")
 */
  }

  /* "plyvel/_plyvel.pyx":208
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 *     if comparator is not None:             # <<<<<<<<<<<<<<
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")
 */
  __pyx_t_8 = (__pyx_v_comparator != Py_None);
  __pyx_t_2 = (__pyx_t_8 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":209
 * 
 *     if comparator is not None:
 *         if not callable(comparator):             # <<<<<<<<<<<<<<
 *             raise TypeError("custom comparator object must be callable")
 * 
 */
    __pyx_t_2 = __Pyx_PyCallable_Check(__pyx_v_comparator); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 209, __pyx_L1_error)
    __pyx_t_8 = ((!(__pyx_t_2 != 0)) != 0);
    if (unlikely(__pyx_t_8)) {

      /* "plyvel/_plyvel.pyx":210
 *     if comparator is not None:
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")             # <<<<<<<<<<<<<<
 * 
 *         options.comparator = NewPlyvelCallbackComparator(
 */
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 210, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 210, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":209
 * 
 *     if comparator is not None:
 *         if not callable(comparator):             # <<<<<<<<<<<<<<
 *             raise TypeError("custom comparator object must be callable")
 * 
 */
    }

    /* "plyvel/_plyvel.pyx":213
 * 
 *         options.comparator = NewPlyvelCallbackComparator(
 *             comparator_name, comparator)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    if (unlikely(__pyx_v_comparator_name == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 213, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_v_comparator_name); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":212
 *             raise TypeError("custom comparator object must be callable")
 * 
 *         options.comparator = NewPlyvelCallbackComparator(             # <<<<<<<<<<<<<<
 *             comparator_name, comparator)
 * 
 */
    __pyx_v_options->comparator = NewPlyvelCallbackComparator(__pyx_t_9, __pyx_v_comparator);

    /* "plyvel/_plyvel.pyx":208
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 *     if comparator is not None:             # <<<<<<<<<<<<<<
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")
 */
  }

  /* "plyvel/_plyvel.pyx":153
 * 
 * 
 * cdef int parse_options(Options *options, c_bool create_if_missing,             # <<<<<<<<<<<<<<
 *                        c_bool error_if_exists, object paranoid_checks,
 *                        object write_buffer_size, object max_open_files,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("plyvel._plyvel.parse_options", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_compression);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_2DB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_2DB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyBoolObject *__pyx_v_create_if_missing = 0;
  PyBoolObject *__pyx_v_error_if_exists = 0;
  PyObject *__pyx_v_paranoid_checks = 0;
  PyObject *__pyx_v_write_buffer_size = 0;
  PyObject *__pyx_v_max_open_files = 0;
  PyObject *__pyx_v_lru_cache_size = 0;
  PyObject *__pyx_v_block_size = 0;
  PyObject *__pyx_v_block_restart_interval = 0;
  PyObject *__pyx_v_max_file_size = 0;
  PyObject *__pyx_v_compression = 0;
  int __pyx_v_bloom_filter_bits;
  PyObject *__pyx_v_comparator = 0;
  PyObject *__pyx_v_comparator_name = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_create_if_missing,&__pyx_n_s_error_if_exists,&__pyx_n_s_paranoid_checks,&__pyx_n_s_write_buffer_size,&__pyx_n_s_max_open_files,&__pyx_n_s_lru_cache_size,&__pyx_n_s_block_size,&__pyx_n_s_block_restart_interval,&__pyx_n_s_max_file_size,&__pyx_n_s_compression,&__pyx_n_s_bloom_filter_bits,&__pyx_n_s_comparator,&__pyx_n_s_comparator_name,0};
    PyObject* values[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    values[1] = (PyObject *)((PyBoolObject *)Py_False);

    /* "plyvel/_plyvel.pyx":229
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,             # <<<<<<<<<<<<<<
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
 */
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    values[3] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":230
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,             # <<<<<<<<<<<<<<
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,
 */
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":231
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
 */
    values[6] = ((PyObject *)Py_None);
    values[7] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":232
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):
 */
    values[8] = ((PyObject *)Py_None);
    values[9] = ((PyObject *)Py_None);
    values[10] = ((PyObject *)__pyx_n_s_snappy);

    /* "plyvel/_plyvel.pyx":234
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *         cdef Status st
 *         cdef string fsname
 */
    values[12] = ((PyObject *)Py_None);
    values[13] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args > 0 && likely(kw_args <= 13)) {
        Py_ssize_t index;
        for (index = 1; index < 14 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 228, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
    __pyx_v_create_if_missing = ((PyBoolObject *)values[1]);
    __pyx_v_error_if_exists = ((PyBoolObject *)values[2]);
    __pyx_v_paranoid_checks = values[3];
    __pyx_v_write_buffer_size = values[4];
    __pyx_v_max_open_files = values[5];
    __pyx_v_lru_cache_size = values[6];
    __pyx_v_block_size = values[7];
    __pyx_v_block_restart_interval = values[8];
    __pyx_v_max_file_size = values[9];
    __pyx_v_compression = values[10];
    if (values[11]) {
      __pyx_v_bloom_filter_bits = __Pyx_PyInt_As_int(values[11]); if (unlikely((__pyx_v_bloom_filter_bits == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 233, __pyx_L3_error)
    } else {
      __pyx_v_bloom_filter_bits = ((int)0);
    }
    __pyx_v_comparator = values[12];
    __pyx_v_comparator_name = ((PyObject*)values[13]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 228, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_create_if_missing), __pyx_ptype_7cpython_4bool_bool, 1, "create_if_missing", 0))) __PYX_ERR(0, 228, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_error_if_exists), __pyx_ptype_7cpython_4bool_bool, 1, "error_if_exists", 0))) __PYX_ERR(0, 229, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_comparator_name), (&PyBytes_Type), 1, "comparator_name", 1))) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB___init__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_name, __pyx_v_create_if_missing, __pyx_v_error_if_exists, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name);

  /* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_2DB___init__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name, PyBoolObject *__pyx_v_create_if_missing, PyBoolObject *__pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  bool __pyx_t_3;
  bool __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":237
 *         cdef Status st
 *         cdef string fsname
 *         self.name = name             # <<<<<<<<<<<<<<
 * 
 *         fsname = to_file_system_name(name)
 */
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "plyvel/_plyvel.pyx":239
 *         self.name = name
 * 
 *         fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *         parse_options(
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __pyx_t_2;

  /* "plyvel/_plyvel.pyx":241
 *         fsname = to_file_system_name(name)
 *         parse_options(
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,             # <<<<<<<<<<<<<<
 *             write_buffer_size, max_open_files, lru_cache_size, block_size,
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_create_if_missing)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_error_if_exists)); if (unlikely((__pyx_t_4 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":240
 * 
 *         fsname = to_file_system_name(name)
 *         parse_options(             # <<<<<<<<<<<<<<
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,
 *             write_buffer_size, max_open_files, lru_cache_size, block_size,
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_parse_options((&__pyx_v_self->options), __pyx_t_3, __pyx_t_4, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 240, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":245
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *             comparator, comparator_name)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":246
 *             comparator, comparator_name)
 *         with nogil:
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
 */
        __pyx_v_st = leveldb::DB::Open(__pyx_v_self->options, __pyx_v_fsname, (&__pyx_v_self->_db));
      }

      /* "plyvel/_plyvel.pyx":245
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *             comparator, comparator_name)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":247
 *         with nogil:
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *         # Keep weak references to open iterators, since deleting a C++
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 247, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":254
 *         # leveldb/db.h). We don't use weakref.WeakValueDictionary here
 *         # for performance reasons.
 *         self.lock = threading.Lock()             # <<<<<<<<<<<<<<
 *         self.iterators = dict()
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_Lock); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->lock);
  __Pyx_DECREF(__pyx_v_self->lock);
  __pyx_v_self->lock = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":255
 *         # for performance reasons.
 *         self.lock = threading.Lock()
 *         self.iterators = dict()             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->iterators);
  __Pyx_DECREF(__pyx_v_self->iterators);
  __pyx_v_self->iterators = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":257
 *         self.iterators = dict()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         # If the constructor raised an exception (and hence never
 *         # completed), self.iterators can be None. In that case no
 */

static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_iterator = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("close", 0);

  /* "plyvel/_plyvel.pyx":262
 *         # iterators need to be cleaned anyway.
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:             # <<<<<<<<<<<<<<
 *             with self.lock:
 *                 while self.iterators:
 */
  __pyx_t_1 = (__pyx_v_self->iterators != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 */
    /*with:*/ {
      __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 263, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L4_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L4_error)
      }
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_9);
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":264
 *         if self.iterators is not None:
 *             with self.lock:
 *                 while self.iterators:             # <<<<<<<<<<<<<<
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:
 */
            while (1) {
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->iterators); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 264, __pyx_L8_error)
              if (!__pyx_t_2) break;

              /* "plyvel/_plyvel.pyx":265
 *             with self.lock:
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()             # <<<<<<<<<<<<<<
 *                     if iterator is not None:
 *                         iterator.close()
 */
              __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->iterators, __pyx_n_s_popitem); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 265, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_6);
              __pyx_t_10 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
                if (likely(__pyx_t_10)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_6, function);
                }
              }
              if (__pyx_t_10) {
                __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 265, __pyx_L8_error)
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else {
                __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 265, __pyx_L8_error)
              }
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_5, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 265, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __pyx_t_5 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
                __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
                if (likely(__pyx_t_5)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                  __Pyx_INCREF(__pyx_t_5);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_6, function);
                }
              }
              if (__pyx_t_5) {
                __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L8_error)
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              } else {
                __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L8_error)
              }
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_6plyvel_7_plyvel_BaseIterator))))) __PYX_ERR(0, 265, __pyx_L8_error)
              __Pyx_XDECREF_SET(__pyx_v_iterator, ((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_t_4));
              __pyx_t_4 = 0;

              /* "plyvel/_plyvel.pyx":266
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:             # <<<<<<<<<<<<<<
 *                         iterator.close()
 * 
 */
              __pyx_t_2 = (((PyObject *)__pyx_v_iterator) != Py_None);
              __pyx_t_1 = (__pyx_t_2 != 0);
              if (__pyx_t_1) {

                /* "plyvel/_plyvel.pyx":267
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:
 *                         iterator.close()             # <<<<<<<<<<<<<<
 * 
 *         if self._db is not NULL:
 */
                __pyx_t_4 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_iterator->__pyx_vtab)->close(__pyx_v_iterator, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 267, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

                /* "plyvel/_plyvel.pyx":266
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:             # <<<<<<<<<<<<<<
 *                         iterator.close()
 * 
 */
              }
            }

            /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 */
          }
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          goto __pyx_L13_try_end;
          __pyx_L8_error:;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_10 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_10, NULL);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (__pyx_t_1 < 0) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
            if (__pyx_t_2) {
              __Pyx_GIVEREF(__pyx_t_4);
              __Pyx_GIVEREF(__pyx_t_6);
              __Pyx_XGIVEREF(__pyx_t_5);
              __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_6, __pyx_t_5);
              __pyx_t_4 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; 
              __PYX_ERR(0, 263, __pyx_L10_except_error)
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            goto __pyx_L9_exception_handled;
          }
          __pyx_L10_except_error:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          goto __pyx_L1_error;
          __pyx_L9_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          __pyx_L13_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_3) {
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__7, NULL);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 263, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
      goto __pyx_L20;
      __pyx_L4_error:;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L1_error;
      __pyx_L20:;
    }

    /* "plyvel/_plyvel.pyx":262
 *         # iterators need to be cleaned anyway.
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:             # <<<<<<<<<<<<<<
 *             with self.lock:
 *                 while self.iterators:
 */
  }

  /* "plyvel/_plyvel.pyx":269
 *                         iterator.close()
 * 
 *         if self._db is not NULL:             # <<<<<<<<<<<<<<
 *             del self._db
 *             self._db = NULL
 */
  __pyx_t_2 = ((__pyx_v_self->_db != NULL) != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":270
 * 
 *         if self._db is not NULL:
 *             del self._db             # <<<<<<<<<<<<<<
 *             self._db = NULL
 * 
 */
    delete __pyx_v_self->_db;

    /* "plyvel/_plyvel.pyx":271
 *         if self._db is not NULL:
 *             del self._db
 *             self._db = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.block_cache is not NULL:
 */
    __pyx_v_self->_db = NULL;

    /* "plyvel/_plyvel.pyx":269
 *                         iterator.close()
 * 
 *         if self._db is not NULL:             # <<<<<<<<<<<<<<
 *             del self._db
 *             self._db = NULL
 */
  }

  /* "plyvel/_plyvel.pyx":273
 *             self._db = NULL
 * 
 *         if self.options.block_cache is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.block_cache
 *             self.options.block_cache = NULL
 */
  __pyx_t_2 = ((__pyx_v_self->options.block_cache != NULL) != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":274
 * 
 *         if self.options.block_cache is not NULL:
 *             del self.options.block_cache             # <<<<<<<<<<<<<<
 *             self.options.block_cache = NULL
 * 
 */
    delete __pyx_v_self->options.block_cache;

    /* "plyvel/_plyvel.pyx":275
 *         if self.options.block_cache is not NULL:
 *             del self.options.block_cache
 *             self.options.block_cache = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.filter_policy is not NULL:
 */
    __pyx_v_self->options.block_cache = NULL;

    /* "plyvel/_plyvel.pyx":273
 *             self._db = NULL
 * 
 *         if self.options.block_cache is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.block_cache
 *             self.options.block_cache = NULL
 */
  }

  /* "plyvel/_plyvel.pyx":277
 *             self.options.block_cache = NULL
 * 
 *         if self.options.filter_policy is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL
 */
  __pyx_t_2 = ((__pyx_v_self->options.filter_policy != NULL) != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":278
 * 
 *         if self.options.filter_policy is not NULL:
 *             del self.options.filter_policy             # <<<<<<<<<<<<<<
 *             self.options.filter_policy = NULL
 * 
 */
    delete __pyx_v_self->options.filter_policy;

    /* "plyvel/_plyvel.pyx":279
 *         if self.options.filter_policy is not NULL:
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.comparator is not NULL:
 */
    __pyx_v_self->options.filter_policy = NULL;

    /* "plyvel/_plyvel.pyx":277
 *             self.options.block_cache = NULL
 * 
 *         if self.options.filter_policy is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL
 */
  }

  /* "plyvel/_plyvel.pyx":281
 *             self.options.filter_policy = NULL
 * 
 *         if self.options.comparator is not NULL:             # <<<<<<<<<<<<<<
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
 */
  __pyx_t_2 = ((__pyx_v_self->options.comparator != NULL) != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":283
 *         if self.options.comparator is not NULL:
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():             # <<<<<<<<<<<<<<
 *                 del self.options.comparator
 *                 self.options.comparator = NULL
 */
    __pyx_t_2 = ((__pyx_v_self->options.comparator != leveldb::BytewiseComparator()) != 0);
    if (__pyx_t_2) {

      /* "plyvel/_plyvel.pyx":284
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
 *                 del self.options.comparator             # <<<<<<<<<<<<<<
 *                 self.options.comparator = NULL
 * 
 */
      delete __pyx_v_self->options.comparator;

      /* "plyvel/_plyvel.pyx":285
 *             if self.options.comparator is not BytewiseComparator():
 *                 del self.options.comparator
 *                 self.options.comparator = NULL             # <<<<<<<<<<<<<<
 * 
 *     property closed:
 */
      __pyx_v_self->options.comparator = NULL;

      /* "plyvel/_plyvel.pyx":283
 *         if self.options.comparator is not NULL:
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():             # <<<<<<<<<<<<<<
 *                 del self.options.comparator
 *                 self.options.comparator = NULL
 */
    }

    /* "plyvel/_plyvel.pyx":281
 *             self.options.filter_policy = NULL
 * 
 *         if self.options.comparator is not NULL:             # <<<<<<<<<<<<<<
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
 */
  }

  /* "plyvel/_plyvel.pyx":257
 *         self.iterators = dict()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         # If the constructor raised an exception (and hence never
 *         # completed), self.iterators can be None. In that case no
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_iterator);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_2close[] = "DB.close(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_2close(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_2close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_2DB_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":288
 * 
 *     property closed:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._db is NULL
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "plyvel/_plyvel.pyx":289
 *     property closed:
 *         def __get__(self):
 *             return self._db is NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_self->_db == NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":288
 * 
 *     property closed:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._db is NULL
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.closed.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":291
 *             return self._db is NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":292
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_2DB_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":291
 *             return self._db is NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.DB.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":294
 *         self.close()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_7__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "plyvel/_plyvel.pyx":295
 * 
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.name,
 *             ' (closed)' if self.closed else '',
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":297
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
 *             ' (closed)' if self.closed else '',             # <<<<<<<<<<<<<<
 *             hex(id(self)),
 *         )
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_closed); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_INCREF(__pyx_kp_s_closed_2);
    __pyx_t_1 = __pyx_kp_s_closed_2;
  } else {
    __Pyx_INCREF(__pyx_kp_s__8);
    __pyx_t_1 = __pyx_kp_s__8;
  }

  /* "plyvel/_plyvel.pyx":298
 *             self.name,
 *             ' (closed)' if self.closed else '',
 *             hex(id(self)),             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_hex, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":296
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,             # <<<<<<<<<<<<<<
 *             ' (closed)' if self.closed else '',
 *             hex(id(self)),
 */
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;

  /* "plyvel/_plyvel.pyx":295
 * 
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.name,
 *             ' (closed)' if self.closed else '',
 */
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_plyvel_DB_with_name_r_s_at_0x_s, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":294
 *         self.close()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_9get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_8get[] = "DB.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_9get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_default,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":302
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    values[3] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 2; index < 4 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get") < 0)) __PYX_ERR(0, 301, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[2]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 301, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 301, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 302, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 302, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_8get(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_8get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  leveldb::ReadOptions __pyx_v_read_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  bool __pyx_t_3;
  __Pyx_RefNannySetupContext("get", 0);

  /* "plyvel/_plyvel.pyx":303
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":304
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 304, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":303
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":307
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":308
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 * 
 *         return db_get(self, key, default, read_options)
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 308, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":310
 *         read_options.fill_cache = fill_cache
 * 
 *         return db_get(self, key, default, read_options)             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_db_get(__pyx_v_self, __pyx_v_key, __pyx_v_default, __pyx_v_read_options); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":312
 *         return db_get(self, key, default, read_options)
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_11put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_10put[] = "DB.put(self, bytes key, value, *, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_11put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,&__pyx_n_s_sync,0};
    PyObject* values[3] = {0,0,0};
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, 1); __PYX_ERR(0, 312, __pyx_L3_error)
        }
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 2;
        PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "put") < 0)) __PYX_ERR(0, 312, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
    __pyx_v_sync = ((PyBoolObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 312, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 312, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 312, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 312, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_10put(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_value, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_10put(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyBoolObject *__pyx_v_sync) {
  leveldb::WriteOptions __pyx_v_write_options;
  leveldb::Slice __pyx_v_key_slice;
  Py_buffer __pyx_v_value_buffer;
  leveldb::Status __pyx_v_st;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  bool __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("put", 0);

  /* "plyvel/_plyvel.pyx":313
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":314
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef WriteOptions write_options = WriteOptions()
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 314, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":313
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":316
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef WriteOptions write_options = WriteOptions()             # <<<<<<<<<<<<<<
 *         write_options.sync = sync
 * 
 */
  __pyx_v_write_options = leveldb::WriteOptions();

  /* "plyvel/_plyvel.pyx":317
 * 
 *         cdef WriteOptions write_options = WriteOptions()
 *         write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_sync)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_v_write_options.sync = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":319
 *         write_options.sync = sync
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         cdef Py_buffer value_buffer
 *         cdef Status st
 */
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

  /* "plyvel/_plyvel.pyx":322
 *         cdef Py_buffer value_buffer
 *         cdef Status st
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)             # <<<<<<<<<<<<<<
 *         try:
 *             with nogil:
 */
  __pyx_t_6 = PyObject_GetBuffer(__pyx_v_value, (&__pyx_v_value_buffer), PyBUF_SIMPLE); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 322, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":323
 *         cdef Status st
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 st = self._db.Put(
 */
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":324
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 st = self._db.Put(
 *                     write_options,
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":325
 *         try:
 *             with nogil:
 *                 st = self._db.Put(             # <<<<<<<<<<<<<<
 *                     write_options,
 *                     key_slice,
 */
          __pyx_v_st = __pyx_v_self->_db->Put(__pyx_v_write_options, __pyx_v_key_slice, leveldb::Slice(((const char *)__pyx_v_value_buffer.buf), __pyx_v_value_buffer.len));
        }

        /* "plyvel/_plyvel.pyx":324
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 st = self._db.Put(
 *                     write_options,
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L9;
          }
          __pyx_L9:;
        }
    }
  }

  /* "plyvel/_plyvel.pyx":330
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
 *         finally:
 *             PyBuffer_Release(&value_buffer)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyBuffer_Release((&__pyx_v_value_buffer));
      goto __pyx_L6;
    }
    __pyx_L6:;
  }

  /* "plyvel/_plyvel.pyx":331
 *         finally:
 *             PyBuffer_Release(&value_buffer)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 */
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 331, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":312
 *         return db_get(self, key, default, read_options)
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":333
 *         raise_for_status(st)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_13delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_12delete[] = "DB.delete(self, bytes key, *, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_13delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_sync,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 1;
        PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "delete") < 0)) __PYX_ERR(0, 333, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_sync = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 333, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 333, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 333, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_12delete(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_12delete(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyBoolObject *__pyx_v_sync) {
  leveldb::Status __pyx_v_st;
  leveldb::WriteOptions __pyx_v_write_options;
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  bool __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "plyvel/_plyvel.pyx":334
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":335
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 335, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":334
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":339
 *         cdef Status st
 *         cdef WriteOptions write_options
 *         write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_sync)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
  __pyx_v_write_options.sync = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":341
 *         write_options.sync = sync
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)
 */
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 341, __pyx_L1_error)
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 341, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

  /* "plyvel/_plyvel.pyx":342
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":343
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
 */
        __pyx_v_st = __pyx_v_self->_db->Delete(__pyx_v_write_options, __pyx_v_key_slice);
      }

      /* "plyvel/_plyvel.pyx":342
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":344
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 */
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 344, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":333
 *         raise_for_status(st)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":346
 *         raise_for_status(st)
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_14write_batch[] = "DB.write_batch(self, *, bool transaction=False, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyBoolObject *__pyx_v_transaction = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_transaction,&__pyx_n_s_sync,0};
    PyObject* values[2] = {0,0};
    values[0] = (PyObject *)((PyBoolObject *)Py_False);
    values[1] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 0; index < 2 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "write_batch") < 0)) __PYX_ERR(0, 346, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_transaction = ((PyBoolObject *)values[0]);
    __pyx_v_sync = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_batch", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 346, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transaction), __pyx_ptype_7cpython_4bool_bool, 1, "transaction", 0))) __PYX_ERR(0, 346, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 346, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyBoolObject *__pyx_v_transaction, PyBoolObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("write_batch", 0);

  /* "plyvel/_plyvel.pyx":347
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":348
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return WriteBatch(self, None, transaction, sync)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 348, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":347
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":350
 *             raise RuntimeError("Database is closed")
 * 
 *         return WriteBatch(self, None, transaction, sync)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_2, 1, Py_None);
  __Pyx_INCREF(((PyObject *)__pyx_v_transaction));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_transaction));
  PyTuple_SET_ITEM(__pyx_t_2, 2, ((PyObject *)__pyx_v_transaction));
  __Pyx_INCREF(((PyObject *)__pyx_v_sync));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sync));
  PyTuple_SET_ITEM(__pyx_t_2, 3, ((PyObject *)__pyx_v_sync));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_WriteBatch), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":346
 *         raise_for_status(st)
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":352
 *         return WriteBatch(self, None, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_17__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_17__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":353
 * 
 *     def __iter__(self):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":354
 *     def __iter__(self):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.iterator()
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 354, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":353
 * 
 *     def __iter__(self):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":356
 *             raise RuntimeError("Database is closed")
 * 
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_iterator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":352
 *         return WriteBatch(self, None, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_19iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_18iterator[] = "DB.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_19iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_reverse,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_include_start,&__pyx_n_s_include_stop,&__pyx_n_s_prefix,&__pyx_n_s_include_key,&__pyx_n_s_include_value,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":359
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 */
    values[3] = ((PyObject *)Py_True);
    values[4] = ((PyObject *)Py_False);
    values[5] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":360
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
 */
    values[6] = ((PyObject *)Py_True);
    values[7] = ((PyObject *)Py_True);

    /* "plyvel/_plyvel.pyx":361
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self,  # db
 */
    values[8] = (PyObject *)((PyBoolObject *)Py_False);
    values[9] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 10)) {
        Py_ssize_t index;
        for (index = 0; index < 10 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "iterator") < 0)) __PYX_ERR(0, 358, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[8]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[9]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 358, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_18iterator(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_18iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":362
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             None,  # db_prefix
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":375
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = PyTuple_New(13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __Pyx_INCREF(__pyx_v_reverse);
  __Pyx_GIVEREF(__pyx_v_reverse);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_reverse);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_start);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_GIVEREF(__pyx_v_stop);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_stop);
  __Pyx_INCREF(__pyx_v_include_start);
  __Pyx_GIVEREF(__pyx_v_include_start);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_v_include_start);
  __Pyx_INCREF(__pyx_v_include_stop);
  __Pyx_GIVEREF(__pyx_v_include_stop);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_v_include_stop);
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_v_prefix);
  __Pyx_INCREF(__pyx_v_include_key);
  __Pyx_GIVEREF(__pyx_v_include_key);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_v_include_key);
  __Pyx_INCREF(__pyx_v_include_value);
  __Pyx_GIVEREF(__pyx_v_include_value);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_v_include_value);
  __Pyx_INCREF(((PyObject *)__pyx_v_verify_checksums));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_verify_checksums));
  PyTuple_SET_ITEM(__pyx_t_1, 10, ((PyObject *)__pyx_v_verify_checksums));
  __Pyx_INCREF(((PyObject *)__pyx_v_fill_cache));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_fill_cache));
  PyTuple_SET_ITEM(__pyx_t_1, 11, ((PyObject *)__pyx_v_fill_cache));
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 12, Py_None);

  /* "plyvel/_plyvel.pyx":362
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             None,  # db_prefix
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":378
 *         )
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return RawIterator(
 *             self,  # db
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_20raw_iterator[] = "DB.raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("raw_iterator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[2] = {0,0};
    values[0] = (PyObject *)((PyBoolObject *)Py_False);
    values[1] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 0; index < 2 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "raw_iterator") < 0)) __PYX_ERR(0, 378, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_verify_checksums = ((PyBoolObject *)values[0]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("raw_iterator", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 378, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 378, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 378, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("raw_iterator", 0);

  /* "plyvel/_plyvel.pyx":379
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             verify_checksums,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":383
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(((PyObject *)__pyx_v_verify_checksums));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_verify_checksums));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_verify_checksums));
  __Pyx_INCREF(((PyObject *)__pyx_v_fill_cache));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_fill_cache));
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_fill_cache));
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 3, Py_None);

  /* "plyvel/_plyvel.pyx":379
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             verify_checksums,
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_RawIterator), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":378
 *         )
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return RawIterator(
 *             self,  # db
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":386
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_22snapshot[] = "DB.snapshot(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("snapshot (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("snapshot", 0);

  /* "plyvel/_plyvel.pyx":387
 * 
 *     def snapshot(self):
 *         return Snapshot(db=self)             # <<<<<<<<<<<<<<
 * 
 *     def get_property(self, bytes name not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 387, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Snapshot), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":386
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.snapshot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":389
 *         return Snapshot(db=self)
 * 
 *     def get_property(self, bytes name not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_25get_property(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_24get_property[] = "DB.get_property(self, bytes name)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_25get_property(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_property (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyBytes_Type), 0, "name", 1))) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_24get_property(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), ((PyObject*)__pyx_v_name));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_24get_property(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name) {
  leveldb::Slice __pyx_v_sl;
  std::string __pyx_v_value;
  bool __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  const char *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("get_property", 0);

  /* "plyvel/_plyvel.pyx":390
 * 
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":391
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice sl = Slice(name, len(name))
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 391, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":390
 * 
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":393
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef Slice sl = Slice(name, len(name))             # <<<<<<<<<<<<<<
 *         cdef string value
 *         cdef c_bool result
 */
  __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_name); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_t_4 = PyBytes_GET_SIZE(__pyx_v_name); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_v_sl = leveldb::Slice(__pyx_t_3, __pyx_t_4);

  /* "plyvel/_plyvel.pyx":397
 *         cdef c_bool result
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             result = self._db.GetProperty(sl, &value)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":398
 * 
 *         with nogil:
 *             result = self._db.GetProperty(sl, &value)             # <<<<<<<<<<<<<<
 * 
 *         return value if result else None
 */
        __pyx_v_result = __pyx_v_self->_db->GetProperty(__pyx_v_sl, (&__pyx_v_value));
      }

      /* "plyvel/_plyvel.pyx":397
 *         cdef c_bool result
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             result = self._db.GetProperty(sl, &value)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":400
 *             result = self._db.GetProperty(sl, &value)
 * 
 *         return value if result else None             # <<<<<<<<<<<<<<
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 */
  __Pyx_XDECREF(__pyx_r);
  if ((__pyx_v_result != 0)) {
    __pyx_t_5 = __pyx_convert_PyBytes_string_to_py_std__in_string(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_2 = Py_None;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":389
 *         return Snapshot(db=self)
 * 
 *     def get_property(self, bytes name not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.DB.get_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":402
 *         return value if result else None
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_26compact_range[] = "DB.compact_range(self, *, bytes start=None, bytes stop=None)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compact_range (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject*)Py_None);
    values[1] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 0; index < 2 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "compact_range") < 0)) __PYX_ERR(0, 402, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_start = ((PyObject*)values[0]);
    __pyx_v_stop = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compact_range", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 402, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.compact_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 1, "start", 1))) __PYX_ERR(0, 402, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 1, "stop", 1))) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop) {
  leveldb::Slice __pyx_v_start_slice;
  leveldb::Slice __pyx_v_stop_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("compact_range", 0);

  /* "plyvel/_plyvel.pyx":403
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":404
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice start_slice
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 404, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":403
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":409
 *         cdef Slice stop_slice
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             start_slice = Slice(start, len(start))
 * 
 */
  __pyx_t_1 = (__pyx_v_start != ((PyObject*)Py_None));
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":410
 * 
 *         if start is not None:
 *             start_slice = Slice(start, len(start))             # <<<<<<<<<<<<<<
 * 
 *         if stop is not None:
 */
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 410, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_start); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L1_error)
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 410, __pyx_L1_error)
    }
    __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_start); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 410, __pyx_L1_error)
    __pyx_v_start_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

    /* "plyvel/_plyvel.pyx":409
 *         cdef Slice stop_slice
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             start_slice = Slice(start, len(start))
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":412
 *             start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             stop_slice = Slice(stop, len(stop))
 * 
 */
  __pyx_t_3 = (__pyx_v_stop != ((PyObject*)Py_None));
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":413
 * 
 *         if stop is not None:
 *             stop_slice = Slice(stop, len(stop))             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 413, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 413, __pyx_L1_error)
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 413, __pyx_L1_error)
    }
    __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_stop); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 413, __pyx_L1_error)
    __pyx_v_stop_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

    /* "plyvel/_plyvel.pyx":412
 *             start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             stop_slice = Slice(stop, len(stop))
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":415
 *             stop_slice = Slice(stop, len(stop))
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":416
 * 
 *         with nogil:
 *             self._db.CompactRange(&start_slice, &stop_slice)             # <<<<<<<<<<<<<<
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 */
        __pyx_v_self->_db->CompactRange((&__pyx_v_start_slice), (&__pyx_v_stop_slice));
      }

      /* "plyvel/_plyvel.pyx":415
 *             stop_slice = Slice(stop, len(stop))
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "plyvel/_plyvel.pyx":402
 *         return value if result else None
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.compact_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":418
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_28approximate_size[] = "DB.approximate_size(self, bytes start, bytes stop)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("approximate_size (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_stop,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("approximate_size", 1, 2, 2, 1); __PYX_ERR(0, 418, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "approximate_size") < 0)) __PYX_ERR(0, 418, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_start = ((PyObject*)values[0]);
    __pyx_v_stop = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("approximate_size", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 418, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 0, "start", 1))) __PYX_ERR(0, 418, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 0, "stop", 1))) __PYX_ERR(0, 418, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("approximate_size", 0);

  /* "plyvel/_plyvel.pyx":419
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":420
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.approximate_sizes((start, stop))[0]
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 420, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":419
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":422
 *             raise RuntimeError("Database is closed")
 * 
 *         return self.approximate_sizes((start, stop))[0]             # <<<<<<<<<<<<<<
 * 
 *     def approximate_sizes(self, *ranges):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_approximate_sizes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_start);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_GIVEREF(__pyx_v_stop);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_stop);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 422, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":418
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":424
 *         return self.approximate_sizes((start, stop))[0]
 * 
 *     def approximate_sizes(self, *ranges):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_30approximate_sizes[] = "DB.approximate_sizes(self, *ranges)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_ranges = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("approximate_sizes (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "approximate_sizes", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_ranges = __pyx_args;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_ranges);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_ranges);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_ranges) {
  int __pyx_v_n_ranges;
  leveldb::Range *__pyx_v_c_ranges;
  uint64_t *__pyx_v_sizes;
  int __pyx_v_i;
  PyObject *__pyx_v_start = NULL;
  PyObject *__pyx_v_stop = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  int __pyx_t_12;
  const char *__pyx_t_13;
  const char *__pyx_t_14;
  Py_ssize_t __pyx_t_15;
  char const *__pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  __Pyx_RefNannySetupContext("approximate_sizes", 0);

  /* "plyvel/_plyvel.pyx":425
 * 
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":426
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef int n_ranges = len(ranges)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 426, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":425
 * 
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":428
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef int n_ranges = len(ranges)             # <<<<<<<<<<<<<<
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 */
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v_ranges); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 428, __pyx_L1_error)
  __pyx_v_n_ranges = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":429
 * 
 *         cdef int n_ranges = len(ranges)
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))             # <<<<<<<<<<<<<<
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:
 */
  __pyx_v_c_ranges = ((leveldb::Range *)malloc((__pyx_v_n_ranges * (sizeof(leveldb::Range)))));

  /* "plyvel/_plyvel.pyx":430
 *         cdef int n_ranges = len(ranges)
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))             # <<<<<<<<<<<<<<
 *         try:
 *             for i in xrange(n_ranges):
 */
  __pyx_v_sizes = ((uint64_t *)malloc((__pyx_v_n_ranges * (sizeof(uint64_t)))));

  /* "plyvel/_plyvel.pyx":431
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:             # <<<<<<<<<<<<<<
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
 */
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":432
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:
 *             for i in xrange(n_ranges):             # <<<<<<<<<<<<<<
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 */
    __pyx_t_4 = __pyx_v_n_ranges;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;

      /* "plyvel/_plyvel.pyx":433
 *         try:
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]             # <<<<<<<<<<<<<<
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 *                     raise TypeError(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_ranges, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 433, __pyx_L5_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        #else
        __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_9 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_10 = Py_TYPE(__pyx_t_9)->tp_iternext;
        index = 0; __pyx_t_7 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_7)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        index = 1; __pyx_t_8 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_8)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_9), 2) < 0) __PYX_ERR(0, 433, __pyx_L5_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 433, __pyx_L5_error)
        __pyx_L10_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_start, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_stop, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "plyvel/_plyvel.pyx":434
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):             # <<<<<<<<<<<<<<
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
 */
      __pyx_t_11 = PyBytes_Check(__pyx_v_start); 
      __pyx_t_12 = ((!(__pyx_t_11 != 0)) != 0);
      if (!__pyx_t_12) {
      } else {
        __pyx_t_1 = __pyx_t_12;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_12 = PyBytes_Check(__pyx_v_stop); 
      __pyx_t_11 = ((!(__pyx_t_12 != 0)) != 0);
      __pyx_t_1 = __pyx_t_11;
      __pyx_L12_bool_binop_done:;
      if (unlikely(__pyx_t_1)) {

        /* "plyvel/_plyvel.pyx":435
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 *                     raise TypeError(             # <<<<<<<<<<<<<<
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(
 */
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 435, __pyx_L5_error)

        /* "plyvel/_plyvel.pyx":434
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):             # <<<<<<<<<<<<<<
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
 */
      }

      /* "plyvel/_plyvel.pyx":438
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(
 *                     Slice(start, len(start)),             # <<<<<<<<<<<<<<
 *                     Slice(stop, len(stop)))
 * 
 */
      __pyx_t_13 = __Pyx_PyObject_AsString(__pyx_v_start); if (unlikely((!__pyx_t_13) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L5_error)
      __pyx_t_3 = PyObject_Length(__pyx_v_start); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 438, __pyx_L5_error)

      /* "plyvel/_plyvel.pyx":439
 *                 c_ranges[i] = Range(
 *                     Slice(start, len(start)),
 *                     Slice(stop, len(stop)))             # <<<<<<<<<<<<<<
 * 
 *             with nogil:
 */
      __pyx_t_14 = __Pyx_PyObject_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 439, __pyx_L5_error)
      __pyx_t_15 = PyObject_Length(__pyx_v_stop); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 439, __pyx_L5_error)

      /* "plyvel/_plyvel.pyx":437
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(             # <<<<<<<<<<<<<<
 *                     Slice(start, len(start)),
 *                     Slice(stop, len(stop)))
 */
      (__pyx_v_c_ranges[__pyx_v_i]) = leveldb::Range(leveldb::Slice(__pyx_t_13, __pyx_t_3), leveldb::Slice(__pyx_t_14, __pyx_t_15));
    }

    /* "plyvel/_plyvel.pyx":441
 *                     Slice(stop, len(stop)))
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":442
 * 
 *             with nogil:
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)             # <<<<<<<<<<<<<<
 * 
 *             return [sizes[i] for i in xrange(n_ranges)]
 */
          __pyx_v_self->_db->GetApproximateSizes(__pyx_v_c_ranges, __pyx_v_n_ranges, __pyx_v_sizes);
        }

        /* "plyvel/_plyvel.pyx":441
 *                     Slice(stop, len(stop)))
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L16;
          }
          __pyx_L16:;
        }
    }

    /* "plyvel/_plyvel.pyx":444
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
 *             return [sizes[i] for i in xrange(n_ranges)]             # <<<<<<<<<<<<<<
 *         finally:
 *             free(c_ranges)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_v_n_ranges;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;
      __pyx_t_8 = __Pyx_PyInt_From_uint64_t((__pyx_v_sizes[__pyx_v_i])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 444, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_8))) __PYX_ERR(0, 444, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L4_return;
  }

  /* "plyvel/_plyvel.pyx":446
 *             return [sizes[i] for i in xrange(n_ranges)]
 *         finally:
 *             free(c_ranges)             # <<<<<<<<<<<<<<
 *             free(sizes)
 * 
 */
  /*finally:*/ {
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_22);
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_16 = __pyx_filename;
      {
        free(__pyx_v_c_ranges);

        /* "plyvel/_plyvel.pyx":447
 *         finally:
 *             free(c_ranges)
 *             free(sizes)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
 */
        free(__pyx_v_sizes);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_XGIVEREF(__pyx_t_21);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
      }
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_ErrRestore(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_16;
      goto __pyx_L1_error;
    }
    __pyx_L4_return: {
      __pyx_t_22 = __pyx_r;
      __pyx_r = 0;

      /* "plyvel/_plyvel.pyx":446
 *             return [sizes[i] for i in xrange(n_ranges)]
 *         finally:
 *             free(c_ranges)             # <<<<<<<<<<<<<<
 *             free(sizes)
 * 
 */
      free(__pyx_v_c_ranges);

      /* "plyvel/_plyvel.pyx":447
 *         finally:
 *             free(c_ranges)
 *             free(sizes)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
 */
      free(__pyx_v_sizes);
      __pyx_r = __pyx_t_22;
      __pyx_t_22 = 0;
      goto __pyx_L0;
    }
  }

  /* "plyvel/_plyvel.pyx":424
 *         return self.approximate_sizes((start, stop))[0]
 * 
 *     def approximate_sizes(self, *ranges):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_stop);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":449
 *             free(sizes)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db(PyObject *__pyx_v_self, PyObject *__pyx_v_prefix); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_32prefixed_db[] = "DB.prefixed_db(self, bytes prefix)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db(PyObject *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefixed_db (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), ((PyObject*)__pyx_v_prefix));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("prefixed_db", 0);

  /* "plyvel/_plyvel.pyx":450
 * 
 *     def prefixed_db(self, bytes prefix not None):
 *         return PrefixedDB(db=self, prefix=prefix)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 450, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_prefix, __pyx_v_prefix) < 0) __PYX_ERR(0, 450, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_PrefixedDB), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":449
 *             free(sizes)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_35__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_34__reduce_cython__[] = "DB.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_35__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_34__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_34__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_37__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2DB_36__setstate_cython__[] = "DB.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_37__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_36__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_36__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":457
 *     cdef readonly bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_db,&__pyx_n_s_prefix,0};
    PyObject* values[2] = {0,0};
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
      else {
        __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_db); __PYX_ERR(0, 457, __pyx_L3_error)
      }
      if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_prefix)) != 0)) kw_args--;
      else {
        __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_prefix); __PYX_ERR(0, 457, __pyx_L3_error)
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "__init__") < 0)) __PYX_ERR(0, 457, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_db); __PYX_ERR(0, 457, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 457, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 457, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 457, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":458
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":459
 *     def __init__(self, *, DB db not None, bytes prefix not None):
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":457
 *     cdef readonly bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":461
 *         self.prefix = prefix
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "plyvel/_plyvel.pyx":462
 * 
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.prefix,
 *             hex(id(self)),
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":464
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
 *             hex(id(self)),             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_hex, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":463
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,             # <<<<<<<<<<<<<<
 *             hex(id(self)),
 *         )
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":462
 * 
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.prefix,
 *             hex(id(self)),
 */
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_plyvel_PrefixedDB_with_prefix_r, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":461
 *         self.prefix = prefix
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":467
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_4get[] = "PrefixedDB.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_default,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":468
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return self.db.get(
 *             self.prefix + key,
 */
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    values[3] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 2; index < 4 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get") < 0)) __PYX_ERR(0, 467, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[2]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 467, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 467, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 468, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 468, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":467
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("get", 0);

  /* "plyvel/_plyvel.pyx":469
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(             # <<<<<<<<<<<<<<
 *             self.prefix + key,
 *             default=default,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->db), __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "plyvel/_plyvel.pyx":470
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
 *             self.prefix + key,             # <<<<<<<<<<<<<<
 *             default=default,
 *             verify_checksums=verify_checksums,
 */
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":469
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(             # <<<<<<<<<<<<<<
 *             self.prefix + key,
 *             default=default,
 */
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":471
 *         return self.db.get(
 *             self.prefix + key,
 *             default=default,             # <<<<<<<<<<<<<<
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_default, __pyx_v_default) < 0) __PYX_ERR(0, 471, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":472
 *             self.prefix + key,
 *             default=default,
 *             verify_checksums=verify_checksums,             # <<<<<<<<<<<<<<
 *             fill_cache=fill_cache)
 * 
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_verify_checksums, ((PyObject *)__pyx_v_verify_checksums)) < 0) __PYX_ERR(0, 471, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":473
 *             default=default,
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache)             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None, *,
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_fill_cache, ((PyObject *)__pyx_v_fill_cache)) < 0) __PYX_ERR(0, 471, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":469
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(             # <<<<<<<<<<<<<<
 *             self.prefix + key,
 *             default=default,
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":467
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":475
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_6put[] = "PrefixedDB.put(self, bytes key, value, *, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,&__pyx_n_s_sync,0};
    PyObject* values[3] = {0,0,0};

    /* "plyvel/_plyvel.pyx":476
 * 
 *     def put(self, bytes key not None, value not None, *,
 *             bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 */
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, 1); __PYX_ERR(0, 475, __pyx_L3_error)
        }
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 2;
        PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "put") < 0)) __PYX_ERR(0, 475, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
    __pyx_v_sync = ((PyBoolObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 475, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 475, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 475, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 476, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_value, __pyx_v_sync);

  /* "plyvel/_plyvel.pyx":475
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyBoolObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("put", 0);

  /* "plyvel/_plyvel.pyx":477
 *     def put(self, bytes key not None, value not None, *,
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->db), __pyx_n_s_put); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_value);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_sync, ((PyObject *)__pyx_v_sync)) < 0) __PYX_ERR(0, 477, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":475
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":479
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_8delete[] = "PrefixedDB.delete(self, bytes key, *, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_sync,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 1;
        PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "delete") < 0)) __PYX_ERR(0, 479, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_sync = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 479, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 479, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 479, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyBoolObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "plyvel/_plyvel.pyx":480
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         return self.db.delete(self.prefix + key, sync=sync)             # <<<<<<<<<<<<<<
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->db), __pyx_n_s_delete); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_sync, ((PyObject *)__pyx_v_sync)) < 0) __PYX_ERR(0, 480, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":479
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":482
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_10write_batch[] = "PrefixedDB.write_batch(self, *, transaction=False, bool sync=False)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_transaction = 0;
  PyBoolObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_transaction,&__pyx_n_s_sync,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = (PyObject *)((PyBoolObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 0; index < 2 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "write_batch") < 0)) __PYX_ERR(0, 482, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_transaction = values[0];
    __pyx_v_sync = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_batch", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 482, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 482, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_transaction, PyBoolObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("write_batch", 0);

  /* "plyvel/_plyvel.pyx":483
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):
 *         return WriteBatch(self.db, self.prefix, transaction, sync)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->db));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->db));
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->prefix);
  __Pyx_INCREF(__pyx_v_transaction);
  __Pyx_GIVEREF(__pyx_v_transaction);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_transaction);
  __Pyx_INCREF(((PyObject *)__pyx_v_sync));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_sync));
  PyTuple_SET_ITEM(__pyx_t_1, 3, ((PyObject *)__pyx_v_sync));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_WriteBatch), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":482
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":485
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":486
 * 
 *     def __iter__(self):
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_iterator); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":485
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":488
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_14iterator[] = "PrefixedDB.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_reverse,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_include_start,&__pyx_n_s_include_stop,&__pyx_n_s_prefix,&__pyx_n_s_include_key,&__pyx_n_s_include_value,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":489
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 */
    values[3] = ((PyObject *)Py_True);
    values[4] = ((PyObject *)Py_False);
    values[5] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":490
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
 */
    values[6] = ((PyObject *)Py_True);
    values[7] = ((PyObject *)Py_True);

    /* "plyvel/_plyvel.pyx":491
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self.db,
 */
    values[8] = (PyObject *)((PyBoolObject *)Py_False);
    values[9] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 10)) {
        Py_ssize_t index;
        for (index = 0; index < 10 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "iterator") < 0)) __PYX_ERR(0, 488, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[8]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[9]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 488, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 491, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 491, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":488
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":492
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self.db,
 *             self.prefix,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":505
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = PyTuple_New(13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->db));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->db));
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->prefix);
  __Pyx_INCREF(__pyx_v_reverse);
  __Pyx_GIVEREF(__pyx_v_reverse);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_reverse);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_start);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_GIVEREF(__pyx_v_stop);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_stop);
  __Pyx_INCREF(__pyx_v_include_start);
  __Pyx_GIVEREF(__pyx_v_include_start);
  PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_v_include_start);
  __Pyx_INCREF(__pyx_v_include_stop);
  __Pyx_GIVEREF(__pyx_v_include_stop);
  PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_v_include_stop);
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_v_prefix);
  __Pyx_INCREF(__pyx_v_include_key);
  __Pyx_GIVEREF(__pyx_v_include_key);
  PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_v_include_key);
  __Pyx_INCREF(__pyx_v_include_value);
  __Pyx_GIVEREF(__pyx_v_include_value);
  PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_v_include_value);
  __Pyx_INCREF(((PyObject *)__pyx_v_verify_checksums));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_verify_checksums));
  PyTuple_SET_ITEM(__pyx_t_1, 10, ((PyObject *)__pyx_v_verify_checksums));
  __Pyx_INCREF(((PyObject *)__pyx_v_fill_cache));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_fill_cache));
  PyTuple_SET_ITEM(__pyx_t_1, 11, ((PyObject *)__pyx_v_fill_cache));
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 12, Py_None);

  /* "plyvel/_plyvel.pyx":492
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self.db,
 *             self.prefix,
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":488
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":508
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_16snapshot[] = "PrefixedDB.snapshot(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("snapshot (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("snapshot", 0);

  /* "plyvel/_plyvel.pyx":509
 * 
 *     def snapshot(self):
 *         return Snapshot(db=self.db, prefix=self.prefix)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self->db)) < 0) __PYX_ERR(0, 509, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_prefix, __pyx_v_self->prefix) < 0) __PYX_ERR(0, 509, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Snapshot), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":508
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.snapshot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":511
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db(PyObject *__pyx_v_self, PyObject *__pyx_v_prefix); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db[] = "PrefixedDB.prefixed_db(self, bytes prefix)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db(PyObject *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefixed_db (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 511, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), ((PyObject*)__pyx_v_prefix));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("prefixed_db", 0);

  /* "plyvel/_plyvel.pyx":512
 * 
 *     def prefixed_db(self, bytes prefix not None):
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_db, ((PyObject *)__pyx_v_self->db)) < 0) __PYX_ERR(0, 512, __pyx_L1_error)
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_prefix); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_prefix, __pyx_t_2) < 0) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_PrefixedDB), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":511
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":454
 * 
 * cdef class PrefixedDB:
 *     cdef readonly DB db             # <<<<<<<<<<<<<<
 *     cdef readonly bytes prefix
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db));
  __pyx_r = ((PyObject *)__pyx_v_self->db);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":455
 * cdef class PrefixedDB:
 *     cdef readonly DB db
 *     cdef readonly bytes prefix             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __pyx_r = __pyx_v_self->prefix;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__[] = "PrefixedDB.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->db));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->db));
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->prefix);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.db is not None or self.prefix is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.db is not None or self.prefix is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, None), state
 */
  /*else*/ {
    __pyx_t_2 = (((PyObject *)__pyx_v_self->db) != Py_None);
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
    __pyx_t_2 = (__pyx_t_5 != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.db is not None or self.prefix is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = self.db is not None or self.prefix is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_PrefixedDB); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_18786289);
    __Pyx_GIVEREF(__pyx_int_18786289);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_18786289);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.db is not None or self.prefix is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, None), state
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_PrefixedDB); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_18786289);
    __Pyx_GIVEREF(__pyx_int_18786289);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_18786289);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__[] = "PrefixedDB.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0x11ea7f1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":515
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_1repair_db(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_repair_db[] = "repair_db(name, *, paranoid_checks=None, write_buffer_size=None, max_open_files=None, lru_cache_size=None, block_size=None, block_restart_interval=None, max_file_size=None, compression='snappy', int bloom_filter_bits=0, comparator=None, bytes comparator_name=None)";
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_1repair_db = {"repair_db", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_1repair_db, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_repair_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_1repair_db(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_paranoid_checks = 0;
  PyObject *__pyx_v_write_buffer_size = 0;
  PyObject *__pyx_v_max_open_files = 0;
  PyObject *__pyx_v_lru_cache_size = 0;
  PyObject *__pyx_v_block_size = 0;
  PyObject *__pyx_v_block_restart_interval = 0;
  PyObject *__pyx_v_max_file_size = 0;
  PyObject *__pyx_v_compression = 0;
  int __pyx_v_bloom_filter_bits;
  PyObject *__pyx_v_comparator = 0;
  PyObject *__pyx_v_comparator_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("repair_db (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_paranoid_checks,&__pyx_n_s_write_buffer_size,&__pyx_n_s_max_open_files,&__pyx_n_s_lru_cache_size,&__pyx_n_s_block_size,&__pyx_n_s_block_restart_interval,&__pyx_n_s_max_file_size,&__pyx_n_s_compression,&__pyx_n_s_bloom_filter_bits,&__pyx_n_s_comparator,&__pyx_n_s_comparator_name,0};
    PyObject* values[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":516
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 */
    values[3] = ((PyObject *)Py_None);
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":517
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
 */
    values[6] = ((PyObject *)Py_None);
    values[7] = ((PyObject *)Py_None);
    values[8] = ((PyObject *)__pyx_n_s_snappy);

    /* "plyvel/_plyvel.pyx":518
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,             # <<<<<<<<<<<<<<
 *               bytes comparator_name=None):
 *     cdef Options options = Options()
 */
    values[10] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":519
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
 */
    values[11] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args > 0 && likely(kw_args <= 11)) {
        Py_ssize_t index;
        for (index = 1; index < 12 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "repair_db") < 0)) __PYX_ERR(0, 515, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
    __pyx_v_paranoid_checks = values[1];
    __pyx_v_write_buffer_size = values[2];
    __pyx_v_max_open_files = values[3];
    __pyx_v_lru_cache_size = values[4];
    __pyx_v_block_size = values[5];
    __pyx_v_block_restart_interval = values[6];
    __pyx_v_max_file_size = values[7];
    __pyx_v_compression = values[8];
    if (values[9]) {
      __pyx_v_bloom_filter_bits = __Pyx_PyInt_As_int(values[9]); if (unlikely((__pyx_v_bloom_filter_bits == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 518, __pyx_L3_error)
    } else {
      __pyx_v_bloom_filter_bits = ((int)0);
    }
    __pyx_v_comparator = values[10];
    __pyx_v_comparator_name = ((PyObject*)values[11]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("repair_db", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 515, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.repair_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_comparator_name), (&PyBytes_Type), 1, "comparator_name", 1))) __PYX_ERR(0, 519, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_repair_db(__pyx_self, __pyx_v_name, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name);

  /* "plyvel/_plyvel.pyx":515
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_repair_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  leveldb::Options __pyx_v_options;
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  int __pyx_v_create_if_missing;
  int __pyx_v_error_if_exists;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("repair_db", 0);

  /* "plyvel/_plyvel.pyx":520
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
 *     cdef Options options = Options()             # <<<<<<<<<<<<<<
 *     cdef Status st
 *     cdef string fsname
 */
  __pyx_v_options = leveldb::Options();

  /* "plyvel/_plyvel.pyx":524
 *     cdef string fsname
 * 
 *     fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *     create_if_missing = False
 *     error_if_exists = True
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __pyx_t_2;

  /* "plyvel/_plyvel.pyx":525
 * 
 *     fsname = to_file_system_name(name)
 *     create_if_missing = False             # <<<<<<<<<<<<<<
 *     error_if_exists = True
 *     parse_options(
 */
  __pyx_v_create_if_missing = 0;

  /* "plyvel/_plyvel.pyx":526
 *     fsname = to_file_system_name(name)
 *     create_if_missing = False
 *     error_if_exists = True             # <<<<<<<<<<<<<<
 *     parse_options(
 *         &options, create_if_missing, error_if_exists, paranoid_checks,
 */
  __pyx_v_error_if_exists = 1;

  /* "plyvel/_plyvel.pyx":527
 *     create_if_missing = False
 *     error_if_exists = True
 *     parse_options(             # <<<<<<<<<<<<<<
 *         &options, create_if_missing, error_if_exists, paranoid_checks,
 *         write_buffer_size, max_open_files, lru_cache_size, block_size,
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_parse_options((&__pyx_v_options), __pyx_v_create_if_missing, __pyx_v_error_if_exists, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 527, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":532
 *         block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *         comparator, comparator_name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":533
 *         comparator, comparator_name)
 *     with nogil:
 *         st = RepairDB(fsname, options)             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
 */
        __pyx_v_st = leveldb::RepairDB(__pyx_v_fsname, __pyx_v_options);
      }

      /* "plyvel/_plyvel.pyx":532
 *         block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *         comparator, comparator_name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":534
 *     with nogil:
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 534, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":515
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.repair_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":537
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_3destroy_db(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_2destroy_db[] = "destroy_db(name)";
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_3destroy_db = {"destroy_db", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_3destroy_db, METH_O, __pyx_doc_6plyvel_7_plyvel_2destroy_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_3destroy_db(PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("destroy_db (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2destroy_db(__pyx_self, ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2destroy_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  leveldb::Options __pyx_v_options;
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("destroy_db", 0);

  /* "plyvel/_plyvel.pyx":538
 * 
 * def destroy_db(name):
 *     cdef Options options = Options()             # <<<<<<<<<<<<<<
 *     cdef Status st
 *     cdef string fsname
 */
  __pyx_v_options = leveldb::Options();

  /* "plyvel/_plyvel.pyx":542
 *     cdef string fsname
 * 
 *     fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         st = DestroyDB(fsname, options)
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __pyx_t_2;

  /* "plyvel/_plyvel.pyx":543
 * 
 *     fsname = to_file_system_name(name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":544
 *     fsname = to_file_system_name(name)
 *     with nogil:
 *         st = DestroyDB(fsname, options)             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
 */
        __pyx_v_st = leveldb::DestroyDB(__pyx_v_fsname, __pyx_v_options);
      }

      /* "plyvel/_plyvel.pyx":543
 * 
 *     fsname = to_file_system_name(name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":545
 *     with nogil:
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 545, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":537
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.destroy_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":560
 *     cdef c_bool transaction
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  PyBoolObject *__pyx_v_transaction = 0;
  PyObject *__pyx_v_sync = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_db,&__pyx_n_s_prefix,&__pyx_n_s_transaction,&__pyx_n_s_sync,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_prefix)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 1); __PYX_ERR(0, 560, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_transaction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 2); __PYX_ERR(0, 560, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sync)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 3); __PYX_ERR(0, 560, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 560, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
    __pyx_v_transaction = ((PyBoolObject *)values[2]);
    __pyx_v_sync = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 560, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 560, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 560, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transaction), __pyx_ptype_7cpython_4bool_bool, 1, "transaction", 0))) __PYX_ERR(0, 560, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix, __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix, PyBoolObject *__pyx_v_transaction, PyObject *__pyx_v_sync) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  bool __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":561
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 *         self.transaction = transaction
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":562
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 *         self.transaction = transaction
 * 
 */
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":563
 *         self.db = db
 *         self.prefix = prefix
 *         self.transaction = transaction             # <<<<<<<<<<<<<<
 * 
 *         self.write_options = WriteOptions()
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_transaction)); if (unlikely((__pyx_t_1 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 563, __pyx_L1_error)
  __pyx_v_self->transaction = __pyx_t_1;

  /* "plyvel/_plyvel.pyx":565
 *         self.transaction = transaction
 * 
 *         self.write_options = WriteOptions()             # <<<<<<<<<<<<<<
 *         if sync is not None:
 *             self.write_options.sync = sync
 */
  __pyx_v_self->write_options = leveldb::WriteOptions();

  /* "plyvel/_plyvel.pyx":566
 * 
 *         self.write_options = WriteOptions()
 *         if sync is not None:             # <<<<<<<<<<<<<<
 *             self.write_options.sync = sync
 * 
 */
  __pyx_t_2 = (__pyx_v_sync != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":567
 *         self.write_options = WriteOptions()
 *         if sync is not None:
 *             self.write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         self._write_batch = new leveldb.WriteBatch()
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_sync); if (unlikely((__pyx_t_1 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 567, __pyx_L1_error)
    __pyx_v_self->write_options.sync = __pyx_t_1;

    /* "plyvel/_plyvel.pyx":566
 * 
 *         self.write_options = WriteOptions()
 *         if sync is not None:             # <<<<<<<<<<<<<<
 *             self.write_options.sync = sync
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":569
 *             self.write_options.sync = sync
 * 
 *         self._write_batch = new leveldb.WriteBatch()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->_write_batch = new leveldb::WriteBatch();

  /* "plyvel/_plyvel.pyx":560
 *     cdef c_bool transaction
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":571
 *         self._write_batch = new leveldb.WriteBatch()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         del self._write_batch
 * 
 */

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":572
 * 
 *     def __dealloc__(self):
 *         del self._write_batch             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None):
 */
  delete __pyx_v_self->_write_batch;

  /* "plyvel/_plyvel.pyx":571
 *         self._write_batch = new leveldb.WriteBatch()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         del self._write_batch
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":574
 *         del self._write_batch
 * 
 *     def put(self, bytes key not None, value not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_4put[] = "WriteBatch.put(self, bytes key, value)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, 1); __PYX_ERR(0, 574, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "put") < 0)) __PYX_ERR(0, 574, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 574, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 574, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 574, __pyx_L1_error)
  }
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_key, __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  leveldb::Slice __pyx_v_key_slice;
  Py_buffer __pyx_v_value_buffer;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("put", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":575
 * 
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":576
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 576, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":575
 * 
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":578
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":579
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 */
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":578
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":581
 *             key = self.prefix + key
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 */
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L1_error)
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 581, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

  /* "plyvel/_plyvel.pyx":583
 *         cdef Slice key_slice = Slice(key, len(key))
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)             # <<<<<<<<<<<<<<
 *         try:
 *             with nogil:
 */
  __pyx_t_6 = PyObject_GetBuffer(__pyx_v_value, (&__pyx_v_value_buffer), PyBUF_SIMPLE); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 583, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":584
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._write_batch.Put(
 */
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":585
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._write_batch.Put(
 *                     key_slice,
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":586
 *         try:
 *             with nogil:
 *                 self._write_batch.Put(             # <<<<<<<<<<<<<<
 *                     key_slice,
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
 */
          __pyx_v_self->_write_batch->Put(__pyx_v_key_slice, leveldb::Slice(((const char *)__pyx_v_value_buffer.buf), __pyx_v_value_buffer.len));
        }

        /* "plyvel/_plyvel.pyx":585
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._write_batch.Put(
 *                     key_slice,
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L10;
          }
          __pyx_L10:;
        }
    }
  }

  /* "plyvel/_plyvel.pyx":590
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
 *         finally:
 *             PyBuffer_Release(&value_buffer)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None):
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyBuffer_Release((&__pyx_v_value_buffer));
      goto __pyx_L7;
    }
    __pyx_L7:;
  }

  /* "plyvel/_plyvel.pyx":574
 *         del self._write_batch
 * 
 *     def put(self, bytes key not None, value not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":592
 *             PyBuffer_Release(&value_buffer)
 * 
 *     def delete(self, bytes key not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_6delete[] = "WriteBatch.delete(self, bytes key)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 592, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), ((PyObject*)__pyx_v_key));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("delete", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":593
 * 
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":594
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 594, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":593
 * 
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":596
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":597
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 */
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":596
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":599
 *             key = self.prefix + key
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._write_batch.Delete(key_slice)
 */
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 599, __pyx_L1_error)
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 599, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

  /* "plyvel/_plyvel.pyx":600
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Delete(key_slice)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":601
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:
 *             self._write_batch.Delete(key_slice)             # <<<<<<<<<<<<<<
 * 
 *     def clear(self):
 */
        __pyx_v_self->_write_batch->Delete(__pyx_v_key_slice);
      }

      /* "plyvel/_plyvel.pyx":600
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Delete(key_slice)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":592
 *             PyBuffer_Release(&value_buffer)
 * 
 *     def delete(self, bytes key not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":603
 *             self._write_batch.Delete(key_slice)
 * 
 *     def clear(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_8clear[] = "WriteBatch.clear(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clear (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("clear", 0);

  /* "plyvel/_plyvel.pyx":604
 * 
 *     def clear(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":605
 *     def clear(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 605, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":604
 * 
 *     def clear(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":607
 *             raise RuntimeError("Database is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Clear()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":608
 * 
 *         with nogil:
 *             self._write_batch.Clear()             # <<<<<<<<<<<<<<
 * 
 *     def write(self):
 */
        __pyx_v_self->_write_batch->Clear();
      }

      /* "plyvel/_plyvel.pyx":607
 *             raise RuntimeError("Database is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Clear()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":603
 *             self._write_batch.Delete(key_slice)
 * 
 *     def clear(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.clear", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":610
 *             self._write_batch.Clear()
 * 
 *     def write(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_10write[] = "WriteBatch.write(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  leveldb::Status __pyx_v_st;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("write", 0);

  /* "plyvel/_plyvel.pyx":611
 * 
 *     def write(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":612
 *     def write(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 612, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 612, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":611
 * 
 *     def write(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":615
 * 
 *         cdef Status st
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":616
 *         cdef Status st
 *         with nogil:
 *             st = self.db._db.Write(self.write_options, self._write_batch)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
 */
        __pyx_v_st = __pyx_v_self->db->_db->Write(__pyx_v_self->write_options, __pyx_v_self->_write_batch);
      }

      /* "plyvel/_plyvel.pyx":615
 * 
 *         cdef Status st
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":617
 *         with nogil:
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 617, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":610
 *             self._write_batch.Clear()
 * 
 *     def write(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":619
 *         raise_for_status(st)
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_12__enter__[] = "WriteBatch.__enter__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_12__enter__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_12__enter__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":620
 * 
 *     def __enter__(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":621
 *     def __enter__(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 621, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 621, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":620
 * 
 *     def __enter__(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":623
 *             raise RuntimeError("Database is closed")
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":619
 *         raise_for_status(st)
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__enter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":625
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_14__exit__[] = "WriteBatch.__exit__(self, exc_type, exc_val, exc_tb)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exc_type,&__pyx_n_s_exc_val,&__pyx_n_s_exc_tb,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 1); __PYX_ERR(0, 625, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 2); __PYX_ERR(0, 625, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) __PYX_ERR(0, 625, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 625, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_14__exit__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_14__exit__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":626
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":627
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.transaction and exc_type is not None:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 627, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":626
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":629
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.transaction and exc_type is not None:             # <<<<<<<<<<<<<<
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
 */
  __pyx_t_3 = (__pyx_v_self->transaction != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_exc_type != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":631
 *         if self.transaction and exc_type is not None:
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()             # <<<<<<<<<<<<<<
 *             return
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clear); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 631, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 631, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":632
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self.write()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":629
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.transaction and exc_type is not None:             # <<<<<<<<<<<<<<
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
 */
  }

  /* "plyvel/_plyvel.pyx":634
 *             return
 * 
 *         self.write()             # <<<<<<<<<<<<<<
 *         self.clear()
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":635
 * 
 *         self.write()
 *         self.clear()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clear); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 635, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 635, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":625
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_16__reduce_cython__[] = "WriteBatch.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_10WriteBatch_18__setstate_cython__[] = "WriteBatch.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":662
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_db,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,&__pyx_n_s_snapshot,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_verify_checksums)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 1); __PYX_ERR(0, 662, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fill_cache)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 2); __PYX_ERR(0, 662, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_snapshot)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 3); __PYX_ERR(0, 662, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 662, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_verify_checksums = ((PyBoolObject *)values[1]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[2]);
    __pyx_v_snapshot = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 662, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_6plyvel_7_plyvel_DB, 1, "db", 0))) __PYX_ERR(0, 662, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 662, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 662, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_snapshot), __pyx_ptype_6plyvel_7_plyvel_Snapshot, 1, "snapshot", 0))) __PYX_ERR(0, 663, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), __pyx_v_db, __pyx_v_verify_checksums, __pyx_v_fill_cache, __pyx_v_snapshot);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":683
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_wr); /*proto*/
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_8__init___lambda = {"lambda", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda, METH_O, 0};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_wr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_wr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_wr) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_cur_scope;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lambda", 0);
  __pyx_outer_scope = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_cur_scope->__pyx_v_ref_dict)) { __Pyx_RaiseClosureNameError("ref_dict"); __PYX_ERR(0, 683, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_ref_dict == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
    __PYX_ERR(0, 683, __pyx_L1_error)
  }
  if (unlikely(!__pyx_cur_scope->__pyx_v_iterator_id)) { __Pyx_RaiseClosureNameError("iterator_id"); __PYX_ERR(0, 683, __pyx_L1_error) }
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_cur_scope->__pyx_v_ref_dict, __pyx_cur_scope->__pyx_v_iterator_id, ((PyObject *)NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":662
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 */

static int __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_cur_scope;
  leveldb::ReadOptions __pyx_v_read_options;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  bool __pyx_t_3;
  int __pyx_t_4;
  leveldb::Snapshot *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);
  __pyx_cur_scope = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 662, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }

  /* "plyvel/_plyvel.pyx":664
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,
 *                  Snapshot snapshot):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":665
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 665, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":664
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,
 *                  Snapshot snapshot):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":667
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.db = db             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":670
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 670, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":671
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 *         if snapshot is not None:
 *             read_options.snapshot = snapshot._snapshot
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 671, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_3;

  /* "plyvel/_plyvel.pyx":672
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:             # <<<<<<<<<<<<<<
 *             read_options.snapshot = snapshot._snapshot
 * 
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_snapshot) != Py_None);
  __pyx_t_4 = (__pyx_t_1 != 0);
  if (__pyx_t_4) {

    /* "plyvel/_plyvel.pyx":673
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:
 *             read_options.snapshot = snapshot._snapshot             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_5 = __pyx_v_snapshot->_snapshot;
    __pyx_v_read_options.snapshot = __pyx_t_5;

    /* "plyvel/_plyvel.pyx":672
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:             # <<<<<<<<<<<<<<
 *             read_options.snapshot = snapshot._snapshot
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":675
 *             read_options.snapshot = snapshot._snapshot
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter = db._db.NewIterator(read_options)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":676
 * 
 *         with nogil:
 *             self._iter = db._db.NewIterator(read_options)             # <<<<<<<<<<<<<<
 * 
 *         # Store a weak reference on the db (needed when closing db)
 */
        __pyx_v_self->_iter = __pyx_v_db->_db->NewIterator(__pyx_v_read_options);
      }

      /* "plyvel/_plyvel.pyx":675
 *             read_options.snapshot = snapshot._snapshot
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter = db._db.NewIterator(read_options)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":679
 * 
 *         # Store a weak reference on the db (needed when closing db)
 *         iterator_id = id(self)             # <<<<<<<<<<<<<<
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_iterator_id = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":680
 *         # Store a weak reference on the db (needed when closing db)
 *         iterator_id = id(self)
 *         ref_dict = db.iterators             # <<<<<<<<<<<<<<
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
 */
  __pyx_t_2 = __pyx_v_db->iterators;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_ref_dict = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":681
 *         iterator_id = id(self)
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(             # <<<<<<<<<<<<<<
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_weakref_ref); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "plyvel/_plyvel.pyx":683
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
 */
  __pyx_t_7 = __Pyx_CyFunction_NewEx(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_8__init___lambda, 0, __pyx_n_s_init___locals_lambda, ((PyObject*)__pyx_cur_scope), __pyx_n_s_plyvel__plyvel, __pyx_d, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_9 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, ((PyObject *)__pyx_v_self), __pyx_t_7};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, ((PyObject *)__pyx_v_self), __pyx_t_7};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, ((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "plyvel/_plyvel.pyx":681
 *         iterator_id = id(self)
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(             # <<<<<<<<<<<<<<
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))
 */
  if (unlikely(__pyx_cur_scope->__pyx_v_ref_dict == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 681, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_ref_dict, __pyx_cur_scope->__pyx_v_iterator_id, __pyx_t_2) < 0)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":662
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":685
 *             lambda wr: ref_dict.pop(iterator_id))
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self._iter is not NULL:
 *             del self._iter
 */

static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_12BaseIterator_close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "plyvel/_plyvel.pyx":686
 * 
 *     cpdef close(self):
 *         if self._iter is not NULL:             # <<<<<<<<<<<<<<
 *             del self._iter
 *             self._iter = NULL
 */
  __pyx_t_5 = ((__pyx_v_self->_iter != NULL) != 0);
  if (__pyx_t_5) {

    /* "plyvel/_plyvel.pyx":687
 *     cpdef close(self):
 *         if self._iter is not NULL:
 *             del self._iter             # <<<<<<<<<<<<<<
 *             self._iter = NULL
 * 
 */
    delete __pyx_v_self->_iter;

    /* "plyvel/_plyvel.pyx":688
 *         if self._iter is not NULL:
 *             del self._iter
 *             self._iter = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
    __pyx_v_self->_iter = NULL;

    /* "plyvel/_plyvel.pyx":686
 * 
 *     cpdef close(self):
 *         if self._iter is not NULL:             # <<<<<<<<<<<<<<
 *             del self._iter
 *             self._iter = NULL
 */
  }

  /* "plyvel/_plyvel.pyx":685
 *             lambda wr: ref_dict.pop(iterator_id))
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self._iter is not NULL:
 *             del self._iter
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_12BaseIterator_2close[] = "BaseIterator.close(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_12BaseIterator_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":690
 *             self._iter = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":691
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self->__pyx_vtab)->close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 691, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":690
 *             self._iter = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.BaseIterator.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":693
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_12BaseIterator_6__enter__[] = "BaseIterator.__enter__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":694
 * 
 *     def __enter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":693
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":696
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_12BaseIterator_8__exit__[] = "BaseIterator.__exit__(self, exc_type, exc_val, exc_tb)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exc_type,&__pyx_n_s_exc_val,&__pyx_n_s_exc_tb,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 1); __PYX_ERR(0, 696, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 2); __PYX_ERR(0, 696, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) __PYX_ERR(0, 696, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 696, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":697
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 *         return False  # propagate exceptions
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self->__pyx_vtab)->close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":698
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()
 *         return False  # propagate exceptions             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":696
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__[] = "BaseIterator.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__[] = "BaseIterator.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":718
 *     cdef size_t db_prefix_len
 * 
 *     def __init__(self, DB db, bytes db_prefix, bool reverse, bytes start,             # <<<<<<<<<<<<<<
 *                  bytes stop, bool include_start, bool include_stop,
 *                  bytes prefix, bool include_key, bool include_value,
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_db_prefix = 0;
  PyBoolObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyBoolObject *__pyx_v_include_start = 0;
  PyBoolObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyBoolObject *__pyx_v_include_key = 0;
  PyBoolObject *__pyx_v_include_value = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_db,&__pyx_n_s_db_prefix,&__pyx_n_s_reverse,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_include_start,&__pyx_n_s_include_stop,&__pyx_n_s_prefix,&__pyx_n_s_include_key,&__pyx_n_s_include_value,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,&__pyx_n_s_snapshot,0};
    PyObject* values[13] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 13: values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
        CYTHON_FALLTHROUGH;
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db_prefix)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 1); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reverse)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 2); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 3); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 4); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_start)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 5); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_stop)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 6); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_prefix)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 7); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_key)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 8); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 9); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_verify_checksums)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 10); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (likely((values[11] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fill_cache)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 11); __PYX_ERR(0, 718, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 12:
        if (likely((values[12] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_snapshot)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, 12); __PYX_ERR(0, 718, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 718, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 13) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
      values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
      values[12] = PyTuple_GET_ITEM(__pyx_args, 12);
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_db_prefix = ((PyObject*)values[1]);
    __pyx_v_reverse = ((PyBoolObject *)values[2]);
    __pyx_v_start = ((PyObject*)values[3]);
    __pyx_v_stop = ((PyObject*)values[4]);
    __pyx_v_include_start = ((PyBoolObject *)values[5]);
    __pyx_v_include_stop = ((PyBoolObject *)values[6]);
    __pyx_v_prefix = ((PyObject*)values[7]);
    __pyx_v_include_key = ((PyBoolObject *)values[8]);
    __pyx_v_include_value = ((PyBoolObject *)values[9]);
    __pyx_v_verify_checksums = ((PyBoolObject *)values[10]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[11]);
    __pyx_v_snapshot = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)values[12]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 718, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_6plyvel_7_plyvel_DB, 1, "db", 0))) __PYX_ERR(0, 718, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db_prefix), (&PyBytes_Type), 1, "db_prefix", 1))) __PYX_ERR(0, 718, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_reverse), __pyx_ptype_7cpython_4bool_bool, 1, "reverse", 0))) __PYX_ERR(0, 718, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 1, "start", 1))) __PYX_ERR(0, 718, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 1, "stop", 1))) __PYX_ERR(0, 719, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_start), __pyx_ptype_7cpython_4bool_bool, 1, "include_start", 0))) __PYX_ERR(0, 719, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_stop), __pyx_ptype_7cpython_4bool_bool, 1, "include_stop", 0))) __PYX_ERR(0, 719, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 720, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_key), __pyx_ptype_7cpython_4bool_bool, 1, "include_key", 0))) __PYX_ERR(0, 720, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_value), __pyx_ptype_7cpython_4bool_bool, 1, "include_value", 0))) __PYX_ERR(0, 720, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 721, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 721, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_snapshot), __pyx_ptype_6plyvel_7_plyvel_Snapshot, 1, "snapshot", 0))) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), __pyx_v_db, __pyx_v_db_prefix, __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache, __pyx_v_snapshot);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_db_prefix, PyBoolObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyBoolObject *__pyx_v_include_start, PyBoolObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyBoolObject *__pyx_v_include_key, PyBoolObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  enum __pyx_t_6plyvel_7_plyvel_IteratorDirection __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  const char *__pyx_t_9;
  bool __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_INCREF((PyObject *)__pyx_v_include_start);
  __Pyx_INCREF((PyObject *)__pyx_v_include_stop);
  __Pyx_INCREF(__pyx_v_prefix);

  /* "plyvel/_plyvel.pyx":723
 *                  bool verify_checksums, bool fill_cache, Snapshot snapshot):
 * 
 *         super(Iterator, self).__init__(             # <<<<<<<<<<<<<<
 *             db=db,
 *             verify_checksums=verify_checksums,
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":724
 * 
 *         super(Iterator, self).__init__(
 *             db=db,             # <<<<<<<<<<<<<<
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_db, ((PyObject *)__pyx_v_db)) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":725
 *         super(Iterator, self).__init__(
 *             db=db,
 *             verify_checksums=verify_checksums,             # <<<<<<<<<<<<<<
 *             fill_cache=fill_cache,
 *             snapshot=snapshot)
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_verify_checksums, ((PyObject *)__pyx_v_verify_checksums)) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":726
 *             db=db,
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,             # <<<<<<<<<<<<<<
 *             snapshot=snapshot)
 * 
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_fill_cache, ((PyObject *)__pyx_v_fill_cache)) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":727
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 *             snapshot=snapshot)             # <<<<<<<<<<<<<<
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_snapshot, ((PyObject *)__pyx_v_snapshot)) < 0) __PYX_ERR(0, 724, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":723
 *                  bool verify_checksums, bool fill_cache, Snapshot snapshot):
 * 
 *         super(Iterator, self).__init__(             # <<<<<<<<<<<<<<
 *             db=db,
 *             verify_checksums=verify_checksums,
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "plyvel/_plyvel.pyx":729
 *             snapshot=snapshot)
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator             # <<<<<<<<<<<<<<
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
 */
  __pyx_v_self->comparator = ((leveldb::Comparator *)__pyx_v_db->options.comparator);

  /* "plyvel/_plyvel.pyx":730
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator
 *         self.direction = FORWARD if not reverse else REVERSE             # <<<<<<<<<<<<<<
 * 
 *         if db_prefix is None:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_reverse)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 730, __pyx_L1_error)
  if (((!__pyx_t_5) != 0)) {
    __pyx_t_4 = __pyx_e_6plyvel_7_plyvel_FORWARD;
  } else {
    __pyx_t_4 = __pyx_e_6plyvel_7_plyvel_REVERSE;
  }
  __pyx_v_self->direction = __pyx_t_4;

  /* "plyvel/_plyvel.pyx":732
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
 *         if db_prefix is None:             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = 0
 *         else:
 */
  __pyx_t_5 = (__pyx_v_db_prefix == ((PyObject*)Py_None));
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "plyvel/_plyvel.pyx":733
 * 
 *         if db_prefix is None:
 *             self.db_prefix_len = 0             # <<<<<<<<<<<<<<
 *         else:
 *             # This is an iterator on a PrefixedDB.
 */
    __pyx_v_self->db_prefix_len = 0;

    /* "plyvel/_plyvel.pyx":732
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
 *         if db_prefix is None:             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = 0
 *         else:
 */
    goto __pyx_L3;
  }

  /* "plyvel/_plyvel.pyx":736
 *         else:
 *             # This is an iterator on a PrefixedDB.
 *             self.db_prefix = db_prefix             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = len(db_prefix)
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_db_prefix);
    __Pyx_GIVEREF(__pyx_v_db_prefix);
    __Pyx_GOTREF(__pyx_v_self->db_prefix);
    __Pyx_DECREF(__pyx_v_self->db_prefix);
    __pyx_v_self->db_prefix = __pyx_v_db_prefix;

    /* "plyvel/_plyvel.pyx":737
 *             # This is an iterator on a PrefixedDB.
 *             self.db_prefix = db_prefix
 *             self.db_prefix_len = len(db_prefix)             # <<<<<<<<<<<<<<
 * 
 *             # Transform args so that the database key prefix is taken
 */
    if (unlikely(__pyx_v_db_prefix == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 737, __pyx_L1_error)
    }
    __pyx_t_7 = PyBytes_GET_SIZE(__pyx_v_db_prefix); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 737, __pyx_L1_error)
    __pyx_v_self->db_prefix_len = __pyx_t_7;

    /* "plyvel/_plyvel.pyx":741
 *             # Transform args so that the database key prefix is taken
 *             # into account.
 *             if prefix is not None:             # <<<<<<<<<<<<<<
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix
 */
    __pyx_t_6 = (__pyx_v_prefix != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_6 != 0);
    if (__pyx_t_5) {

      /* "plyvel/_plyvel.pyx":743
 *             if prefix is not None:
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix             # <<<<<<<<<<<<<<
 *             else:
 *                 # Adapt start and stop keys to use the database key
 */
      __pyx_t_3 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 743, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_prefix, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "plyvel/_plyvel.pyx":741
 *             # Transform args so that the database key prefix is taken
 *             # into account.
 *             if prefix is not None:             # <<<<<<<<<<<<<<
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix
 */
      goto __pyx_L4;
    }

    /* "plyvel/_plyvel.pyx":747
 *                 # Adapt start and stop keys to use the database key
 *                 # prefix.
 *                 if start is None:             # <<<<<<<<<<<<<<
 *                     start = db_prefix
 *                     include_start = True
 */
    /*else*/ {
      __pyx_t_5 = (__pyx_v_start == ((PyObject*)Py_None));
      __pyx_t_6 = (__pyx_t_5 != 0);
      if (__pyx_t_6) {

        /* "plyvel/_plyvel.pyx":748
 *                 # prefix.
 *                 if start is None:
 *                     start = db_prefix             # <<<<<<<<<<<<<<
 *                     include_start = True
 *                 else:
 */
        __Pyx_INCREF(__pyx_v_db_prefix);
        __Pyx_DECREF_SET(__pyx_v_start, __pyx_v_db_prefix);

        /* "plyvel/_plyvel.pyx":749
 *                 if start is None:
 *                     start = db_prefix
 *                     include_start = True             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = db_prefix + start
 */
        __Pyx_INCREF(Py_True);
        __Pyx_DECREF_SET(__pyx_v_include_start, ((PyBoolObject *)Py_True));

        /* "plyvel/_plyvel.pyx":747
 *                 # Adapt start and stop keys to use the database key
 *                 # prefix.
 *                 if start is None:             # <<<<<<<<<<<<<<
 *                     start = db_prefix
 *                     include_start = True
 */
        goto __pyx_L5;
      }

      /* "plyvel/_plyvel.pyx":751
 *                     include_start = True
 *                 else:
 *                     start = db_prefix + start             # <<<<<<<<<<<<<<
 * 
 *                 if stop is None:
 */
      /*else*/ {
        __pyx_t_3 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 751, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_start, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;
      }
      __pyx_L5:;

      /* "plyvel/_plyvel.pyx":753
 *                     start = db_prefix + start
 * 
 *                 if stop is None:             # <<<<<<<<<<<<<<
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False
 */
      __pyx_t_6 = (__pyx_v_stop == ((PyObject*)Py_None));
      __pyx_t_5 = (__pyx_t_6 != 0);
      if (__pyx_t_5) {

        /* "plyvel/_plyvel.pyx":754
 * 
 *                 if stop is None:
 *                     stop = bytes_increment(db_prefix)             # <<<<<<<<<<<<<<
 *                     include_stop = False
 *                 else:
 */
        __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_bytes_increment(__pyx_v_db_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;

        /* "plyvel/_plyvel.pyx":755
 *                 if stop is None:
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False             # <<<<<<<<<<<<<<
 *                 else:
 *                     stop = db_prefix + stop
 */
        __Pyx_INCREF(Py_False);
        __Pyx_DECREF_SET(__pyx_v_include_stop, ((PyBoolObject *)Py_False));

        /* "plyvel/_plyvel.pyx":753
 *                     start = db_prefix + start
 * 
 *                 if stop is None:             # <<<<<<<<<<<<<<
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False
 */
        goto __pyx_L6;
      }

      /* "plyvel/_plyvel.pyx":757
 *                     include_stop = False
 *                 else:
 *                     stop = db_prefix + stop             # <<<<<<<<<<<<<<
 * 
 *         if prefix is not None:
 */
      /*else*/ {
        __pyx_t_3 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_stop); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 757, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;
      }
      __pyx_L6:;
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "plyvel/_plyvel.pyx":759
 *                     stop = db_prefix + stop
 * 
 *         if prefix is not None:             # <<<<<<<<<<<<<<
 *             if start is not None or stop is not None:
 *                 raise TypeError(
 */
  __pyx_t_5 = (__pyx_v_prefix != ((PyObject*)Py_None));
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "plyvel/_plyvel.pyx":760
 * 
 *         if prefix is not None:
 *             if start is not None or stop is not None:             # <<<<<<<<<<<<<<
 *                 raise TypeError(
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
 */
    __pyx_t_5 = (__pyx_v_start != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_5 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_6 = __pyx_t_8;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_stop != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_8 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L9_bool_binop_done:;
    if (unlikely(__pyx_t_6)) {

      /* "plyvel/_plyvel.pyx":761
 *         if prefix is not None:
 *             if start is not None or stop is not None:
 *                 raise TypeError(             # <<<<<<<<<<<<<<
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
 *             # Use prefix to construct start and stop keys, and ignore
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 761, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 761, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":760
 * 
 *         if prefix is not None:
 *             if start is not None or stop is not None:             # <<<<<<<<<<<<<<
 *                 raise TypeError(
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
 */
    }

    /* "plyvel/_plyvel.pyx":765
 *             # Use prefix to construct start and stop keys, and ignore
 *             # include_start and include_stop args
 *             start = prefix             # <<<<<<<<<<<<<<
 *             stop = bytes_increment(prefix)
 *             include_start = True
 */
    __Pyx_INCREF(__pyx_v_prefix);
    __Pyx_DECREF_SET(__pyx_v_start, __pyx_v_prefix);

    /* "plyvel/_plyvel.pyx":766
 *             # include_start and include_stop args
 *             start = prefix
 *             stop = bytes_increment(prefix)             # <<<<<<<<<<<<<<
 *             include_start = True
 *             include_stop = False
 */
    __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_bytes_increment(__pyx_v_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "plyvel/_plyvel.pyx":767
 *             start = prefix
 *             stop = bytes_increment(prefix)
 *             include_start = True             # <<<<<<<<<<<<<<
 *             include_stop = False
 * 
 */
    __Pyx_INCREF(Py_True);
    __Pyx_DECREF_SET(__pyx_v_include_start, ((PyBoolObject *)Py_True));

    /* "plyvel/_plyvel.pyx":768
 *             stop = bytes_increment(prefix)
 *             include_start = True
 *             include_stop = False             # <<<<<<<<<<<<<<
 * 
 *         if start is not None:
 */
    __Pyx_INCREF(Py_False);
    __Pyx_DECREF_SET(__pyx_v_include_stop, ((PyBoolObject *)Py_False));

    /* "plyvel/_plyvel.pyx":759
 *                     stop = db_prefix + stop
 * 
 *         if prefix is not None:             # <<<<<<<<<<<<<<
 *             if start is not None or stop is not None:
 *                 raise TypeError(
 */
  }

  /* "plyvel/_plyvel.pyx":770
 *             include_stop = False
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))
 */
  __pyx_t_6 = (__pyx_v_start != ((PyObject*)Py_None));
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "plyvel/_plyvel.pyx":771
 * 
 *         if start is not None:
 *             self.start = start             # <<<<<<<<<<<<<<
 *             self.start_slice = Slice(start, len(start))
 * 
 */
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    __Pyx_GOTREF(__pyx_v_self->start);
    __Pyx_DECREF(__pyx_v_self->start);
    __pyx_v_self->start = __pyx_v_start;

    /* "plyvel/_plyvel.pyx":772
 *         if start is not None:
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))             # <<<<<<<<<<<<<<
 * 
 *         if stop is not None:
 */
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 772, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_v_start); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 772, __pyx_L1_error)
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 772, __pyx_L1_error)
    }
    __pyx_t_7 = PyBytes_GET_SIZE(__pyx_v_start); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 772, __pyx_L1_error)
    __pyx_v_self->start_slice = leveldb::Slice(__pyx_t_9, __pyx_t_7);

    /* "plyvel/_plyvel.pyx":770
 *             include_stop = False
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))
 */
  }

  /* "plyvel/_plyvel.pyx":774
 *             self.start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))
 */
  __pyx_t_5 = (__pyx_v_stop != ((PyObject*)Py_None));
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "plyvel/_plyvel.pyx":775
 * 
 *         if stop is not None:
 *             self.stop = stop             # <<<<<<<<<<<<<<
 *             self.stop_slice = Slice(stop, len(stop))
 * 
 */
    __Pyx_INCREF(__pyx_v_stop);
    __Pyx_GIVEREF(__pyx_v_stop);
    __Pyx_GOTREF(__pyx_v_self->stop);
    __Pyx_DECREF(__pyx_v_self->stop);
    __pyx_v_self->stop = __pyx_v_stop;

    /* "plyvel/_plyvel.pyx":776
 *         if stop is not None:
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))             # <<<<<<<<<<<<<<
 * 
 *         self.include_start = include_start
 */
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 776, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 776, __pyx_L1_error)
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 776, __pyx_L1_error)
    }
    __pyx_t_7 = PyBytes_GET_SIZE(__pyx_v_stop); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 776, __pyx_L1_error)
    __pyx_v_self->stop_slice = leveldb::Slice(__pyx_t_9, __pyx_t_7);

    /* "plyvel/_plyvel.pyx":774
 *             self.start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))
 */
  }

  /* "plyvel/_plyvel.pyx":778
 *             self.stop_slice = Slice(stop, len(stop))
 * 
 *         self.include_start = include_start             # <<<<<<<<<<<<<<
 *         self.include_stop = include_stop
 *         self.include_key = include_key
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_start)); if (unlikely((__pyx_t_10 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 778, __pyx_L1_error)
  __pyx_v_self->include_start = __pyx_t_10;

  /* "plyvel/_plyvel.pyx":779
 * 
 *         self.include_start = include_start
 *         self.include_stop = include_stop             # <<<<<<<<<<<<<<
 *         self.include_key = include_key
 *         self.include_value = include_value
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_stop)); if (unlikely((__pyx_t_10 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 779, __pyx_L1_error)
  __pyx_v_self->include_stop = __pyx_t_10;

  /* "plyvel/_plyvel.pyx":780
 *         self.include_start = include_start
 *         self.include_stop = include_stop
 *         self.include_key = include_key             # <<<<<<<<<<<<<<
 *         self.include_value = include_value
 * 
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_key)); if (unlikely((__pyx_t_10 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 780, __pyx_L1_error)
  __pyx_v_self->include_key = __pyx_t_10;

  /* "plyvel/_plyvel.pyx":781
 *         self.include_stop = include_stop
 *         self.include_key = include_key
 *         self.include_value = include_value             # <<<<<<<<<<<<<<
 * 
 *         if self.direction == FORWARD:
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_value)); if (unlikely((__pyx_t_10 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 781, __pyx_L1_error)
  __pyx_v_self->include_value = __pyx_t_10;

  /* "plyvel/_plyvel.pyx":783
 *         self.include_value = include_value
 * 
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             self.seek_to_start()
 *         else:
 */
  __pyx_t_6 = ((__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD) != 0);
  if (__pyx_t_6) {

    /* "plyvel/_plyvel.pyx":784
 * 
 *         if self.direction == FORWARD:
 *             self.seek_to_start()             # <<<<<<<<<<<<<<
 *         else:
 *             self.seek_to_stop()
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_seek_to_start); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 784, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 784, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 784, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "plyvel/_plyvel.pyx":783
 *         self.include_value = include_value
 * 
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             self.seek_to_start()
 *         else:
 */
    goto __pyx_L13;
  }

  /* "plyvel/_plyvel.pyx":786
 *             self.seek_to_start()
 *         else:
 *             self.seek_to_stop()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_seek_to_stop); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 786, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 786, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 786, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L13:;

  /* "plyvel/_plyvel.pyx":788
 *             self.seek_to_stop()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __pyx_t_11 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 788, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":718
 *     cdef size_t db_prefix_len
 * 
 *     def __init__(self, DB db, bytes db_prefix, bool reverse, bytes start,             # <<<<<<<<<<<<<<
 *                  bytes stop, bool include_start, bool include_stop,
 *                  bytes prefix, bool include_key, bool include_value,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_stop);
  __Pyx_XDECREF((PyObject *)__pyx_v_include_start);
  __Pyx_XDECREF((PyObject *)__pyx_v_include_stop);
  __Pyx_XDECREF(__pyx_v_prefix);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":790
 *         raise_for_status(self._iter.status())
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":791
 * 
 *     def __iter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     cdef object current(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":790
 *         raise_for_status(self._iter.status())
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":793
 *         return self
 * 
 *     cdef object current(self):             # <<<<<<<<<<<<<<
 *         """Return the current iterator key/value.
 * 
 */

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_v_key = 0;
  leveldb::Slice __pyx_v_value_slice;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("current", 0);

  /* "plyvel/_plyvel.pyx":800
 *         """
 *         cdef Slice key_slice
 *         cdef bytes key = None             # <<<<<<<<<<<<<<
 *         cdef Slice value_slice
 *         cdef bytes value = None
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_key = ((PyObject*)Py_None);

  /* "plyvel/_plyvel.pyx":802
 *         cdef bytes key = None
 *         cdef Slice value_slice
 *         cdef bytes value = None             # <<<<<<<<<<<<<<
 * 
 *         # Only build Python strings that will be returned. Also chop off
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_value = ((PyObject*)Py_None);

  /* "plyvel/_plyvel.pyx":806
 *         # Only build Python strings that will be returned. Also chop off
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 */
  __pyx_t_1 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":807
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:
 *             key_slice = self._iter.key()             # <<<<<<<<<<<<<<
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
 */
    __pyx_v_key_slice = __pyx_v_self->__pyx_base._iter->key();

    /* "plyvel/_plyvel.pyx":808
 *         if self.include_key:
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *         if self.include_value:
 */
    __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_key_slice.data() + __pyx_v_self->db_prefix_len, __pyx_v_key_slice.size() - __pyx_v_self->db_prefix_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":806
 *         # Only build Python strings that will be returned. Also chop off
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 */
  }

  /* "plyvel/_plyvel.pyx":810
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]
 */
  __pyx_t_1 = (__pyx_v_self->include_value != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":811
 * 
 *         if self.include_value:
 *             value_slice = self._iter.value()             # <<<<<<<<<<<<<<
 *             value = value_slice.data()[:value_slice.size()]
 * 
 */
    __pyx_v_value_slice = __pyx_v_self->__pyx_base._iter->value();

    /* "plyvel/_plyvel.pyx":812
 *         if self.include_value:
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *         if self.include_key and self.include_value:
 */
    __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_value_slice.data() + 0, __pyx_v_value_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_value, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":810
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]
 */
  }

  /* "plyvel/_plyvel.pyx":814
 *             value = value_slice.data()[:value_slice.size()]
 * 
 *         if self.include_key and self.include_value:             # <<<<<<<<<<<<<<
 *             return (key, value)
 *         if self.include_key:
 */
  __pyx_t_3 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->include_value != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":815
 * 
 *         if self.include_key and self.include_value:
 *             return (key, value)             # <<<<<<<<<<<<<<
 *         if self.include_key:
 *             return key
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 815, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_key);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_value);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":814
 *             value = value_slice.data()[:value_slice.size()]
 * 
 *         if self.include_key and self.include_value:             # <<<<<<<<<<<<<<
 *             return (key, value)
 *         if self.include_key:
 */
  }

  /* "plyvel/_plyvel.pyx":816
 *         if self.include_key and self.include_value:
 *             return (key, value)
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             return key
 *         if self.include_value:
 */
  __pyx_t_1 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":817
 *             return (key, value)
 *         if self.include_key:
 *             return key             # <<<<<<<<<<<<<<
 *         if self.include_value:
 *             return value
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_key);
    __pyx_r = __pyx_v_key;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":816
 *         if self.include_key and self.include_value:
 *             return (key, value)
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             return key
 *         if self.include_value:
 */
  }

  /* "plyvel/_plyvel.pyx":818
 *         if self.include_key:
 *             return key
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             return value
 *         return None
 */
  __pyx_t_1 = (__pyx_v_self->include_value != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":819
 *             return key
 *         if self.include_value:
 *             return value             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":818
 *         if self.include_key:
 *             return key
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             return value
 *         return None
 */
  }

  /* "plyvel/_plyvel.pyx":820
 *         if self.include_value:
 *             return value
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def __next__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":793
 *         return self
 * 
 *     cdef object current(self):             # <<<<<<<<<<<<<<
 *         """Return the current iterator key/value.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.current", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":822
 *         return None
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         """Return the next iterator entry.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__[] = "Return the next iterator entry.\n\n        Note: Cython will also create a .next() method that does the\n        same as this method.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__;
#endif
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "plyvel/_plyvel.pyx":828
 *         same as this method.
 *         """
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_next()
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD) != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":829
 *         """
 *         if self.direction == FORWARD:
 *             return self.real_next()             # <<<<<<<<<<<<<<
 *         else:
 *             return self.real_prev()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_next(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":828
 *         same as this method.
 *         """
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_next()
 *         else:
 */
  }

  /* "plyvel/_plyvel.pyx":831
 *             return self.real_next()
 *         else:
 *             return self.real_prev()             # <<<<<<<<<<<<<<
 * 
 *     def prev(self):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":822
 *         return None
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         """Return the next iterator entry.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":833
 *             return self.real_prev()
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self.direction == FORWARD:
 *             return self.real_prev()
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_6prev[] = "Iterator.prev(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prev (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("prev", 0);

  /* "plyvel/_plyvel.pyx":834
 * 
 *     def prev(self):
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_prev()
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD) != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":835
 *     def prev(self):
 *         if self.direction == FORWARD:
 *             return self.real_prev()             # <<<<<<<<<<<<<<
 *         else:
 *             return self.real_next()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":834
 * 
 *     def prev(self):
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_prev()
 *         else:
 */
  }

  /* "plyvel/_plyvel.pyx":837
 *             return self.real_prev()
 *         else:
 *             return self.real_next()             # <<<<<<<<<<<<<<
 * 
 *     cdef real_next(self):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_next(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":833
 *             return self.real_prev()
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self.direction == FORWARD:
 *             return self.real_prev()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":839
 *             return self.real_next()
 * 
 *     cdef real_next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  long __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  long __pyx_t_6;
  __Pyx_RefNannySetupContext("real_next", 0);

  /* "plyvel/_plyvel.pyx":840
 * 
 *     cdef real_next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":841
 *     cdef real_next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 841, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":840
 * 
 *     cdef real_next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":843
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Next()
 */
  switch (__pyx_v_self->state) {
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN:

    /* "plyvel/_plyvel.pyx":844
 * 
 *         if self.state == IN_BETWEEN:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Next()
 *             if not self._iter.Valid():
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":845
 *         if self.state == IN_BETWEEN:
 *             with nogil:
 *                 self._iter.Next()             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP
 */
          __pyx_v_self->__pyx_base._iter->Next();
        }

        /* "plyvel/_plyvel.pyx":844
 * 
 *         if self.state == IN_BETWEEN:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Next()
 *             if not self._iter.Valid():
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }

    /* "plyvel/_plyvel.pyx":846
 *             with nogil:
 *                 self._iter.Next()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 */
    __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":847
 *                 self._iter.Next()
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 */
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

      /* "plyvel/_plyvel.pyx":848
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 848, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":846
 *             with nogil:
 *                 self._iter.Next()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 */
    }

    /* "plyvel/_plyvel.pyx":843
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Next()
 */
    break;

    /* "plyvel/_plyvel.pyx":849
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 */
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED:

    /* "plyvel/_plyvel.pyx":850
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
 */
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

    /* "plyvel/_plyvel.pyx":849
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 */
    break;

    /* "plyvel/_plyvel.pyx":851
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             if self.start is None:
 *                 with nogil:
 */
    case __pyx_e_6plyvel_7_plyvel_BEFORE_START:

    /* "plyvel/_plyvel.pyx":852
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     self._iter.SeekToFirst()
 */
    __pyx_t_1 = (__pyx_v_self->start == ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_1 != 0);
    if (__pyx_t_3) {

      /* "plyvel/_plyvel.pyx":853
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToFirst()
 *             else:
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":854
 *             if self.start is None:
 *                 with nogil:
 *                     self._iter.SeekToFirst()             # <<<<<<<<<<<<<<
 *             else:
 *                 with nogil:
 */
            __pyx_v_self->__pyx_base._iter->SeekToFirst();
          }

          /* "plyvel/_plyvel.pyx":853
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToFirst()
 *             else:
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "plyvel/_plyvel.pyx":852
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     self._iter.SeekToFirst()
 */
      goto __pyx_L8;
    }

    /* "plyvel/_plyvel.pyx":856
 *                     self._iter.SeekToFirst()
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():
 */
    /*else*/ {
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":857
 *             else:
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 # Iterator is empty
 */
            __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_self->start_slice);
          }

          /* "plyvel/_plyvel.pyx":856
 *                     self._iter.SeekToFirst()
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L14;
            }
            __pyx_L14:;
          }
      }
    }
    __pyx_L8:;

    /* "plyvel/_plyvel.pyx":858
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # Iterator is empty
 *                 raise StopIteration
 */
    __pyx_t_3 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
    if (unlikely(__pyx_t_3)) {

      /* "plyvel/_plyvel.pyx":860
 *             if not self._iter.Valid():
 *                 # Iterator is empty
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *             if self.start is not None and not self.include_start:
 *                 # Start key is excluded, so skip past it if the db
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 860, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":858
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # Iterator is empty
 *                 raise StopIteration
 */
    }

    /* "plyvel/_plyvel.pyx":861
 *                 # Iterator is empty
 *                 raise StopIteration
 *             if self.start is not None and not self.include_start:             # <<<<<<<<<<<<<<
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 */
    __pyx_t_1 = (__pyx_v_self->start != ((PyObject*)Py_None));
    __pyx_t_4 = (__pyx_t_1 != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L17_bool_binop_done;
    }
    __pyx_t_4 = ((!(__pyx_v_self->include_start != 0)) != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L17_bool_binop_done:;
    if (__pyx_t_3) {

      /* "plyvel/_plyvel.pyx":865
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Next()
 */
      __pyx_t_3 = ((__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->start_slice) == 0) != 0);

      /* "plyvel/_plyvel.pyx":864
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),             # <<<<<<<<<<<<<<
 *                                            self.start_slice) == 0:
 *                     with nogil:
 */
      if (__pyx_t_3) {

        /* "plyvel/_plyvel.pyx":866
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
 */
        {
            #ifdef WITH_THREAD
            PyThreadState *_save;
            Py_UNBLOCK_THREADS
            __Pyx_FastGIL_Remember();
            #endif
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":867
 *                                            self.start_slice) == 0:
 *                     with nogil:
 *                         self._iter.Next()             # <<<<<<<<<<<<<<
 *                     if not self._iter.Valid():
 *                         raise StopIteration
 */
              __pyx_v_self->__pyx_base._iter->Next();
            }

            /* "plyvel/_plyvel.pyx":866
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
 */
            /*finally:*/ {
              /*normal exit:*/{
                #ifdef WITH_THREAD
                __Pyx_FastGIL_Forget();
                Py_BLOCK_THREADS
                #endif
                goto __pyx_L22;
              }
              __pyx_L22:;
            }
        }

        /* "plyvel/_plyvel.pyx":868
 *                     with nogil:
 *                         self._iter.Next()
 *                     if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
 */
        __pyx_t_3 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
        if (unlikely(__pyx_t_3)) {

          /* "plyvel/_plyvel.pyx":869
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
 *                         raise StopIteration             # <<<<<<<<<<<<<<
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:
 */
          __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
          __PYX_ERR(0, 869, __pyx_L1_error)

          /* "plyvel/_plyvel.pyx":868
 *                     with nogil:
 *                         self._iter.Next()
 *                     if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
 */
        }

        /* "plyvel/_plyvel.pyx":864
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),             # <<<<<<<<<<<<<<
 *                                            self.start_slice) == 0:
 *                     with nogil:
 */
      }

      /* "plyvel/_plyvel.pyx":861
 *                 # Iterator is empty
 *                 raise StopIteration
 *             if self.start is not None and not self.include_start:             # <<<<<<<<<<<<<<
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 */
    }

    /* "plyvel/_plyvel.pyx":870
 *                     if not self._iter.Valid():
 *                         raise StopIteration
 *             self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *         elif self.state == AFTER_STOP:
 *             raise StopIteration
 */
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

    /* "plyvel/_plyvel.pyx":851
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             if self.start is None:
 *                 with nogil:
 */
    break;

    /* "plyvel/_plyvel.pyx":871
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 * 
 */
    case __pyx_e_6plyvel_7_plyvel_AFTER_STOP:

    /* "plyvel/_plyvel.pyx":872
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:
 *             raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
    __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
    __PYX_ERR(0, 872, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":871
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 * 
 */
    break;
    default: break;
  }

  /* "plyvel/_plyvel.pyx":874
 *             raise StopIteration
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *         # Check range boundaries
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 874, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":877
 * 
 *         # Check range boundaries
 *         if self.stop is not None:             # <<<<<<<<<<<<<<
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 */
  __pyx_t_3 = (__pyx_v_self->stop != ((PyObject*)Py_None));
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "plyvel/_plyvel.pyx":878
 *         # Check range boundaries
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0             # <<<<<<<<<<<<<<
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP
 */
    if ((__pyx_v_self->include_stop != 0)) {
      __pyx_t_6 = 1;
    } else {
      __pyx_t_6 = 0;
    }
    __pyx_v_n = __pyx_t_6;

    /* "plyvel/_plyvel.pyx":879
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 */
    __pyx_t_4 = ((__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->stop_slice) >= __pyx_v_n) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "plyvel/_plyvel.pyx":880
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
 */
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

      /* "plyvel/_plyvel.pyx":881
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         return self.current()
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 881, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":879
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 */
    }

    /* "plyvel/_plyvel.pyx":877
 * 
 *         # Check range boundaries
 *         if self.stop is not None:             # <<<<<<<<<<<<<<
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 */
  }

  /* "plyvel/_plyvel.pyx":883
 *                 raise StopIteration
 * 
 *         return self.current()             # <<<<<<<<<<<<<<
 * 
 *     cdef real_prev(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_current(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":839
 *             return self.real_next()
 * 
 *     cdef real_next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.real_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":885
 *         return self.current()
 * 
 *     cdef real_prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_v_out = NULL;
  long __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  long __pyx_t_6;
  __Pyx_RefNannySetupContext("real_prev", 0);

  /* "plyvel/_plyvel.pyx":886
 * 
 *     cdef real_prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":887
 *     cdef real_prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 887, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 887, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":886
 * 
 *     cdef real_prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":889
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 */
  switch (__pyx_v_self->state) {
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN:
    break;

    /* "plyvel/_plyvel.pyx":891
 *         if self.state == IN_BETWEEN:
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             assert self._iter.Valid()
 *             with nogil:
 */
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED:

    /* "plyvel/_plyvel.pyx":892
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Prev()
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!(__pyx_v_self->__pyx_base._iter->Valid() != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 892, __pyx_L1_error)
      }
    }
    #endif

    /* "plyvel/_plyvel.pyx":893
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Prev()
 *             if not self._iter.Valid():
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":894
 *             assert self._iter.Valid()
 *             with nogil:
 *                 self._iter.Prev()             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 # The .seek() resulted in the first key in the database
 */
          __pyx_v_self->__pyx_base._iter->Prev();
        }

        /* "plyvel/_plyvel.pyx":893
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Prev()
 *             if not self._iter.Valid():
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }

    /* "plyvel/_plyvel.pyx":895
 *             with nogil:
 *                 self._iter.Prev()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
 */
    __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":897
 *             if not self._iter.Valid():
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())
 */
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

      /* "plyvel/_plyvel.pyx":898
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 898, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":895
 *             with nogil:
 *                 self._iter.Prev()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
 */
    }

    /* "plyvel/_plyvel.pyx":899
 *                 self.state = BEFORE_START
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 *         elif self.state == BEFORE_START:
 *             raise StopIteration
 */
    __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 899, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":891
 *         if self.state == IN_BETWEEN:
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             assert self._iter.Valid()
 *             with nogil:
 */
    break;

    /* "plyvel/_plyvel.pyx":900
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 */
    case __pyx_e_6plyvel_7_plyvel_BEFORE_START:

    /* "plyvel/_plyvel.pyx":901
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:
 *             raise StopIteration             # <<<<<<<<<<<<<<
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:
 */
    __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
    __PYX_ERR(0, 901, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":900
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 */
    break;

    /* "plyvel/_plyvel.pyx":902
 *         elif self.state == BEFORE_START:
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 */
    case __pyx_e_6plyvel_7_plyvel_AFTER_STOP:

    /* "plyvel/_plyvel.pyx":903
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:             # <<<<<<<<<<<<<<
 *                 # No stop key, seek to last entry
 *                 with nogil:
 */
    __pyx_t_1 = (__pyx_v_self->stop == ((PyObject*)Py_None));
    __pyx_t_4 = (__pyx_t_1 != 0);
    if (__pyx_t_4) {

      /* "plyvel/_plyvel.pyx":905
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToLast()
 *             else:
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":906
 *                 # No stop key, seek to last entry
 *                 with nogil:
 *                     self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 *             else:
 *                 # Seek to stop key
 */
            __pyx_v_self->__pyx_base._iter->SeekToLast();
          }

          /* "plyvel/_plyvel.pyx":905
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToLast()
 *             else:
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "plyvel/_plyvel.pyx":903
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:             # <<<<<<<<<<<<<<
 *                 # No stop key, seek to last entry
 *                 with nogil:
 */
      goto __pyx_L8;
    }

    /* "plyvel/_plyvel.pyx":909
 *             else:
 *                 # Seek to stop key
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.stop_slice)
 * 
 */
    /*else*/ {
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":910
 *                 # Seek to stop key
 *                 with nogil:
 *                     self._iter.Seek(self.stop_slice)             # <<<<<<<<<<<<<<
 * 
 *                 if self._iter.Valid():
 */
            __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_self->stop_slice);
          }

          /* "plyvel/_plyvel.pyx":909
 *             else:
 *                 # Seek to stop key
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.stop_slice)
 * 
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L14;
            }
            __pyx_L14:;
          }
      }

      /* "plyvel/_plyvel.pyx":912
 *                     self._iter.Seek(self.stop_slice)
 * 
 *                 if self._iter.Valid():             # <<<<<<<<<<<<<<
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 */
      __pyx_t_4 = (__pyx_v_self->__pyx_base._iter->Valid() != 0);
      if (__pyx_t_4) {

        /* "plyvel/_plyvel.pyx":914
 *                 if self._iter.Valid():
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:             # <<<<<<<<<<<<<<
 *                         with nogil:
 *                             self._iter.Prev()
 */
        __pyx_t_4 = ((!(__pyx_v_self->include_stop != 0)) != 0);
        if (__pyx_t_4) {

          /* "plyvel/_plyvel.pyx":915
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 *                         with nogil:             # <<<<<<<<<<<<<<
 *                             self._iter.Prev()
 *                 else:
 */
          {
              #ifdef WITH_THREAD
              PyThreadState *_save;
              Py_UNBLOCK_THREADS
              __Pyx_FastGIL_Remember();
              #endif
              /*try:*/ {

                /* "plyvel/_plyvel.pyx":916
 *                     if not self.include_stop:
 *                         with nogil:
 *                             self._iter.Prev()             # <<<<<<<<<<<<<<
 *                 else:
 *                     # Stop key did not exist; position at the last
 */
                __pyx_v_self->__pyx_base._iter->Prev();
              }

              /* "plyvel/_plyvel.pyx":915
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 *                         with nogil:             # <<<<<<<<<<<<<<
 *                             self._iter.Prev()
 *                 else:
 */
              /*finally:*/ {
                /*normal exit:*/{
                  #ifdef WITH_THREAD
                  __Pyx_FastGIL_Forget();
                  Py_BLOCK_THREADS
                  #endif
                  goto __pyx_L19;
                }
                __pyx_L19:;
              }
          }

          /* "plyvel/_plyvel.pyx":914
 *                 if self._iter.Valid():
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:             # <<<<<<<<<<<<<<
 *                         with nogil:
 *                             self._iter.Prev()
 */
        }

        /* "plyvel/_plyvel.pyx":912
 *                     self._iter.Seek(self.stop_slice)
 * 
 *                 if self._iter.Valid():             # <<<<<<<<<<<<<<
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 */
        goto __pyx_L15;
      }

      /* "plyvel/_plyvel.pyx":920
 *                     # Stop key did not exist; position at the last
 *                     # database entry instead.
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.SeekToLast()
 * 
 */
      /*else*/ {
        {
            #ifdef WITH_THREAD
            PyThreadState *_save;
            Py_UNBLOCK_THREADS
            __Pyx_FastGIL_Remember();
            #endif
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":921
 *                     # database entry instead.
 *                     with nogil:
 *                         self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 * 
 *                 # Make sure the iterator is not past the stop key
 */
              __pyx_v_self->__pyx_base._iter->SeekToLast();
            }

            /* "plyvel/_plyvel.pyx":920
 *                     # Stop key did not exist; position at the last
 *                     # database entry instead.
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.SeekToLast()
 * 
 */
            /*finally:*/ {
              /*normal exit:*/{
                #ifdef WITH_THREAD
                __Pyx_FastGIL_Forget();
                Py_BLOCK_THREADS
                #endif
                goto __pyx_L22;
              }
              __pyx_L22:;
            }
        }
      }
      __pyx_L15:;

      /* "plyvel/_plyvel.pyx":924
 * 
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Prev()
 */
      __pyx_t_1 = (__pyx_v_self->__pyx_base._iter->Valid() != 0);
      if (__pyx_t_1) {
      } else {
        __pyx_t_4 = __pyx_t_1;
        goto __pyx_L24_bool_binop_done;
      }
      __pyx_t_1 = ((__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->stop_slice) > 0) != 0);
      __pyx_t_4 = __pyx_t_1;
      __pyx_L24_bool_binop_done:;
      if (__pyx_t_4) {

        /* "plyvel/_plyvel.pyx":925
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Prev()
 * 
 */
        {
            #ifdef WITH_THREAD
            PyThreadState *_save;
            Py_UNBLOCK_THREADS
            __Pyx_FastGIL_Remember();
            #endif
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":926
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:
 *                         self._iter.Prev()             # <<<<<<<<<<<<<<
 * 
 *             if not self._iter.Valid():
 */
              __pyx_v_self->__pyx_base._iter->Prev();
            }

            /* "plyvel/_plyvel.pyx":925
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Prev()
 * 
 */
            /*finally:*/ {
              /*normal exit:*/{
                #ifdef WITH_THREAD
                __Pyx_FastGIL_Forget();
                Py_BLOCK_THREADS
                #endif
                goto __pyx_L28;
              }
              __pyx_L28:;
            }
        }

        /* "plyvel/_plyvel.pyx":924
 * 
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Prev()
 */
      }
    }
    __pyx_L8:;

    /* "plyvel/_plyvel.pyx":928
 *                         self._iter.Prev()
 * 
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # No entries left
 *                 raise StopIteration
 */
    __pyx_t_4 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "plyvel/_plyvel.pyx":930
 *             if not self._iter.Valid():
 *                 # No entries left
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *             # After all the stepping back, we might even have ended up
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 930, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":928
 *                         self._iter.Prev()
 * 
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # No entries left
 *                 raise StopIteration
 */
    }

    /* "plyvel/_plyvel.pyx":935
 *             # *before* the start key. In this case the iterator does not
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
 */
    __pyx_t_1 = (__pyx_v_self->start != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_1 != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_5 = ((__pyx_v_self->comparator->Compare(__pyx_v_self->start_slice, __pyx_v_self->__pyx_base._iter->key()) >= 0) != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L31_bool_binop_done:;
    if (unlikely(__pyx_t_4)) {

      /* "plyvel/_plyvel.pyx":936
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *             raise_for_status(self._iter.status())
 */
      __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
      __PYX_ERR(0, 936, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":935
 *             # *before* the start key. In this case the iterator does not
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
 */
    }

    /* "plyvel/_plyvel.pyx":938
 *                 raise StopIteration
 * 
 *             raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *         # Unlike .real_next(), first obtain the value, then move the
 */
    __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 938, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":902
 *         elif self.state == BEFORE_START:
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 */
    break;
    default: break;
  }

  /* "plyvel/_plyvel.pyx":944
 *         # repeatedly calling it.prev() and next(it) will work as
 *         # designed.
 *         out = self.current()             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._iter.Prev()
 */
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_current(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 944, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_out = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":945
 *         # designed.
 *         out = self.current()
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 *         if not self._iter.Valid():
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":946
 *         out = self.current()
 *         with nogil:
 *             self._iter.Prev()             # <<<<<<<<<<<<<<
 *         if not self._iter.Valid():
 *             # Moved before the first key in the database
 */
        __pyx_v_self->__pyx_base._iter->Prev();
      }

      /* "plyvel/_plyvel.pyx":945
 *         # designed.
 *         out = self.current()
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 *         if not self._iter.Valid():
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L35;
        }
        __pyx_L35:;
      }
  }

  /* "plyvel/_plyvel.pyx":947
 *         with nogil:
 *             self._iter.Prev()
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START
 */
  __pyx_t_4 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (__pyx_t_4) {

    /* "plyvel/_plyvel.pyx":949
 *         if not self._iter.Valid():
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START             # <<<<<<<<<<<<<<
 *         else:
 *             if self.start is None:
 */
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

    /* "plyvel/_plyvel.pyx":947
 *         with nogil:
 *             self._iter.Prev()
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START
 */
    goto __pyx_L36;
  }

  /* "plyvel/_plyvel.pyx":951
 *             self.state = BEFORE_START
 *         else:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->start == ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "plyvel/_plyvel.pyx":953
 *             if self.start is None:
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *             else:
 *                 # Check range boundaries
 */
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

      /* "plyvel/_plyvel.pyx":951
 *             self.state = BEFORE_START
 *         else:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN
 */
      goto __pyx_L37;
    }

    /* "plyvel/_plyvel.pyx":956
 *             else:
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1             # <<<<<<<<<<<<<<
 *                 if self.comparator.Compare(
 *                         self._iter.key(), self.start_slice) >= n:
 */
    /*else*/ {
      if ((__pyx_v_self->include_start != 0)) {
        __pyx_t_6 = 0;
      } else {
        __pyx_t_6 = 1;
      }
      __pyx_v_n = __pyx_t_6;

      /* "plyvel/_plyvel.pyx":958
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(
 *                         self._iter.key(), self.start_slice) >= n:             # <<<<<<<<<<<<<<
 *                     # Iterator is valid and within range boundaries
 *                     self.state = IN_BETWEEN
 */
      __pyx_t_5 = ((__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->start_slice) >= __pyx_v_n) != 0);

      /* "plyvel/_plyvel.pyx":957
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
 */
      if (__pyx_t_5) {

        /* "plyvel/_plyvel.pyx":960
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
 *                     self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *                 else:
 *                     # Iterator is valid, but has moved before the
 */
        __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

        /* "plyvel/_plyvel.pyx":957
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
 */
        goto __pyx_L38;
      }

      /* "plyvel/_plyvel.pyx":964
 *                     # Iterator is valid, but has moved before the
 *                     # 'start' key
 *                     self.state = BEFORE_START             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
      /*else*/ {
        __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;
      }
      __pyx_L38:;
    }
    __pyx_L37:;
  }
  __pyx_L36:;

  /* "plyvel/_plyvel.pyx":966
 *                     self.state = BEFORE_START
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 *         return out
 * 
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 966, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":967
 * 
 *         raise_for_status(self._iter.status())
 *         return out             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_start(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_out);
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":885
 *         return self.current()
 * 
 *     cdef real_prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.real_prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":969
 *         return out
 * 
 *     def seek_to_start(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_8seek_to_start[] = "Iterator.seek_to_start(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_start (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("seek_to_start", 0);

  /* "plyvel/_plyvel.pyx":970
 * 
 *     def seek_to_start(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":971
 *     def seek_to_start(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = BEFORE_START
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 971, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 971, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":970
 * 
 *     def seek_to_start(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":973
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.state = BEFORE_START             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_stop(self):
 */
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

  /* "plyvel/_plyvel.pyx":969
 *         return out
 * 
 *     def seek_to_start(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek_to_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":975
 *         self.state = BEFORE_START
 * 
 *     def seek_to_stop(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_10seek_to_stop[] = "Iterator.seek_to_stop(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_stop (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("seek_to_stop", 0);

  /* "plyvel/_plyvel.pyx":976
 * 
 *     def seek_to_stop(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":977
 *     def seek_to_stop(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = AFTER_STOP
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 977, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":976
 * 
 *     def seek_to_stop(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":979
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 * 
 *     def seek(self, bytes target not None):
 */
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

  /* "plyvel/_plyvel.pyx":975
 *         self.state = BEFORE_START
 * 
 *     def seek_to_stop(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek_to_stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":981
 *         self.state = AFTER_STOP
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek(PyObject *__pyx_v_self, PyObject *__pyx_v_target); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_12seek[] = "Iterator.seek(self, bytes target)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek(PyObject *__pyx_v_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), (&PyBytes_Type), 0, "target", 1))) __PYX_ERR(0, 981, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), ((PyObject*)__pyx_v_target));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, PyObject *__pyx_v_target) {
  leveldb::Slice __pyx_v_target_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  const char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  leveldb::Slice __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("seek", 0);
  __Pyx_INCREF(__pyx_v_target);

  /* "plyvel/_plyvel.pyx":982
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":983
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.db_prefix is not None:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 983, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 983, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":982
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":985
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.db_prefix is not None:             # <<<<<<<<<<<<<<
 *             target = self.db_prefix + target
 * 
 */
  __pyx_t_1 = (__pyx_v_self->db_prefix != ((PyObject*)Py_None));
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":986
 * 
 *         if self.db_prefix is not None:
 *             target = self.db_prefix + target             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 */
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->db_prefix, __pyx_v_target); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 986, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_target, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":985
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.db_prefix is not None:             # <<<<<<<<<<<<<<
 *             target = self.db_prefix + target
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":988
 *             target = self.db_prefix + target
 * 
 *         cdef Slice target_slice = Slice(target, len(target))             # <<<<<<<<<<<<<<
 * 
 *         # Seek only within the start/stop boundaries
 */
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_v_target); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 988, __pyx_L1_error)
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_v_target); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 988, __pyx_L1_error)
  __pyx_v_target_slice = leveldb::Slice(__pyx_t_4, __pyx_t_5);

  /* "plyvel/_plyvel.pyx":991
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 */
  __pyx_t_1 = (__pyx_v_self->start != ((PyObject*)Py_None));
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }

  /* "plyvel/_plyvel.pyx":992
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(
 *                 target_slice, self.start_slice) < 0:             # <<<<<<<<<<<<<<
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(
 */
  __pyx_t_6 = ((__pyx_v_self->comparator->Compare(__pyx_v_target_slice, __pyx_v_self->start_slice) < 0) != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L6_bool_binop_done:;

  /* "plyvel/_plyvel.pyx":991
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 */
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":993
 *         if self.start is not None and self.comparator.Compare(
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice             # <<<<<<<<<<<<<<
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:
 */
    __pyx_t_7 = __pyx_v_self->start_slice;
    __pyx_v_target_slice = __pyx_t_7;

    /* "plyvel/_plyvel.pyx":991
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 */
  }

  /* "plyvel/_plyvel.pyx":994
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
 */
  __pyx_t_6 = (__pyx_v_self->stop != ((PyObject*)Py_None));
  __pyx_t_1 = (__pyx_t_6 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L9_bool_binop_done;
  }

  /* "plyvel/_plyvel.pyx":995
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *             target_slice = self.stop_slice
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->comparator->Compare(__pyx_v_target_slice, __pyx_v_self->stop_slice) > 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L9_bool_binop_done:;

  /* "plyvel/_plyvel.pyx":994
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
 */
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":996
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_7 = __pyx_v_self->stop_slice;
    __pyx_v_target_slice = __pyx_t_7;

    /* "plyvel/_plyvel.pyx":994
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
 */
  }

  /* "plyvel/_plyvel.pyx":998
 *             target_slice = self.stop_slice
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":999
 * 
 *         with nogil:
 *             self._iter.Seek(target_slice)             # <<<<<<<<<<<<<<
 *         if not self._iter.Valid():
 *             # Moved past the end (or empty database)
 */
        __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_target_slice);
      }

      /* "plyvel/_plyvel.pyx":998
 *             target_slice = self.stop_slice
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L13;
        }
        __pyx_L13:;
      }
  }

  /* "plyvel/_plyvel.pyx":1000
 *         with nogil:
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
 */
  __pyx_t_3 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":1002
 *         if not self._iter.Valid():
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *             return
 * 
 */
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

    /* "plyvel/_plyvel.pyx":1003
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":1000
 *         with nogil:
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
 */
  }

  /* "plyvel/_plyvel.pyx":1005
 *             return
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED             # <<<<<<<<<<<<<<
 *         raise_for_status(self._iter.status())
 * 
 */
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED;

  /* "plyvel/_plyvel.pyx":1006
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_8 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 1006, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":981
 *         self.state = AFTER_STOP
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_target);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_14__reduce_cython__[] = "Iterator.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Iterator_16__setstate_cython__[] = "Iterator.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1011
 * @cython.final
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_valid[] = "RawIterator.valid(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("valid (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("valid", 0);

  /* "plyvel/_plyvel.pyx":1012
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1013
 *     def valid(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self._iter.Valid()
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1013, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1013, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1012
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1015
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         return self._iter.Valid()             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_first(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base._iter->Valid()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1015, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1011
 * @cython.final
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.valid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1017
 *         return self._iter.Valid()
 * 
 *     def seek_to_first(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_2seek_to_first[] = "RawIterator.seek_to_first(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_first (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("seek_to_first", 0);

  /* "plyvel/_plyvel.pyx":1018
 * 
 *     def seek_to_first(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1019
 *     def seek_to_first(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1019, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1019, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1018
 * 
 *     def seek_to_first(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1021
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToFirst()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1022
 * 
 *         with nogil:
 *             self._iter.SeekToFirst()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
        __pyx_v_self->__pyx_base._iter->SeekToFirst();
      }

      /* "plyvel/_plyvel.pyx":1021
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToFirst()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1024
 *             self._iter.SeekToFirst()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_last(self):
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1024, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1017
 *         return self._iter.Valid()
 * 
 *     def seek_to_first(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek_to_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1026
 *         raise_for_status(self._iter.status())
 * 
 *     def seek_to_last(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_4seek_to_last[] = "RawIterator.seek_to_last(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_last (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("seek_to_last", 0);

  /* "plyvel/_plyvel.pyx":1027
 * 
 *     def seek_to_last(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1028
 *     def seek_to_last(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1028, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1028, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1027
 * 
 *     def seek_to_last(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1030
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToLast()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1031
 * 
 *         with nogil:
 *             self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
        __pyx_v_self->__pyx_base._iter->SeekToLast();
      }

      /* "plyvel/_plyvel.pyx":1030
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToLast()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1033
 *             self._iter.SeekToLast()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def seek(self, bytes target not None):
 */
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1033, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1026
 *         raise_for_status(self._iter.status())
 * 
 *     def seek_to_last(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek_to_last", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1035
 *         raise_for_status(self._iter.status())
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek(PyObject *__pyx_v_self, PyObject *__pyx_v_target); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_6seek[] = "RawIterator.seek(self, bytes target)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek(PyObject *__pyx_v_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), (&PyBytes_Type), 0, "target", 1))) __PYX_ERR(0, 1035, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self), ((PyObject*)__pyx_v_target));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, PyObject *__pyx_v_target) {
  leveldb::Slice __pyx_v_target_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  const char *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("seek", 0);

  /* "plyvel/_plyvel.pyx":1036
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1037
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__43, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1037, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1037, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1036
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1039
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         cdef Slice target_slice = Slice(target, len(target))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._iter.Seek(target_slice)
 */
  __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_target); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 1039, __pyx_L1_error)
  __pyx_t_4 = PyBytes_GET_SIZE(__pyx_v_target); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1039, __pyx_L1_error)
  __pyx_v_target_slice = leveldb::Slice(__pyx_t_3, __pyx_t_4);

  /* "plyvel/_plyvel.pyx":1040
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1041
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:
 *             self._iter.Seek(target_slice)             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
        __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_target_slice);
      }

      /* "plyvel/_plyvel.pyx":1040
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1043
 *             self._iter.Seek(target_slice)
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def next(self):
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1043, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1035
 *         raise_for_status(self._iter.status())
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1045
 *         raise_for_status(self._iter.status())
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_8next[] = "RawIterator.next(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("next", 0);

  /* "plyvel/_plyvel.pyx":1046
 * 
 *     def next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1047
 *     def next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__44, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1047, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1046
 * 
 *     def next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1049
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1050
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IteratorInvalidError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1050, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1050, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1050, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1050, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1049
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1052
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Next()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1053
 * 
 *         with nogil:
 *             self._iter.Next()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
        __pyx_v_self->__pyx_base._iter->Next();
      }

      /* "plyvel/_plyvel.pyx":1052
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Next()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":1055
 *             self._iter.Next()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def prev(self):
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1055, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1045
 *         raise_for_status(self._iter.status())
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1057
 *         raise_for_status(self._iter.status())
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_10prev[] = "RawIterator.prev(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prev (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("prev", 0);

  /* "plyvel/_plyvel.pyx":1058
 * 
 *     def prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1059
 *     def prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1059, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1059, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1058
 * 
 *     def prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1061
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1062
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IteratorInvalidError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1062, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1061
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1064
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1065
 * 
 *         with nogil:
 *             self._iter.Prev()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
 */
        __pyx_v_self->__pyx_base._iter->Prev();
      }

      /* "plyvel/_plyvel.pyx":1064
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":1067
 *             self._iter.Prev()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     cpdef key(self):
 */
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1067, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1057
 *         raise_for_status(self._iter.status())
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1069
 *         raise_for_status(self._iter.status())
 * 
 *     cpdef key(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("key", 0);

  /* "plyvel/_plyvel.pyx":1070
 * 
 *     cpdef key(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1071
 *     cpdef key(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1071, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1071, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1070
 * 
 *     cpdef key(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1073
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1074
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IteratorInvalidError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1074, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1074, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1074, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1073
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1077
 * 
 *         cdef Slice key_slice
 *         key_slice = self._iter.key()             # <<<<<<<<<<<<<<
 *         return key_slice.data()[:key_slice.size()]
 * 
 */
  __pyx_v_key_slice = __pyx_v_self->__pyx_base._iter->key();

  /* "plyvel/_plyvel.pyx":1078
 *         cdef Slice key_slice
 *         key_slice = self._iter.key()
 *         return key_slice.data()[:key_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *     cpdef value(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_key_slice.data() + 0, __pyx_v_key_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1069
 *         raise_for_status(self._iter.status())
 * 
 *     cpdef key(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_12key[] = "RawIterator.key(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("key (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("key", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_key(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1080
 *         return key_slice.data()[:key_slice.size()]
 * 
 *     cpdef value(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  leveldb::Slice __pyx_v_value_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("value", 0);

  /* "plyvel/_plyvel.pyx":1081
 * 
 *     cpdef value(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base._iter == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1082
 *     cpdef value(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1082, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1082, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1081
 * 
 *     cpdef value(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1084
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->__pyx_base._iter->Valid() != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1085
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice value_slice
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IteratorInvalidError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1085, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1085, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1085, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1085, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1084
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1088
 * 
 *         cdef Slice value_slice
 *         value_slice = self._iter.value()             # <<<<<<<<<<<<<<
 *         return value_slice.data()[:value_slice.size()]
 * 
 */
  __pyx_v_value_slice = __pyx_v_self->__pyx_base._iter->value();

  /* "plyvel/_plyvel.pyx":1089
 *         cdef Slice value_slice
 *         value_slice = self._iter.value()
 *         return value_slice.data()[:value_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *     def item(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_value_slice.data() + 0, __pyx_v_value_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1089, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1080
 *         return key_slice.data()[:key_slice.size()]
 * 
 *     cpdef value(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_14value[] = "RawIterator.value(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("value", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_value(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1080, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1091
 *         return value_slice.data()[:value_slice.size()]
 * 
 *     def item(self):             # <<<<<<<<<<<<<<
 *         return self.key(), self.value()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_16item[] = "RawIterator.item(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("item (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("item", 0);

  /* "plyvel/_plyvel.pyx":1092
 * 
 *     def item(self):
 *         return self.key(), self.value()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_key(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1092, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_11RawIterator_value(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1092, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1092, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1091
 *         return value_slice.data()[:value_slice.size()]
 * 
 *     def item(self):             # <<<<<<<<<<<<<<
 *         return self.key(), self.value()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__[] = "RawIterator.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__48, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__[] = "RawIterator.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1105
 *     cdef bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):             # <<<<<<<<<<<<<<
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 */

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_db,&__pyx_n_s_prefix,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject*)Py_None);
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_db)) != 0)) kw_args--;
      else {
        __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_db); __PYX_ERR(0, 1105, __pyx_L3_error)
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 1;
        PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "__init__") < 0)) __PYX_ERR(0, 1105, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_db); __PYX_ERR(0, 1105, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1105, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 1105, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 1105, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":1106
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
 */
  __pyx_t_1 = ((__pyx_v_db->_db == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1107
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__50, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1107, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1106
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1109
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 *         with nogil:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_db));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_db));
  __Pyx_GOTREF(__pyx_v_self->db);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->db));
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":1110
 * 
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 */
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":1111
 *         self.db = db
 *         self.prefix = prefix
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1112
 *         self.prefix = prefix
 *         with nogil:
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
        __pyx_v_self->_snapshot = ((leveldb::Snapshot *)__pyx_v_db->_db->GetSnapshot());
      }

      /* "plyvel/_plyvel.pyx":1111
 *         self.db = db
 *         self.prefix = prefix
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1105
 *     cdef bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):             # <<<<<<<<<<<<<<
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1114
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":1115
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1114
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.Snapshot.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":1117
 *         self.close()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do
 */

static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("close", 0);

  /* "plyvel/_plyvel.pyx":1118
 * 
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             return  # nothing to do
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_self->_snapshot == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1119
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":1118
 * 
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             return  # nothing to do
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1121
 *             return  # nothing to do
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1122
 * 
 *         with nogil:
 *             self.db._db.ReleaseSnapshot(self._snapshot)             # <<<<<<<<<<<<<<
 *             self._snapshot = NULL
 * 
 */
        __pyx_v_self->db->_db->ReleaseSnapshot(__pyx_v_self->_snapshot);

        /* "plyvel/_plyvel.pyx":1123
 *         with nogil:
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL             # <<<<<<<<<<<<<<
 * 
 *     def release(self):
 */
        __pyx_v_self->_snapshot = NULL;
      }

      /* "plyvel/_plyvel.pyx":1121
 *             return  # nothing to do
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "plyvel/_plyvel.pyx":1117
 *         self.close()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_4close[] = "Snapshot.close(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1125
 *             self._snapshot = NULL
 * 
 *     def release(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_6release[] = "Snapshot.release(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("release (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("release", 0);

  /* "plyvel/_plyvel.pyx":1126
 * 
 *     def release(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1125
 *             self._snapshot = NULL
 * 
 *     def release(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.release", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1128
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_8__enter__[] = "Snapshot.__enter__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":1129
 * 
 *     def __enter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1128
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1131
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_10__exit__[] = "Snapshot.__exit__(self, exc_type, exc_val, exc_tb)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exc_type,&__pyx_n_s_exc_val,&__pyx_n_s_exc_tb,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 1); __PYX_ERR(0, 1131, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, 2); __PYX_ERR(0, 1131, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) __PYX_ERR(0, 1131, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1131, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":1132
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 *         return False  # propagate exceptions
 * 
 */
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1133
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()
 *         return False  # propagate exceptions             # <<<<<<<<<<<<<<
 * 
 *     def get(self, bytes key not None, default=None, *,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1131
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1135
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_12get[] = "Snapshot.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_default,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":1136
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
 */
    values[2] = (PyObject *)((PyBoolObject *)Py_False);
    values[3] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 2; index < 4 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get") < 0)) __PYX_ERR(0, 1135, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[2]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1135, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 1135, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1136, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1136, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1135
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  leveldb::ReadOptions __pyx_v_read_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  bool __pyx_t_4;
  leveldb::Snapshot *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("get", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":1137
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_self->_snapshot == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1138
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__51, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1138, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1137
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1141
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 *         read_options.snapshot = self._snapshot
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_4 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1141, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_4;

  /* "plyvel/_plyvel.pyx":1142
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 *         read_options.snapshot = self._snapshot
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_4 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1142, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_4;

  /* "plyvel/_plyvel.pyx":1143
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         read_options.snapshot = self._snapshot             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
 */
  __pyx_t_5 = __pyx_v_self->_snapshot;
  __pyx_v_read_options.snapshot = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":1145
 *         read_options.snapshot = self._snapshot
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":1146
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         return db_get(self.db, key, default, read_options)
 */
    __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "plyvel/_plyvel.pyx":1145
 *         read_options.snapshot = self._snapshot
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1148
 *             key = self.prefix + key
 * 
 *         return db_get(self.db, key, default, read_options)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_v_self->db);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_db_get(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_t_3), __pyx_v_key, __pyx_v_default, __pyx_v_read_options); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1135
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1150
 *         return db_get(self.db, key, default, read_options)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":1151
 * 
 *     def __iter__(self):
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_iterator); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1151, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1151, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1150
 *         return db_get(self.db, key, default, read_options)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1153
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_16iterator[] = "Snapshot.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_reverse,&__pyx_n_s_start,&__pyx_n_s_stop,&__pyx_n_s_include_start,&__pyx_n_s_include_stop,&__pyx_n_s_prefix,&__pyx_n_s_include_key,&__pyx_n_s_include_value,&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":1154
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 */
    values[3] = ((PyObject *)Py_True);
    values[4] = ((PyObject *)Py_False);
    values[5] = ((PyObject *)Py_None);

    /* "plyvel/_plyvel.pyx":1155
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */
    values[6] = ((PyObject *)Py_True);
    values[7] = ((PyObject *)Py_True);

    /* "plyvel/_plyvel.pyx":1156
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
 */
    values[8] = (PyObject *)((PyBoolObject *)Py_False);
    values[9] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 10)) {
        Py_ssize_t index;
        for (index = 0; index < 10 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "iterator") < 0)) __PYX_ERR(0, 1153, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyBoolObject *)values[8]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[9]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1153, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1156, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1156, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1153
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":1157
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_self->_snapshot == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1158
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return Iterator(
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__52, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1158, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1157
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1160
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,
 *             stop=stop, include_start=include_start, include_stop=include_stop,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":1161
 * 
 *         return Iterator(
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,             # <<<<<<<<<<<<<<
 *             stop=stop, include_start=include_start, include_stop=include_stop,
 *             prefix=prefix, include_key=include_key,
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(13); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_db, ((PyObject *)__pyx_v_self->db)) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_db_prefix, __pyx_v_self->prefix) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_reverse, __pyx_v_reverse) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_start, __pyx_v_start) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1162
 *         return Iterator(
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,
 *             stop=stop, include_start=include_start, include_stop=include_stop,             # <<<<<<<<<<<<<<
 *             prefix=prefix, include_key=include_key,
 *             include_value=include_value, verify_checksums=verify_checksums,
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_stop, __pyx_v_stop) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_include_start, __pyx_v_include_start) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_include_stop, __pyx_v_include_stop) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1163
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,
 *             stop=stop, include_start=include_start, include_stop=include_stop,
 *             prefix=prefix, include_key=include_key,             # <<<<<<<<<<<<<<
 *             include_value=include_value, verify_checksums=verify_checksums,
 *             fill_cache=fill_cache, snapshot=self)
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_prefix, __pyx_v_prefix) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_include_key, __pyx_v_include_key) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1164
 *             stop=stop, include_start=include_start, include_stop=include_stop,
 *             prefix=prefix, include_key=include_key,
 *             include_value=include_value, verify_checksums=verify_checksums,             # <<<<<<<<<<<<<<
 *             fill_cache=fill_cache, snapshot=self)
 * 
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_include_value, __pyx_v_include_value) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_verify_checksums, ((PyObject *)__pyx_v_verify_checksums)) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1165
 *             prefix=prefix, include_key=include_key,
 *             include_value=include_value, verify_checksums=verify_checksums,
 *             fill_cache=fill_cache, snapshot=self)             # <<<<<<<<<<<<<<
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_fill_cache, ((PyObject *)__pyx_v_fill_cache)) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_snapshot, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 1161, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1160
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,
 *             stop=stop, include_start=include_start, include_stop=include_stop,
 */
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_Iterator), __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1153
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1167
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_18raw_iterator[] = "Snapshot.raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyBoolObject *__pyx_v_verify_checksums = 0;
  PyBoolObject *__pyx_v_fill_cache = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("raw_iterator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_verify_checksums,&__pyx_n_s_fill_cache,0};
    PyObject* values[2] = {0,0};
    values[0] = (PyObject *)((PyBoolObject *)Py_False);

    /* "plyvel/_plyvel.pyx":1168
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
 */
    values[1] = (PyObject *)((PyBoolObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 0; index < 2 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "raw_iterator") < 0)) __PYX_ERR(0, 1167, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_verify_checksums = ((PyBoolObject *)values[0]);
    __pyx_v_fill_cache = ((PyBoolObject *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("raw_iterator", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1167, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1167, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1168, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1167
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyBoolObject *__pyx_v_verify_checksums, PyBoolObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("raw_iterator", 0);

  /* "plyvel/_plyvel.pyx":1169
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->db->_db == NULL) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_self->_snapshot == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1170
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return RawIterator(
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__53, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1170, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1169
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 */
  }

  /* "plyvel/_plyvel.pyx":1172
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             db=self.db,
 *             verify_checksums=verify_checksums,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":1173
 * 
 *         return RawIterator(
 *             db=self.db,             # <<<<<<<<<<<<<<
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_db, ((PyObject *)__pyx_v_self->db)) < 0) __PYX_ERR(0, 1173, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1174
 *         return RawIterator(
 *             db=self.db,
 *             verify_checksums=verify_checksums,             # <<<<<<<<<<<<<<
 *             fill_cache=fill_cache,
 *             snapshot=self)
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_verify_checksums, ((PyObject *)__pyx_v_verify_checksums)) < 0) __PYX_ERR(0, 1173, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1175
 *             db=self.db,
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,             # <<<<<<<<<<<<<<
 *             snapshot=self)
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_fill_cache, ((PyObject *)__pyx_v_fill_cache)) < 0) __PYX_ERR(0, 1173, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1176
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 *             snapshot=self)             # <<<<<<<<<<<<<<
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_snapshot, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 1173, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1172
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             db=self.db,
 *             verify_checksums=verify_checksums,
 */
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_RawIterator), __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1167
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__[] = "Snapshot.__reduce_cython__(self)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__54, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__[] = "Snapshot.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__55, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB[] = "__pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB = {"__pyx_unpickle_PrefixedDB", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB};
static PyObject *__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixedDB", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixedDB", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_PrefixedDB") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixedDB", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x11ea7f1:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x11ea7f1) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x11, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x11ea7f1:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6plyvel_7_plyvel_PrefixedDB), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x11ea7f1 = (db, prefix))" % __pyx_checksum)
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6plyvel_7_plyvel_DB))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->db);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->db));
  __pyx_v___pyx_result->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->prefix);
  __Pyx_DECREF(__pyx_v___pyx_result->prefix);
  __pyx_v___pyx_result->prefix = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 2) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":12
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":31
 * 
 * @cname("__pyx_convert_PyObject_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyObject_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

static CYTHON_INLINE PyObject *__pyx_convert_PyObject_string_to_py_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_PyObject_string_to_py_std__in_string", 0);

  /* "string.to_py":32
 * @cname("__pyx_convert_PyObject_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyObject_string_to_py_std__in_string(const string& s):
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyUnicode_FromStringAndSize(const char*, size_t)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_FromStringAndSize(__pyx_v_s.data(), __pyx_v_s.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "string.to_py":31
 * 
 * @cname("__pyx_convert_PyObject_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyObject_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyObject_string_to_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":37
 * 
 * @cname("__pyx_convert_PyUnicode_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

static CYTHON_INLINE PyObject *__pyx_convert_PyUnicode_string_to_py_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_PyUnicode_string_to_py_std__in_string", 0);

  /* "string.to_py":38
 * @cname("__pyx_convert_PyUnicode_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_std__in_string(const string& s):
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyStr_FromStringAndSize(const char*, size_t)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyUnicode_FromStringAndSize(__pyx_v_s.data(), __pyx_v_s.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "string.to_py":37
 * 
 * @cname("__pyx_convert_PyUnicode_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyUnicode_string_to_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":43
 * 
 * @cname("__pyx_convert_PyStr_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyStr_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyStr_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

static CYTHON_INLINE PyObject *__pyx_convert_PyStr_string_to_py_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_PyStr_string_to_py_std__in_string", 0);

  /* "string.to_py":44
 * @cname("__pyx_convert_PyStr_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyStr_string_to_py_std__in_string(const string& s):
 *     return __Pyx_PyStr_FromStringAndSize(s.data(), s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyBytes_FromStringAndSize(const char*, size_t)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyStr_FromStringAndSize(__pyx_v_s.data(), __pyx_v_s.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "string.to_py":43
 * 
 * @cname("__pyx_convert_PyStr_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyStr_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyStr_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyStr_string_to_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":49
 * 
 * @cname("__pyx_convert_PyBytes_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyBytes_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

static CYTHON_INLINE PyObject *__pyx_convert_PyBytes_string_to_py_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_PyBytes_string_to_py_std__in_string", 0);

  /* "string.to_py":50
 * @cname("__pyx_convert_PyBytes_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyBytes_string_to_py_std__in_string(const string& s):
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyByteArray_FromStringAndSize(const char*, size_t)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_s.data(), __pyx_v_s.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "string.to_py":49
 * 
 * @cname("__pyx_convert_PyBytes_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyBytes_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), s.size())
 * cdef extern from *:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyBytes_string_to_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":55
 * 
 * @cname("__pyx_convert_PyByteArray_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), s.size())
 * 
 */

static CYTHON_INLINE PyObject *__pyx_convert_PyByteArray_string_to_py_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_PyByteArray_string_to_py_std__in_string", 0);

  /* "string.to_py":56
 * @cname("__pyx_convert_PyByteArray_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_std__in_string(const string& s):
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), s.size())             # <<<<<<<<<<<<<<
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyByteArray_FromStringAndSize(__pyx_v_s.data(), __pyx_v_s.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "string.to_py":55
 * 
 * @cname("__pyx_convert_PyByteArray_string_to_py_std__in_string")
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_std__in_string(const string& s):             # <<<<<<<<<<<<<<
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), s.size())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyByteArray_string_to_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.from_py":13
 * 
 * @cname("__pyx_convert_string_from_py_std__in_string")
 * cdef string __pyx_convert_string_from_py_std__in_string(object o) except *:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t length
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 */

static std::string __pyx_convert_string_from_py_std__in_string(PyObject *__pyx_v_o) {
  Py_ssize_t __pyx_v_length;
  char const *__pyx_v_data;
  std::string __pyx_r;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  __Pyx_RefNannySetupContext("__pyx_convert_string_from_py_std__in_string", 0);

  /* "string.from_py":15
 * cdef string __pyx_convert_string_from_py_std__in_string(object o) except *:
 *     cdef Py_ssize_t length
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)             # <<<<<<<<<<<<<<
 *     return string(data, length)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_AsStringAndSize(__pyx_v_o, (&__pyx_v_length)); if (unlikely(__pyx_t_1 == ((char const *)NULL))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_v_data = __pyx_t_1;

  /* "string.from_py":16
 *     cdef Py_ssize_t length
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 *     return string(data, length)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = std::string(__pyx_v_data, __pyx_v_length);
  goto __pyx_L0;

  /* "string.from_py":13
 * 
 * @cname("__pyx_convert_string_from_py_std__in_string")
 * cdef string __pyx_convert_string_from_py_std__in_string(object o) except *:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t length
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("string.from_py.__pyx_convert_string_from_py_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_6plyvel_7_plyvel_DB __pyx_vtable_6plyvel_7_plyvel_DB;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_DB(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *p;
  PyObject *o;
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_DB;
  new((void*)&(p->options)) leveldb::Options();
  p->name = Py_None; Py_INCREF(Py_None);
  p->lock = Py_None; Py_INCREF(Py_None);
  p->iterators = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_DB(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  __Pyx_call_destructor(p->options);
  Py_CLEAR(p->name);
  Py_CLEAR(p->lock);
  Py_CLEAR(p->iterators);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_DB(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  if (p->lock) {
    e = (*v)(p->lock, a); if (e) return e;
  }
  if (p->iterators) {
    e = (*v)(p->iterators, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_DB(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->lock);
  p->lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->iterators);
  p->iterators = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_2DB_closed(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(o);
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_DB[] = {
  {"close", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_3close, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_2DB_2close},
  {"get", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_9get, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_8get},
  {"put", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_11put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_10put},
  {"delete", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_13delete, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_12delete},
  {"write_batch", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_14write_batch},
  {"iterator", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_19iterator, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_18iterator},
  {"raw_iterator", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_20raw_iterator},
  {"snapshot", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_2DB_22snapshot},
  {"get_property", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_25get_property, METH_O, __pyx_doc_6plyvel_7_plyvel_2DB_24get_property},
  {"compact_range", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_26compact_range},
  {"approximate_size", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_28approximate_size},
  {"approximate_sizes", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_30approximate_sizes},
  {"prefixed_db", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db, METH_O, __pyx_doc_6plyvel_7_plyvel_2DB_32prefixed_db},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_35__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_2DB_34__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_2DB_37__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_2DB_36__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6plyvel_7_plyvel_DB[] = {
  {(char *)"closed", __pyx_getprop_6plyvel_7_plyvel_2DB_closed, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_DB = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.DB", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_DB), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_DB, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_2DB_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "DB(name, bool create_if_missing=False, *, bool error_if_exists=False, paranoid_checks=None, write_buffer_size=None, max_open_files=None, lru_cache_size=None, block_size=None, block_restart_interval=None, max_file_size=None, compression='snappy', int bloom_filter_bits=0, comparator=None, bytes comparator_name=None)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_DB, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_DB, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_2DB_17__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_DB, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6plyvel_7_plyvel_DB, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_2DB_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_DB, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_PrefixedDB(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_PrefixedDB(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_PrefixedDB(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_db(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(o);
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_prefix(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(o);
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_PrefixedDB[] = {
  {"get", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_4get},
  {"put", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_6put},
  {"delete", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_8delete},
  {"write_batch", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_10write_batch},
  {"iterator", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_14iterator},
  {"snapshot", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_16snapshot},
  {"prefixed_db", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db, METH_O, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6plyvel_7_plyvel_PrefixedDB[] = {
  {(char *)"db", __pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_db, 0, (char *)0, 0},
  {(char *)"prefix", __pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_prefix, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_PrefixedDB = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.PrefixedDB", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "PrefixedDB(DB db, *, bytes prefix)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_PrefixedDB, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_PrefixedDB, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_PrefixedDB, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6plyvel_7_plyvel_PrefixedDB, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_PrefixedDB, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_WriteBatch(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p;
  PyObject *o;
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o);
  new((void*)&(p->write_options)) leveldb::WriteOptions();
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_WriteBatch(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  __Pyx_call_destructor(p->write_options);
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_WriteBatch(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_WriteBatch(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_WriteBatch[] = {
  {"put", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_4put},
  {"delete", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete, METH_O, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_6delete},
  {"clear", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_8clear},
  {"write", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_10write},
  {"__enter__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13__enter__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_12__enter__},
  {"__exit__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_14__exit__},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_16__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_18__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_WriteBatch = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.WriteBatch", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_WriteBatch, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "WriteBatch(DB db, bytes prefix, bool transaction, sync)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_WriteBatch, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_WriteBatch, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_WriteBatch, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_WriteBatch, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_vtable_6plyvel_7_plyvel_BaseIterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_BaseIterator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  Py_CLEAR(p->db);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_BaseIterator[] = {
  {"close", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_2close},
  {"__enter__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_6__enter__},
  {"__exit__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_8__exit__},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_BaseIterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.BaseIterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "BaseIterator(DB db, bool verify_checksums, bool fill_cache, Snapshot snapshot)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_BaseIterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_BaseIterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator __pyx_vtable_6plyvel_7_plyvel_Iterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Iterator(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_Iterator *p;
  PyObject *o = __pyx_tp_new_6plyvel_7_plyvel_BaseIterator(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator*)__pyx_vtabptr_6plyvel_7_plyvel_Iterator;
  new((void*)&(p->start_slice)) leveldb::Slice();
  new((void*)&(p->stop_slice)) leveldb::Slice();
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->stop = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->db_prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_Iterator(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_Iterator *p = (struct __pyx_obj_6plyvel_7_plyvel_Iterator *)o;
  PyObject_GC_UnTrack(o);
  __Pyx_call_destructor(p->start_slice);
  __Pyx_call_destructor(p->stop_slice);
  Py_CLEAR(p->start);
  Py_CLEAR(p->stop);
  Py_CLEAR(p->db_prefix);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator(o);
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_Iterator[] = {
  {"__next__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__, METH_NOARGS|METH_COEXIST, __pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__},
  {"prev", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Iterator_6prev},
  {"seek_to_start", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Iterator_8seek_to_start},
  {"seek_to_stop", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Iterator_10seek_to_stop},
  {"seek", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek, METH_O, __pyx_doc_6plyvel_7_plyvel_8Iterator_12seek},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Iterator_14__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_8Iterator_16__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_Iterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.Iterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Iterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_Iterator, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Iterator(DB db, bytes db_prefix, bool reverse, bytes start, bytes stop, bool include_start, bool include_stop, bytes prefix, bool include_key, bool include_value, bool verify_checksums, bool fill_cache, Snapshot snapshot)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__, /*tp_iter*/
  __pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_Iterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_Iterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator __pyx_vtable_6plyvel_7_plyvel_RawIterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_RawIterator(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_RawIterator *p;
  PyObject *o = __pyx_tp_new_6plyvel_7_plyvel_BaseIterator(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator*)__pyx_vtabptr_6plyvel_7_plyvel_RawIterator;
  return o;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_RawIterator[] = {
  {"valid", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_valid},
  {"seek_to_first", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_2seek_to_first},
  {"seek_to_last", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_4seek_to_last},
  {"seek", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek, METH_O, __pyx_doc_6plyvel_7_plyvel_11RawIterator_6seek},
  {"next", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_8next},
  {"prev", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_10prev},
  {"key", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_12key},
  {"value", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_14value},
  {"item", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_16item},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_RawIterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.RawIterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_RawIterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_RawIterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_RawIterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot __pyx_vtable_6plyvel_7_plyvel_Snapshot;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Snapshot(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p;
  PyObject *o;
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_Snapshot;
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_Snapshot(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_Snapshot(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_Snapshot(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_Snapshot[] = {
  {"close", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_4close},
  {"release", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_6release},
  {"__enter__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_8__enter__},
  {"__exit__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_10__exit__},
  {"get", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_12get},
  {"iterator", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_16iterator},
  {"raw_iterator", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_18raw_iterator},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__, METH_NOARGS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__, METH_O, __pyx_doc_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6plyvel_7_plyvel_Snapshot = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.Snapshot", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Snapshot), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_Snapshot, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Snapshot(DB db, *, bytes prefix=None)", /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_Snapshot, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_Snapshot, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_Snapshot, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_Snapshot, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[8];
static int __pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__ = 0;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__)))) {
    o = (PyObject*)__pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[--__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__];
    memset(o, 0, sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_iterator_id);
  Py_CLEAR(p->__pyx_v_ref_dict);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__)))) {
    __pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__++] = ((struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  if (p->__pyx_v_iterator_id) {
    e = (*v)(p->__pyx_v_iterator_id, a); if (e) return e;
  }
  if (p->__pyx_v_ref_dict) {
    e = (*v)(p->__pyx_v_ref_dict, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  tmp = ((PyObject*)p->__pyx_v_iterator_id);
  p->__pyx_v_iterator_id = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_ref_dict);
  p->__pyx_v_ref_dict = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.__pyx_scope_struct____init__", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__plyvel(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__plyvel},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_plyvel",
    __pyx_k_Plyvel_a_Python_LevelDB_interfa, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_Cannot_convert_unicode_name_to_a, __pyx_k_Cannot_convert_unicode_name_to_a, sizeof(__pyx_k_Cannot_convert_unicode_name_to_a), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_operate_on_closed_LevelDB, __pyx_k_Cannot_operate_on_closed_LevelDB, sizeof(__pyx_k_Cannot_operate_on_closed_LevelDB), 0, 0, 1, 0},
  {&__pyx_n_s_CorruptionError, __pyx_k_CorruptionError, sizeof(__pyx_k_CorruptionError), 0, 0, 1, 1},
  {&__pyx_kp_s_Database_is_closed, __pyx_k_Database_is_closed, sizeof(__pyx_k_Database_is_closed), 0, 0, 1, 0},
  {&__pyx_kp_s_Database_or_iterator_is_closed, __pyx_k_Database_or_iterator_is_closed, sizeof(__pyx_k_Database_or_iterator_is_closed), 0, 0, 1, 0},
  {&__pyx_kp_s_Database_or_snapshot_is_closed, __pyx_k_Database_or_snapshot_is_closed, sizeof(__pyx_k_Database_or_snapshot_is_closed), 0, 0, 1, 0},
  {&__pyx_n_s_Error, __pyx_k_Error, sizeof(__pyx_k_Error), 0, 0, 1, 1},
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x11, __pyx_k_Incompatible_checksums_s_vs_0x11, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x11), 0, 0, 1, 0},
  {&__pyx_n_s_IteratorInvalidError, __pyx_k_IteratorInvalidError, sizeof(__pyx_k_IteratorInvalidError), 0, 0, 1, 1},
  {&__pyx_n_s_Lock, __pyx_k_Lock, sizeof(__pyx_k_Lock), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Start_and_stop_of_range_must_be, __pyx_k_Start_and_stop_of_range_must_be, sizeof(__pyx_k_Start_and_stop_of_range_must_be), 0, 0, 1, 0},
  {&__pyx_n_s_StopIteration, __pyx_k_StopIteration, sizeof(__pyx_k_StopIteration), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_UTF_8, __pyx_k_UTF_8, sizeof(__pyx_k_UTF_8), 0, 0, 1, 0},
  {&__pyx_n_s_UnicodeEncodeError, __pyx_k_UnicodeEncodeError, sizeof(__pyx_k_UnicodeEncodeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 0, 1, 0},
  {&__pyx_n_s_approximate_sizes, __pyx_k_approximate_sizes, sizeof(__pyx_k_approximate_sizes), 0, 0, 1, 1},
  {&__pyx_n_s_ascii, __pyx_k_ascii, sizeof(__pyx_k_ascii), 0, 0, 1, 1},
  {&__pyx_n_s_block_restart_interval, __pyx_k_block_restart_interval, sizeof(__pyx_k_block_restart_interval), 0, 0, 1, 1},
  {&__pyx_n_s_block_size, __pyx_k_block_size, sizeof(__pyx_k_block_size), 0, 0, 1, 1},
  {&__pyx_n_s_bloom_filter_bits, __pyx_k_bloom_filter_bits, sizeof(__pyx_k_bloom_filter_bits), 0, 0, 1, 1},
  {&__pyx_n_s_clear, __pyx_k_clear, sizeof(__pyx_k_clear), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_closed, __pyx_k_closed, sizeof(__pyx_k_closed), 0, 0, 1, 1},
  {&__pyx_kp_s_closed_2, __pyx_k_closed_2, sizeof(__pyx_k_closed_2), 0, 0, 1, 0},
  {&__pyx_n_s_comparator, __pyx_k_comparator, sizeof(__pyx_k_comparator), 0, 0, 1, 1},
  {&__pyx_kp_s_comparator_and_comparator_name, __pyx_k_comparator_and_comparator_name, sizeof(__pyx_k_comparator_and_comparator_name), 0, 0, 1, 0},
  {&__pyx_n_s_comparator_name, __pyx_k_comparator_name, sizeof(__pyx_k_comparator_name), 0, 0, 1, 1},
  {&__pyx_n_s_compression, __pyx_k_compression, sizeof(__pyx_k_compression), 0, 0, 1, 1},
  {&__pyx_kp_s_compression_must_be_None_or_a_s, __pyx_k_compression_must_be_None_or_a_s, sizeof(__pyx_k_compression_must_be_None_or_a_s), 0, 0, 1, 0},
  {&__pyx_kp_s_compression_must_be_None_or_sna, __pyx_k_compression_must_be_None_or_sna, sizeof(__pyx_k_compression_must_be_None_or_sna), 0, 0, 1, 0},
  {&__pyx_n_s_create_if_missing, __pyx_k_create_if_missing, sizeof(__pyx_k_create_if_missing), 0, 0, 1, 1},
  {&__pyx_kp_s_custom_comparator_object_must_be, __pyx_k_custom_comparator_object_must_be, sizeof(__pyx_k_custom_comparator_object_must_be), 0, 0, 1, 0},
  {&__pyx_kp_s_d_d, __pyx_k_d_d, sizeof(__pyx_k_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_db, __pyx_k_db, sizeof(__pyx_k_db), 0, 0, 1, 1},
  {&__pyx_n_s_db_prefix, __pyx_k_db_prefix, sizeof(__pyx_k_db_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_default, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {&__pyx_n_s_delete, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {&__pyx_n_s_destroy_db, __pyx_k_destroy_db, sizeof(__pyx_k_destroy_db), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
  {&__pyx_n_s_error_if_exists, __pyx_k_error_if_exists, sizeof(__pyx_k_error_if_exists), 0, 0, 1, 1},
  {&__pyx_n_s_exc_tb, __pyx_k_exc_tb, sizeof(__pyx_k_exc_tb), 0, 0, 1, 1},
  {&__pyx_n_s_exc_type, __pyx_k_exc_type, sizeof(__pyx_k_exc_type), 0, 0, 1, 1},
  {&__pyx_n_s_exc_val, __pyx_k_exc_val, sizeof(__pyx_k_exc_val), 0, 0, 1, 1},
  {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
  {&__pyx_n_s_fill_cache, __pyx_k_fill_cache, sizeof(__pyx_k_fill_cache), 0, 0, 1, 1},
  {&__pyx_n_s_fsname, __pyx_k_fsname, sizeof(__pyx_k_fsname), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getfilesystemencoding, __pyx_k_getfilesystemencoding, sizeof(__pyx_k_getfilesystemencoding), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_hex, __pyx_k_hex, sizeof(__pyx_k_hex), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_include_key, __pyx_k_include_key, sizeof(__pyx_k_include_key), 0, 0, 1, 1},
  {&__pyx_n_s_include_start, __pyx_k_include_start, sizeof(__pyx_k_include_start), 0, 0, 1, 1},
  {&__pyx_n_s_include_stop, __pyx_k_include_stop, sizeof(__pyx_k_include_stop), 0, 0, 1, 1},
  {&__pyx_n_s_include_value, __pyx_k_include_value, sizeof(__pyx_k_include_value), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_init___locals_lambda, __pyx_k_init___locals_lambda, sizeof(__pyx_k_init___locals_lambda), 0, 0, 1, 1},
  {&__pyx_n_s_iterator, __pyx_k_iterator, sizeof(__pyx_k_iterator), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_leveldb_version, __pyx_k_leveldb_version, sizeof(__pyx_k_leveldb_version), 0, 0, 1, 1},
  {&__pyx_n_s_lru_cache_size, __pyx_k_lru_cache_size, sizeof(__pyx_k_lru_cache_size), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_file_size, __pyx_k_max_file_size, sizeof(__pyx_k_max_file_size), 0, 0, 1, 1},
  {&__pyx_n_s_max_open_files, __pyx_k_max_open_files, sizeof(__pyx_k_max_open_files), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_kp_s_name_arg_must_be_a_byte_string, __pyx_k_name_arg_must_be_a_byte_string, sizeof(__pyx_k_name_arg_must_be_a_byte_string), 0, 0, 1, 0},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_options, __pyx_k_options, sizeof(__pyx_k_options), 0, 0, 1, 1},
  {&__pyx_n_s_paranoid_checks, __pyx_k_paranoid_checks, sizeof(__pyx_k_paranoid_checks), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_kp_s_plyvel_DB_with_name_r_s_at_0x_s, __pyx_k_plyvel_DB_with_name_r_s_at_0x_s, sizeof(__pyx_k_plyvel_DB_with_name_r_s_at_0x_s), 0, 0, 1, 0},
  {&__pyx_kp_s_plyvel_PrefixedDB_with_prefix_r, __pyx_k_plyvel_PrefixedDB_with_prefix_r, sizeof(__pyx_k_plyvel_PrefixedDB_with_prefix_r), 0, 0, 1, 0},
  {&__pyx_n_s_plyvel__plyvel, __pyx_k_plyvel__plyvel, sizeof(__pyx_k_plyvel__plyvel), 0, 0, 1, 1},
  {&__pyx_kp_s_plyvel__plyvel_pyx, __pyx_k_plyvel__plyvel_pyx, sizeof(__pyx_k_plyvel__plyvel_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_popitem, __pyx_k_popitem, sizeof(__pyx_k_popitem), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_kp_s_prefix_cannot_be_used_together, __pyx_k_prefix_cannot_be_used_together, sizeof(__pyx_k_prefix_cannot_be_used_together), 0, 0, 1, 0},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_put, __pyx_k_put, sizeof(__pyx_k_put), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PrefixedDB, __pyx_k_pyx_unpickle_PrefixedDB, sizeof(__pyx_k_pyx_unpickle_PrefixedDB), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_ref, __pyx_k_ref, sizeof(__pyx_k_ref), 0, 0, 1, 1},
  {&__pyx_n_s_repair_db, __pyx_k_repair_db, sizeof(__pyx_k_repair_db), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_seek_to_start, __pyx_k_seek_to_start, sizeof(__pyx_k_seek_to_start), 0, 0, 1, 1},
  {&__pyx_n_s_seek_to_stop, __pyx_k_seek_to_stop, sizeof(__pyx_k_seek_to_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_self__db_self_options_cannot_be, __pyx_k_self__db_self_options_cannot_be, sizeof(__pyx_k_self__db_self_options_cannot_be), 0, 0, 1, 0},
  {&__pyx_kp_s_self__iter_cannot_be_converted_t, __pyx_k_self__iter_cannot_be_converted_t, sizeof(__pyx_k_self__iter_cannot_be_converted_t), 0, 0, 1, 0},
  {&__pyx_kp_s_self__iter_self_comparator_self, __pyx_k_self__iter_self_comparator_self, sizeof(__pyx_k_self__iter_self_comparator_self), 0, 0, 1, 0},
  {&__pyx_kp_s_self__snapshot_cannot_be_convert, __pyx_k_self__snapshot_cannot_be_convert, sizeof(__pyx_k_self__snapshot_cannot_be_convert), 0, 0, 1, 0},
  {&__pyx_kp_s_self__write_batch_self_write_opt, __pyx_k_self__write_batch_self_write_opt, sizeof(__pyx_k_self__write_batch_self_write_opt), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_snappy, __pyx_k_snappy, sizeof(__pyx_k_snappy), 0, 0, 1, 1},
  {&__pyx_n_u_snappy, __pyx_k_snappy, sizeof(__pyx_k_snappy), 0, 1, 0, 1},
  {&__pyx_n_s_snapshot, __pyx_k_snapshot, sizeof(__pyx_k_snapshot), 0, 0, 1, 1},
  {&__pyx_n_s_st, __pyx_k_st, sizeof(__pyx_k_st), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_sync, __pyx_k_sync, sizeof(__pyx_k_sync), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_threading, __pyx_k_threading, sizeof(__pyx_k_threading), 0, 0, 1, 1},
  {&__pyx_n_s_transaction, __pyx_k_transaction, sizeof(__pyx_k_transaction), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_verify_checksums, __pyx_k_verify_checksums, sizeof(__pyx_k_verify_checksums), 0, 0, 1, 1},
  {&__pyx_n_s_weakref, __pyx_k_weakref, sizeof(__pyx_k_weakref), 0, 0, 1, 1},
  {&__pyx_n_s_weakref_ref, __pyx_k_weakref_ref, sizeof(__pyx_k_weakref_ref), 0, 0, 1, 1},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {&__pyx_n_s_write_buffer_size, __pyx_k_write_buffer_size, sizeof(__pyx_k_write_buffer_size), 0, 0, 1, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 121, __pyx_L1_error)
  __pyx_builtin_UnicodeEncodeError = __Pyx_GetBuiltinName(__pyx_n_s_UnicodeEncodeError); if (!__pyx_builtin_UnicodeEncodeError) __PYX_ERR(0, 127, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_builtin_hex = __Pyx_GetBuiltinName(__pyx_n_s_hex); if (!__pyx_builtin_hex) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 304, __pyx_L1_error)
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 432, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 432, __pyx_L1_error)
  #endif
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 723, __pyx_L1_error)
  __pyx_builtin_StopIteration = __Pyx_GetBuiltinName(__pyx_n_s_StopIteration); if (!__pyx_builtin_StopIteration) __PYX_ERR(0, 848, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "plyvel/_plyvel.pyx":121
 * 
 *     if not isinstance(name, unicode):
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "'name' arg must be a byte string or a unicode string")
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_name_arg_must_be_a_byte_string); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "plyvel/_plyvel.pyx":192
 *     else:
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')             # <<<<<<<<<<<<<<
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_UTF_8); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "plyvel/_plyvel.pyx":194
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")             # <<<<<<<<<<<<<<
 *         if compression == u'snappy':
 *             options.compression = leveldb.kSnappyCompression
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_compression_must_be_None_or_a_s); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "plyvel/_plyvel.pyx":198
 *             options.compression = leveldb.kSnappyCompression
 *         else:
 *             raise ValueError("'compression' must be None or 'snappy'")             # <<<<<<<<<<<<<<
 * 
 *     if bloom_filter_bits > 0:
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_compression_must_be_None_or_sna); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "plyvel/_plyvel.pyx":205
 * 
 *     if (comparator is None) != (comparator_name is None):
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_comparator_and_comparator_name); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "plyvel/_plyvel.pyx":210
 *     if comparator is not None:
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")             # <<<<<<<<<<<<<<
 * 
 *         options.comparator = NewPlyvelCallbackComparator(
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_custom_comparator_object_must_be); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 */
  __pyx_tuple__7 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "plyvel/_plyvel.pyx":304
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "plyvel/_plyvel.pyx":314
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef WriteOptions write_options = WriteOptions()
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "plyvel/_plyvel.pyx":335
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "plyvel/_plyvel.pyx":348
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return WriteBatch(self, None, transaction, sync)
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "plyvel/_plyvel.pyx":354
 *     def __iter__(self):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.iterator()
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "plyvel/_plyvel.pyx":391
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice sl = Slice(name, len(name))
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "plyvel/_plyvel.pyx":404
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice start_slice
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "plyvel/_plyvel.pyx":420
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.approximate_sizes((start, stop))[0]
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "plyvel/_plyvel.pyx":426
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef int n_ranges = len(ranges)
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "plyvel/_plyvel.pyx":435
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 *                     raise TypeError(             # <<<<<<<<<<<<<<
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Start_and_stop_of_range_must_be); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_self__db_self_options_cannot_be); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "(tree fragment)":4
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._db,self.options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_self__db_self_options_cannot_be); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "plyvel/_plyvel.pyx":576
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "plyvel/_plyvel.pyx":594
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "plyvel/_plyvel.pyx":605
 *     def clear(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "plyvel/_plyvel.pyx":612
 *     def write(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "plyvel/_plyvel.pyx":621
 *     def __enter__(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "plyvel/_plyvel.pyx":627
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.transaction and exc_type is not None:
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_Database_is_closed); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_self__write_batch_self_write_opt); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":4
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write_batch,self.write_options cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_self__write_batch_self_write_opt); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "plyvel/_plyvel.pyx":665
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_self__iter_cannot_be_converted_t); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_s_self__iter_cannot_be_converted_t); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "plyvel/_plyvel.pyx":761
 *         if prefix is not None:
 *             if start is not None or stop is not None:
 *                 raise TypeError(             # <<<<<<<<<<<<<<
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
 *             # Use prefix to construct start and stop keys, and ignore
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_kp_s_prefix_cannot_be_used_together); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "plyvel/_plyvel.pyx":841
 *     cdef real_next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "plyvel/_plyvel.pyx":887
 *     cdef real_prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 887, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "plyvel/_plyvel.pyx":971
 *     def seek_to_start(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = BEFORE_START
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 971, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "plyvel/_plyvel.pyx":977
 *     def seek_to_stop(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = AFTER_STOP
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(0, 977, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "plyvel/_plyvel.pyx":983
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.db_prefix is not None:
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_s_self__iter_self_comparator_self); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_s_self__iter_self_comparator_self); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "plyvel/_plyvel.pyx":1013
 *     def valid(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self._iter.Valid()
 */
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(0, 1013, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "plyvel/_plyvel.pyx":1019
 *     def seek_to_first(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 1019, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "plyvel/_plyvel.pyx":1028
 *     def seek_to_last(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
  __pyx_tuple__42 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "plyvel/_plyvel.pyx":1037
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 */
  __pyx_tuple__43 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 1037, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "plyvel/_plyvel.pyx":1047
 *     def next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "plyvel/_plyvel.pyx":1059
 *     def prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "plyvel/_plyvel.pyx":1071
 *     cpdef key(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "plyvel/_plyvel.pyx":1082
 *     cpdef value(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
 */
  __pyx_tuple__47 = PyTuple_Pack(1, __pyx_kp_s_Database_or_iterator_is_closed); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 1082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_kp_s_self__iter_cannot_be_converted_t); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "(tree fragment)":4
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._iter cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_self__iter_cannot_be_converted_t); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "plyvel/_plyvel.pyx":1107
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
 */
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_kp_s_Cannot_operate_on_closed_LevelDB); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 1107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "plyvel/_plyvel.pyx":1138
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
 */
  __pyx_tuple__51 = PyTuple_Pack(1, __pyx_kp_s_Database_or_snapshot_is_closed); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(0, 1138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "plyvel/_plyvel.pyx":1158
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return Iterator(
 */
  __pyx_tuple__52 = PyTuple_Pack(1, __pyx_kp_s_Database_or_snapshot_is_closed); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "plyvel/_plyvel.pyx":1170
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return RawIterator(
 */
  __pyx_tuple__53 = PyTuple_Pack(1, __pyx_kp_s_Database_or_snapshot_is_closed); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(0, 1170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__54 = PyTuple_Pack(1, __pyx_kp_s_self__snapshot_cannot_be_convert); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "(tree fragment)":4
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._snapshot cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__55 = PyTuple_Pack(1, __pyx_kp_s_self__snapshot_cannot_be_convert); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "plyvel/_plyvel.pyx":515
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 */
  __pyx_tuple__56 = PyTuple_Pack(17, __pyx_n_s_name, __pyx_n_s_paranoid_checks, __pyx_n_s_write_buffer_size, __pyx_n_s_max_open_files, __pyx_n_s_lru_cache_size, __pyx_n_s_block_size, __pyx_n_s_block_restart_interval, __pyx_n_s_max_file_size, __pyx_n_s_compression, __pyx_n_s_bloom_filter_bits, __pyx_n_s_comparator, __pyx_n_s_comparator_name, __pyx_n_s_options, __pyx_n_s_st, __pyx_n_s_fsname, __pyx_n_s_create_if_missing, __pyx_n_s_error_if_exists); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);
  __pyx_codeobj__57 = (PyObject*)__Pyx_PyCode_New(1, 11, 17, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__56, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_plyvel__plyvel_pyx, __pyx_n_s_repair_db, 515, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__57)) __PYX_ERR(0, 515, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":537
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
 */
  __pyx_tuple__58 = PyTuple_Pack(4, __pyx_n_s_name, __pyx_n_s_options, __pyx_n_s_st, __pyx_n_s_fsname); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_codeobj__59 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__58, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_plyvel__plyvel_pyx, __pyx_n_s_destroy_db, 537, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__59)) __PYX_ERR(0, 537, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__60 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PrefixedDB, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_18786289 = PyInt_FromLong(18786289L); if (unlikely(!__pyx_int_18786289)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_modinit_global_init_code(void); /*proto*/
static int __Pyx_modinit_variable_export_code(void); /*proto*/
static int __Pyx_modinit_function_export_code(void); /*proto*/
static int __Pyx_modinit_type_init_code(void); /*proto*/
static int __Pyx_modinit_type_import_code(void); /*proto*/
static int __Pyx_modinit_variable_import_code(void); /*proto*/
static int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_6plyvel_7_plyvel_DB = &__pyx_vtable_6plyvel_7_plyvel_DB;
  __pyx_vtable_6plyvel_7_plyvel_DB.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_2DB_close;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_DB) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_DB.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_DB.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_DB.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_DB.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_6plyvel_7_plyvel_DB.tp_dict, __pyx_vtabptr_6plyvel_7_plyvel_DB) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "DB", (PyObject *)&__pyx_type_6plyvel_7_plyvel_DB) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_DB) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_DB = &__pyx_type_6plyvel_7_plyvel_DB;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_PrefixedDB) < 0) __PYX_ERR(0, 453, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_PrefixedDB.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_PrefixedDB.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_PrefixedDB.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_PrefixedDB.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "PrefixedDB", (PyObject *)&__pyx_type_6plyvel_7_plyvel_PrefixedDB) < 0) __PYX_ERR(0, 453, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_PrefixedDB) < 0) __PYX_ERR(0, 453, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_PrefixedDB = &__pyx_type_6plyvel_7_plyvel_PrefixedDB;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_WriteBatch) < 0) __PYX_ERR(0, 553, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_WriteBatch.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_WriteBatch.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_WriteBatch.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_WriteBatch.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (PyObject_SetAttrString(__pyx_m, "WriteBatch", (PyObject *)&__pyx_type_6plyvel_7_plyvel_WriteBatch) < 0) __PYX_ERR(0, 553, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_WriteBatch) < 0) __PYX_ERR(0, 553, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_WriteBatch = &__pyx_type_6plyvel_7_plyvel_WriteBatch;
  __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator = &__pyx_vtable_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_BaseIterator.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_12BaseIterator_close;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_BaseIterator) < 0) __PYX_ERR(0, 654, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_BaseIterator.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_BaseIterator.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_BaseIterator.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_BaseIterator.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6plyvel_7_plyvel_BaseIterator.tp_dict, __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator) < 0) __PYX_ERR(0, 654, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "BaseIterator", (PyObject *)&__pyx_type_6plyvel_7_plyvel_BaseIterator) < 0) __PYX_ERR(0, 654, __pyx_L1_error)
  if (__pyx_type_6plyvel_7_plyvel_BaseIterator.tp_weaklistoffset == 0) __pyx_type_6plyvel_7_plyvel_BaseIterator.tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator, __weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_BaseIterator) < 0) __PYX_ERR(0, 654, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_BaseIterator = &__pyx_type_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtabptr_6plyvel_7_plyvel_Iterator = &__pyx_vtable_6plyvel_7_plyvel_Iterator;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.__pyx_base = *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.current = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_current;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.real_next = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_real_next;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.real_prev = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev;
  __pyx_type_6plyvel_7_plyvel_Iterator.tp_base = __pyx_ptype_6plyvel_7_plyvel_BaseIterator;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_Iterator) < 0) __PYX_ERR(0, 703, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_Iterator.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_Iterator.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_Iterator.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_Iterator.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6plyvel_7_plyvel_Iterator, "__next__"); if (unlikely(!wrapper)) __PYX_ERR(0, 703, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__.doc = __pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_6plyvel_7_plyvel_Iterator.tp_dict, __pyx_vtabptr_6plyvel_7_plyvel_Iterator) < 0) __PYX_ERR(0, 703, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Iterator", (PyObject *)&__pyx_type_6plyvel_7_plyvel_Iterator) < 0) __PYX_ERR(0, 703, __pyx_L1_error)
  if (__pyx_type_6plyvel_7_plyvel_Iterator.tp_weaklistoffset == 0) __pyx_type_6plyvel_7_plyvel_Iterator.tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_Iterator, __pyx_base.__weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_Iterator) < 0) __PYX_ERR(0, 703, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_Iterator = &__pyx_type_6plyvel_7_plyvel_Iterator;
  __pyx_vtabptr_6plyvel_7_plyvel_RawIterator = &__pyx_vtable_6plyvel_7_plyvel_RawIterator;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.__pyx_base = *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.key = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_11RawIterator_key;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.value = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_11RawIterator_value;
  __pyx_type_6plyvel_7_plyvel_RawIterator.tp_base = __pyx_ptype_6plyvel_7_plyvel_BaseIterator;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_RawIterator) < 0) __PYX_ERR(0, 1010, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_RawIterator.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_RawIterator.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_RawIterator.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_RawIterator.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_6plyvel_7_plyvel_RawIterator.tp_dict, __pyx_vtabptr_6plyvel_7_plyvel_RawIterator) < 0) __PYX_ERR(0, 1010, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RawIterator", (PyObject *)&__pyx_type_6plyvel_7_plyvel_RawIterator) < 0) __PYX_ERR(0, 1010, __pyx_L1_error)
  if (__pyx_type_6plyvel_7_plyvel_RawIterator.tp_weaklistoffset == 0) __pyx_type_6plyvel_7_plyvel_RawIterator.tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_RawIterator, __pyx_base.__weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_RawIterator) < 0) __PYX_ERR(0, 1010, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_RawIterator = &__pyx_type_6plyvel_7_plyvel_RawIterator;
  __pyx_vtabptr_6plyvel_7_plyvel_Snapshot = &__pyx_vtable_6plyvel_7_plyvel_Snapshot;
  __pyx_vtable_6plyvel_7_plyvel_Snapshot.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_8Snapshot_close;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel_Snapshot) < 0) __PYX_ERR(0, 1100, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel_Snapshot.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel_Snapshot.tp_dictoffset && __pyx_type_6plyvel_7_plyvel_Snapshot.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel_Snapshot.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_6plyvel_7_plyvel_Snapshot.tp_dict, __pyx_vtabptr_6plyvel_7_plyvel_Snapshot) < 0) __PYX_ERR(0, 1100, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Snapshot", (PyObject *)&__pyx_type_6plyvel_7_plyvel_Snapshot) < 0) __PYX_ERR(0, 1100, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6plyvel_7_plyvel_Snapshot) < 0) __PYX_ERR(0, 1100, __pyx_L1_error)
  __pyx_ptype_6plyvel_7_plyvel_Snapshot = &__pyx_type_6plyvel_7_plyvel_Snapshot;
  if (PyType_Ready(&__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__) < 0) __PYX_ERR(0, 662, __pyx_L1_error)
  __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__.tp_dictoffset && __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__ = &__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(2, 9, __pyx_L1_error)
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), 0); if (unlikely(!__pyx_ptype_7cpython_4bool_bool)) __PYX_ERR(3, 8, __pyx_L1_error)
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), 0); if (unlikely(!__pyx_ptype_7cpython_7complex_complex)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (!(defined(__cplusplus)) || (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 4)))
    #define CYTHON_SMALL_CODE __attribute__((optimize("Os")))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_plyvel(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_plyvel(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__plyvel(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__plyvel(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec__plyvel(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__plyvel(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_plyvel", __pyx_methods, __pyx_k_Plyvel_a_Python_LevelDB_interfa, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_plyvel___plyvel) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "plyvel._plyvel")) {
      if (unlikely(PyDict_SetItemString(modules, "plyvel._plyvel", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "plyvel/_plyvel.pyx":19
 * """
 * 
 * import sys             # <<<<<<<<<<<<<<
 * import threading
 * from weakref import ref as weakref_ref
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":20
 * 
 * import sys
 * import threading             # <<<<<<<<<<<<<<
 * from weakref import ref as weakref_ref
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_1) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":21
 * import sys
 * import threading
 * from weakref import ref as weakref_ref             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ref);
  __Pyx_GIVEREF(__pyx_n_s_ref);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ref);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_weakref, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ref); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_weakref_ref, __pyx_t_1) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":59
 * 
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,             # <<<<<<<<<<<<<<
 *                                  leveldb.kMinorVersion)
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(leveldb::kMajorVersion); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":60
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,
 *                                  leveldb.kMinorVersion)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(leveldb::kMinorVersion); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "plyvel/_plyvel.pyx":59
 * 
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,             # <<<<<<<<<<<<<<
 *                                  leveldb.kMinorVersion)
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_d_d, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_leveldb_version, __pyx_t_1) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":67
 * #
 * 
 * class Error(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_3 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_3, __pyx_t_1, __pyx_n_s_Error, __pyx_n_s_Error, (PyObject *) NULL, __pyx_n_s_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_3, __pyx_n_s_Error, __pyx_t_1, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Error, __pyx_t_4) < 0) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":71
 * 
 * 
 * class IOError(Error, IOError):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_Error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IOError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_3, __pyx_t_2, __pyx_n_s_IOError, __pyx_n_s_IOError, (PyObject *) NULL, __pyx_n_s_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_3, __pyx_n_s_IOError, __pyx_t_2, __pyx_t_1, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IOError, __pyx_t_4) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":75
 * 
 * 
 * class CorruptionError(Error):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_3, __pyx_n_s_CorruptionError, __pyx_n_s_CorruptionError, (PyObject *) NULL, __pyx_n_s_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_CorruptionError, __pyx_t_3, __pyx_t_1, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CorruptionError, __pyx_t_4) < 0) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "plyvel/_plyvel.pyx":79
 * 
 * 
 * class IteratorInvalidError(Error):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_Error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_3, __pyx_t_2, __pyx_n_s_IteratorInvalidError, __pyx_n_s_IteratorInvalidError, (PyObject *) NULL, __pyx_n_s_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_3, __pyx_n_s_IteratorInvalidError, __pyx_t_2, __pyx_t_1, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IteratorInvalidError, __pyx_t_4) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":515
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6plyvel_7_plyvel_1repair_db, NULL, __pyx_n_s_plyvel__plyvel); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_repair_db, __pyx_t_2) < 0) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":537
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6plyvel_7_plyvel_3destroy_db, NULL, __pyx_n_s_plyvel__plyvel); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_destroy_db, __pyx_t_2) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x11ea7f1:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB, NULL, __pyx_n_s_plyvel__plyvel); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PrefixedDB, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * 
 * #
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "string.from_py":13
 * 
 * @cname("__pyx_convert_string_from_py_std__in_string")
 * cdef string __pyx_convert_string_from_py_std__in_string(object o) except *:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t length
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init plyvel._plyvel", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init plyvel._plyvel");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    if (likely(result)) {
        Py_INCREF(result);
    } else if (unlikely(PyErr_Occurred())) {
        result = NULL;
    } else {
#else
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyCFunctionFastCall */
    #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
    #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
    #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectCallNoArg */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* SaveResetException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A3
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* GetItemIntByteArray */
        static CYTHON_INLINE int __Pyx_GetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i,
                                                         int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (wraparound | boundscheck) {
        length = PyByteArray_GET_SIZE(string);
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely((0 <= i) & (i < length))) {
            return (unsigned char) (PyByteArray_AS_STRING(string)[i]);
        } else {
            PyErr_SetString(PyExc_IndexError, "bytearray index out of range");
            return -1;
        }
    } else {
        return (unsigned char) (PyByteArray_AS_STRING(string)[i]);
    }
}

/* SetItemIntByteArray */
        static CYTHON_INLINE int __Pyx_SetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i, unsigned char v,
                                                         int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (wraparound | boundscheck) {
        length = PyByteArray_GET_SIZE(string);
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely((0 <= i) & (i < length))) {
            PyByteArray_AS_STRING(string)[i] = (char) v;
            return 0;
        } else {
            PyErr_SetString(PyExc_IndexError, "bytearray index out of range");
            return -1;
        }
    } else {
        PyByteArray_AS_STRING(string)[i] = (char) v;
        return 0;
    }
}

/* BytesEquals */
        static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
        static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseDoubleKeywords */
        static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
        static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
        static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
        static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* GetItemInt */
        static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ExtTypeTest */
        static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* WriteUnraisableException */
        static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* KeywordStringCheck */
        static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* RaiseTooManyValuesToUnpack */
        static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
        static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
        static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
        static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* SwapException */
        #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = *type;
    tstate->exc_state.exc_value = *value;
    tstate->exc_state.exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* RaiseKeywordRequired */
        static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() needs keyword-only argument %U", func_name, kw_name);
        #else
        "%s() needs keyword-only argument %s", func_name,
        PyString_AS_STRING(kw_name));
        #endif
}

/* GetAttr */
        static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
        static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* UnpackUnboundCMethod */
        static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod2 */
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod1 */
        #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
        static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* None */
        static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* FetchCommonType */
        static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
        #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Import */
            static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
            static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
            static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
            #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* SetVTable */
            static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
            static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* PyObject_GenericGetAttr */
            #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* CalculateMetaclass */
            static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
            static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
            #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
            static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
            #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
            static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
            #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
            static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
            static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value) {
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
            static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
            static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
            static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* FastTypeChecks */
            #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
            static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
            #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
            #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
            static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
